/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:51 $
 * $Revision: 1.4 $
 * $State: Exp $
 * $RCSfile: AbstractIndividual.java,v $
 *
 */
package org.evolvica.core;

/** Abstract base class for individuals.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.4 $ $Date: 2004/05/26 11:57:51 $
 */
abstract public class AbstractIndividual implements IIndividual {
	
	/**
	 * The phenotype builder that is used to create phenotypes for given genotypes.
	 */
	protected IPhenotypeBuilder phenoBuilder = null;
	
	/**
	 * The genotype of this individual.
	 */
	protected IGenotype genotype = null;
	
	/**
	 * The phenotype of this individual.
	 * A phenotype MAY be assigned but in general it is optional.
	 */
	protected IPhenotype phenotype = null;
	
	/**
	 * The associated score of this individual.
	 */
	protected IScore score = null;
	
	/**
	 * The fitness of this individual.
	 * The fitness value is optional. Target objective functions should only calculate
	 * score values. The fitness value should only be used for mapping scores to non-negative
	 * range of values or assigned an additional rank or something similiar.
	 */
	protected IFitness fitness = null;
	
	/**
	 * Returns the assigned genotype.
	 * This methods may never return null.
	 * 
	 * @return the genotype of this individual
	 */
	public IGenotype getGenotype() {
		return genotype;
	}

	/**
	 * Sets the genotype of this individual.
	 * 
	 * @param genotype the genotype to set
	 */
	public void setGenotype( IGenotype genotype ) {
		this.genotype = genotype;
		if ( phenoBuilder != null ) {
			this.phenotype = phenoBuilder.newPhenotype( this.genotype );
		}
	}

	/**
	 * Returns the associated phenotype.
	 * 
	 * @return phenotype or null if no phenotype has been assigned
	 */
	public IPhenotype getPhenotype() {
		return phenotype;
	}

	/**
	 * Returns the assigned score.
	 * 
	 * @return score or null if no score has been assigned yet
	 */
	public IScore getScore() {
		return score;
	}

	/**
	 * Sets the score.
	 * 
	 * @param score the score to set
	 */
	public void setScore( IScore score ) {
		this.score = score;
	}

	/**
	 * Returns the assigned fitness.
	 * 
	 * @return fitness or null if no fitness has been assigned yet
	 */
	public IFitness getFitness() {
		return fitness;
	}

	/**
	 * Sets the fitness.
	 * 
	 * @param fitness the fitness to set
	 */
	public void setFitness( IFitness fitness ) {
		this.fitness = fitness;
	}

	/**
	 * Returns a string representation of this individual.
	 * 
	 * @return string representation
	 */
	public String toString() {
		return "<S " + score + "> " + genotype; 
	}
	
}
