/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:53 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: GenotypeRepository.java,v $
 *
 */
package org.evolvica.core;

import java.util.HashMap;

import org.evolvica.engine.IDatatype;
import org.evolvica.util.BitVector;

/** Repository for collecting and creating genotype IDs.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:57:53 $
 */
public class GenotypeRepository {
	
	private static final HashMap map = new HashMap();
	
	static {
		getID( IDatatype.UniversalDatatype.class );
		getID( Double.TYPE );
		getID( Float.TYPE );
		getID( Integer.TYPE );
		getID( Long.TYPE );
		getID( double[].class );
		getID( float[].class );
		getID( int[].class );
		getID( long[].class );
		getID( String.class );
		getID( BitVector.class );
	}
	
	private GenotypeRepository() {
		// prevent instantiation
	}
	
	/**
	 * Returns a genotype ID for a given genotype.
	 * This methods returns an ID if the genotype is already known or
	 * creates a new ID if the genotype is unknown.
	 * 
	 * @param gene genotype for which an ID should be created
	 * @return ID for the given genotype 
	 */
	public static final IDatatype getID( IGenotype gene ) {
		IDatatype type = gene.getType();
		if ( map.containsKey( type.getReferencedClass() ) ) {
			return (IDatatype) map.get( type.getReferencedClass() );
		} else {
			map.put( type.getReferencedClass(), type );
			return type;
		}
	}
	
	/**
	 * Returns a genotype ID for a given class.
	 * This is a helper method for creating genotype IDs.
	 * 
	 * @param clazz class for which an ID should be created
	 * @return ID for the given genotype class or null if the class is not an instance of IGenotype
	 */
	public static final IDatatype getID( Class clazz ) {
		if ( map.containsKey( clazz ) ) {
			return (IDatatype) map.get( clazz );
		} else {
			IDatatype id = null;
			if ( clazz.getName().equals( "java.lang.Object" ) ) {
				id = IDatatype.UNIVERSAL;
			} else {
				id = new GenotypeID( clazz );
			}
			map.put( clazz, id );
			return id;
		}
	}
	
}
