/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:54 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: IIndividualSet.java,v $
 *
 */
package org.evolvica.core;

import java.io.Serializable;

/**
 * Interface for a set of individuals. Implementing classes are used to hold several individuals
 * together in a collection. In principle this interface is much the same as specified in
 * java.util.Collection, however it has been modified to avoid adding other objects than
 * individuals. Only the most important methods from java.util.Collection have been incorporated,
 * but other methods may be added in the future (i.e. retainAll()).
 * Note: The name set does not specify the underlying data structure, it has only been used because
 * the word set is shorter than the word collection.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:57:54 $
 *
 * @since 0.4.0
 */
public interface IIndividualSet extends Serializable {
	/**
	 * Adds an individual to this set.
	 *
	 * @param ind individual to be added
	 */
	public void add( IIndividual ind );

	/**
	 * Adds the individuals from the array to this set.
	 *
	 * @param indArray array with individuals to be added
	 */
	public void add( IIndividual[] indArray );

	/**
	 * Removes all individuals from this set.
	 */
	public void clear(  );

	/**
	 * Checks if the given individual is already contained in this set.
	 *
	 * @param ind individual that should be checked
	 *
	 * @return true if the given individual is contained in this set, false otherwise
	 */
	public boolean contains( IIndividual ind );

	/**
	 * Checks if this set is empty.
	 *
	 * @return true if this set does not contain any individuals, false otherwise
	 */
	public boolean isEmpty();

	/**
	 * Returns an iterator over this set.
	 *
	 * @return iterator for this set
	 */
	public ISetIterator iterator();

	/**
	 * Removes the given individual from this set, if it contained in the set.
	 *
	 * @param ind individual to be removed from this set
	 */
	public void remove( IIndividual ind );

	/**
	 * Removes all individuals in the given array from this set.
	 *
	 * @param indArray array with individuals to be removed
	 */
	public void remove( IIndividual[] indArray );

	/**
	 * Returns the number of individuals contained in this set.
	 *
	 * @return size of the set
	 */
	public int size();

	/**
	 * Returns the contents of this set as an array.
	 *
	 * @return array containing the individuals of this set
	 */
	public IIndividual[] toArray();

}
