
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:57 $
 * $Revision: 1.9 $
 * $State: Exp $
 * $RCSfile: ASCIIFileWriter.java,v $
 *
 */

package org.evolvica.core.common;

import java.io.FileOutputStream;
import java.io.PrintWriter;

import org.apache.commons.lang.SystemUtils;
import org.evolvica.core.IIndividual;
import org.evolvica.core.IIndividualSet;
import org.evolvica.core.ISetIterator;
import org.evolvica.util.EVLog;
import org.evolvica.util.UniqueID;

/** Processor for saving individual sets to plain ASCII files.
 * This processor performs a toString() on each individual in the collection.
 * Everytime the write() method is invoked the collection is written to a different file.
 * A set written to an ASCII file cannot be restored from that file.
 *
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.9 $ $Date: 2004/05/26 11:57:57 $
 */
public class ASCIIFileWriter extends FileWriter {
	
	/** Indicator if printing of genetic representation should be surpressed.
	* The default value is false.
	* 
	*/
	protected boolean genotypeSurpressed = false;
	
	/**
	 * Constructor.
	 */
	public ASCIIFileWriter() {
		this( "iset", "txt" );
	}
	
	/** Constructor with filename body and suffix.
	* 
	* @param body body of filename to write to
	* @param suffix suffix to be used
	*/
	public ASCIIFileWriter( String body, String suffix ) {
		super( body, suffix );
	}
	
	/** Saves the individual collection to a file.
	* The collection is saved to a file named &lt;filename body&gt;_&lt;identifier&gt;.&lt;suffix&gt;.
	* The identifier is generated automatically to prevent overwriting old files. The default
	* suffix is .txt.
	* 
	* @param set individual set to be saved
	*/
	public void write( IIndividualSet set ) {
		String file = directory + SystemUtils.FILE_SEPARATOR + body + "_" + UniqueID.nextLong() + "." + suffix;
		try {
			PrintWriter writer = new PrintWriter( new FileOutputStream( file ) );
			int index = 0;
			for( ISetIterator it = set.iterator(); it.hasNext(); ) {
				IIndividual ind = it.next();
				writer.print( "I" + index++ );
				writer.print( " score " + ind.getScore() + ", fitness " + ind.getFitness() );
				if ( genotypeSurpressed ) {
					writer.println( "" );
				} else {
					writer.println( ", " + ind.getGenotype() );
				}
			}
			writer.close();
		} catch ( Exception fnfe ) {
			EVLog.error( "saving individual collection to file " + file + " failed : unable to open file", fnfe );
		}
	}
	
	/**
	 * Checks if the printing of the genotype will be surpressed.
	 * 
	 * @return true if printing is surpressed false instead
	 */
	public boolean isGenotypeSurpressed() {
		return genotypeSurpressed;
	}

	/**
	 * Sets a flag to surpress the printing of the genotype.
	 * 
	 * @param b flag if genotype printing should be surpressed
	 */
	public void setGenotypeSurpressed( boolean b ) {
		genotypeSurpressed = b;
	}

}
