
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:58 $
 * $Revision: 1.12 $
 * $State: Exp $
 * $RCSfile: ConsoleWriter.java,v $
 *
 */

package org.evolvica.core.common;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;
import java.io.PrintWriter;

import org.apache.commons.lang.StringUtils;
import org.evolvica.core.IIndividual;
import org.evolvica.core.IIndividualSet;
import org.evolvica.core.ISetIterator;

/** Processor for printing individual collections.
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.12 $ $Date: 2004/05/26 11:57:58 $
 */
public class ConsoleWriter extends AbstractWriter {
	
	/**
	 * Printwriter to write information to.
	 */
	protected PrintWriter writer = null;
	
	/**
	 * Pattern that is to be written for each individual.
	 * The pattern string is expanded as follows:<br/>
	 * %f - expands to the fitness value of an individual<br/>
	 * %g - expands to the genotype of an individual<br/>
	 * %i - an optional index<br/>
	 * %s - expands to the score of an individual<br/>
	 */
	protected String pattern = "{%g}, S %s, F %f";
	
	/** Default constructor.
	* This sets the printwriter to System.out.
	* 
	*/
	public ConsoleWriter() {
		this( new PrintWriter( System.out, true ) );
	}
	
	/** Constructor with printwriter.
	* 
	* @param pw printwriter to write to
	*/
	public ConsoleWriter( PrintWriter pw ) {
		this.writer = pw;
	}
	
	/**
	 * Prints all individuals to the console.
	 * 
	 * @param set individual set to be written
	 */
	public void write( IIndividualSet set ) {
		int index = 0;
		for( ISetIterator it = set.iterator(); it.hasNext(); ) {
			IIndividual ind = it.next();
			String s = pattern;
			if ( ind.getFitness() != null ) {
				s = StringUtils.replace( s, "%f", ind.getFitness().toString() );
			} else {
				s = StringUtils.replace( s, "%f", "null" );
			}
			if ( ind.getGenotype() != null ) {
				s = StringUtils.replace( s, "%g", ind.getGenotype().toString() );
			} else {
				s = StringUtils.replace( s, "%g", "null" );
			}
			s = StringUtils.replace( s, "%i", String.valueOf( index ) );
			if ( ind.getScore() != null ) {
				s = StringUtils.replace( s, "%s", ind.getScore().toString() );
			} else {
				s = StringUtils.replace( s, "%s", "null" );
			}
			writer.println( s );
		}
	}
	
	/**
	 * Returns the current print pattern.
	 * 
	 * @return pattern used when printing individuals.
	 */
	public String getPattern() {
		return pattern;
	}

	/**
	 * Sets the pattern to be used when printing individuals.
	 * 
	 * @param string print pattern
	 */
	public void setPattern( String string ) throws PropertyVetoException {
		
		if ( StringUtils.isNotEmpty( string ) ) {
			this.pattern = string;
		} else {
			throw new PropertyVetoException( "pattern may not be empty",
				new PropertyChangeEvent( this, "pattern", this.pattern, string )
			);
		}
	}
	
}
