/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:00 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: CompoundScoreComparator.java,v $
 * 
 */
package org.evolvica.core.eval;

import org.evolvica.core.IScore;
import org.evolvica.core.IScoreComparator;

/** Compound score comparator.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:58:00 $
 */
public class CompoundScoreComparator extends AbstractScoreComparator {
	
	/**
	 * Array of comparators.
	 * 
	 */
	protected IScoreComparator comparators[] = null;
	
	/**
	 * Constructor.
	 */
	public CompoundScoreComparator() {
		this( null );
	}
	
	/**
	 * Constructor with array of comparators.
	 * 
	 * @param comparators array of comparators
	 */
	public CompoundScoreComparator( IScoreComparator[] comparators ) {
		this.comparators = comparators;
	}
	
	/** Compares two compound scores.
	 * This method returns 0 if the comparison has a indifferent result.
	 * It returns -2 resp. -1 if score0 strongly resp. weakly dominates
	 * score1. It returns 2 resp. 1 if score0 is strongly resp.
	 * weakly dominated by score1.
	 * 
	 * @param score0 first compound score
	 * @param score1 second compound score
	 * @return result of comparison
	 */
	public int compare( IScore score0, IScore score1 ) {
		CompoundScore compound0 = (CompoundScore) score0;
		CompoundScore compound1 = (CompoundScore) score1;
		int better0 = 0;
		int equal = 0;
		int better1 = 0;
		// now look at all scores and count the 'betters'
		for ( int i = 0; i < comparators.length; i++ ) {
			int rval = comparators[i].compare( compound0.getScore( i ), compound1.getScore( i ) );
			if ( rval < 0 ) {
				++better0;
			} else if ( rval > 0 ) {
				++better1;
			} else {
				++equal;
			}
		}
		// analyse the counted set, [this,equal,other]
		if ( equal == 0 ) { // we have no equals : [?,0,?]
			if ( better0 != 0 && better1 != 0 ) { // case indifferent : [X,0,X]
				return 0;
			} else if ( better0 != 0 ) { // case score0 strong dominance : [X,0,0]
				return -2;
			} else { // case score1 strong dominance : [0,0,X]
				return 2;
			}
		} else { // we have equals : [?,X,?]
			if ( better0 == better1 ) { // case indifferent : [Y,X,Y] && [0,X,0]
				return 0;
			} else if ( better0 == 0 ) { // case other weak dominance : [0,X,Y]
				return 1;
			} else if ( better1 == 0 ) { // case this weak dominance : [Y,X,0]
				return -1;
			} else { // case indifferent : [Y,X,Z]
				return 0;
			}
		}
 	}

	/**
	 * Returns the comparators.
	 * 
	 * @return array of comparators.
	 */
	public IScoreComparator[] getComparators() {
		return comparators;
	}
	
	/**
	 * Returns the comparator at the given index.
	 * 
	 * @param index index of the comparator
	 * @return comparator at the given index
	 */
	public IScoreComparator getComparators( int index ) {
		return comparators[index];
	}
	
	/**
	 * Sets the array of comparators.
	 * 
	 * @param comparators array of comparators.
	 */
	public void setComparators( IScoreComparator[] comparators ) {
		this.comparators = comparators;
	}
	
	
	/**
	 * Sets a comparator.
	 * 
	 * @param index index of the comparator
	 * @param comparator comparator
	 */
	public void setComparators( int index, IScoreComparator comparator ) {
		comparators[index] = comparator;
	}
}

