/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:04 $
 * $Revision: 1.12 $
 * $State: Exp $
 * $RCSfile: AbstractInitializer.java,v $
 *
 */
 
package org.evolvica.core.init;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;

import org.evolvica.core.*;
import org.evolvica.core.common.AbstractGeneticOperator;
import org.evolvica.engine.ISource;

/** Abstract base class for an initializer.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.12 $ $Date: 2004/05/26 11:58:04 $
 */
abstract public class AbstractInitializer extends AbstractGeneticOperator implements ISource {
	
	/**
	 * Number of individuals created by this initializer.
	 * 
	 */
	protected int setSize = 1;
	
	/**
	 * Individual builder to be used.
	 * 
	 */
	protected IIndividualBuilder individualBuilder = new DefaultIndividualBuilder();;
	
	/**
	 * Constructor.
	 */
	public AbstractInitializer() {
		// this.descriptor.setInterface( 0, 1 );
	}
	
	/**
	 * @see org.evolvica.engine.ISource#create()
	 */
	public Object create() {
		return createIndividualSet();
	}
	
	/**
	 * Creates a set of individuals.
	 * 
	 * @return created set of individuals
	 */
	public IIndividualSet createIndividualSet() {
		IIndividualSet set = new IndividualArray();
		for (int i = 0; i < setSize; i++) {
			IIndividual individual = individualBuilder.newIndividual();
			individual.setGenotype( createGenotype() ); 
			set.add( individual );
		}
		return set;
	}
	
	/**
	 * Creates a new genotype.
	 * 
	 * @return newly created genotype
	 */
	abstract protected IGenotype createGenotype();
	
	/**
	 * Returns the individual builder.
	 * 
	 * @return individual builder
	 */
	public IIndividualBuilder getIndividualBuilder() {
		return individualBuilder;
	}

	/**
	 * Returns the number of created individuals.
	 * 
	 * @return number of created individuals
	 */
	public int getSetSize() {
		return setSize;
	}

	/**
	 * Sets the individualBuilder.
	 * 
	 * @param individualBuilder the individual builder to set
	 */
	public void setIndividualBuilder( IIndividualBuilder individualBuilder ) throws PropertyVetoException {
		if ( individualBuilder != null ) {
			this.individualBuilder = individualBuilder;
		} else {
			throw new PropertyVetoException( "individual builder may not be set to null",
				new PropertyChangeEvent( this, "individualBuilder", this.individualBuilder, individualBuilder )
			);
		}
	}

	/**
	 * Sets the number of individuals to be created.
	 * 
	 * @param setSize number of individuals to be created
	 * @exception PropertyVetoException thrown if setting of this property fails
	 */
	public void setSetSize( int setSize ) throws PropertyVetoException {
		if ( setSize > 0 ) {
			this.setSize = setSize;
		} else {
			throw new PropertyVetoException( "setSize must be > 0",
				new PropertyChangeEvent( this, "setSize", new Integer( this.setSize ), new Integer( setSize ) )
			);
		}
	}

}
