/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:04 $
 * $Revision: 1.3 $
 * $State: Exp $
 * $RCSfile: FloatArrayInitializer.java,v $
 *
 */
package org.evolvica.core.init;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;

import org.evolvica.core.FloatArrayGene;
import org.evolvica.core.IGenotype;
import org.evolvica.util.RandomUtils;

/** Creation of float arrays.
 * This class creates an individuals containing an array of float numbers.
 * The array size and the range of values can be specified.
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.3 $ $Date: 2004/05/26 11:58:04 $
 */
public class FloatArrayInitializer extends AbstractInitializer {
	
	/** Size of the array to generated.
	* 
	*/
	protected int arraySize;
	
	/** Lower bound for the random integers (inclusive).
	* 
	*/
	protected float lowerBound;
	
	/** Upper bound for the random integers (exclusive).
	* 
	*/
	protected float upperBound;

	/**
	 * Constructor.
	 */	
	public FloatArrayInitializer() {
		this( 10, 0.0f, 10.0f );
	}
	
	/** Constructor.
	* The number of individuals to generated is set to 1.
	* 
	* @param arraySize size of the array to be generated
	* @param lowerBound lower bound for integers
	* @param upperBound upper bound for integers
	*/
	public FloatArrayInitializer( int arraySize, float lowerBound, float upperBound ) {
		this.arraySize = arraySize;
		this.lowerBound = lowerBound;
		this.upperBound = upperBound;
	}
	
	/**
	 * @see org.evolvica.core.init.AbstractInitializer#createGenotype()
	 */
	protected IGenotype createGenotype() {
		float[] array = new float[arraySize];
		for ( int i = 0; i < arraySize; i++ ) {
			array[i] = RandomUtils.randomFloat( lowerBound, upperBound );
		}
		return new FloatArrayGene( array );
	}

	/**
	 * Returns the array size.
	 * 
	 * @return int arrays size
	 */
	public int getArraySize() {
		return arraySize;
	}

	/**
	 * Returns the lower bound.
	 * 
	 * @return int lower bound
	 */
	public float getLowerBound() {
		return lowerBound;
	}

	/**
	 * Returns the upper bound.
	 * 
	 * @return int upper bound
	 */
	public float getUpperBound() {
		return upperBound;
	}

	/**
	 * Sets the array size.
	 * @param arraySize the array size to set
	 */
	public void setArraySize( int arraySize ) throws PropertyVetoException {
		if ( arraySize > 0 ) {
			this.arraySize = arraySize;
		} else {
			throw new PropertyVetoException( "array size must be > 0",
				new PropertyChangeEvent( this, "arraySize", new Integer( this.arraySize ), new Integer( arraySize ) )
			);
		}
	}
	
	/**
	 * Sets the lower bound.
	 * @param lowerBound the lower bound to set
	 */
	public void setLowerBound( float lowerBound ) throws PropertyVetoException {
		if ( lowerBound < upperBound ) {
			this.lowerBound = lowerBound;
		} else {
			throw new PropertyVetoException( "lower bound must be smaller than value of upper bound",
			new PropertyChangeEvent( this, "lowerBound", new Float( this.lowerBound ), new Float( lowerBound ) )
			);
		}
	}

	/**
	 * Sets the upper bound.
	 * @param upperBound the upper bound to set
	 */
	public void setUpperBound( float upperBound ) throws PropertyVetoException {
		if ( upperBound > lowerBound ) {
			this.upperBound = upperBound;
		} else {
			throw new PropertyVetoException( "upper bound must be greater than value of lower bound",
			new PropertyChangeEvent( this, "upperBound", new Float( this.upperBound ), new Float( upperBound ) )
			);
		}
	}

}
