/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:05 $
 * $Revision: 1.10 $
 * $State: Exp $
 * $RCSfile: IntegerArrayInitializer.java,v $
 *
 */
package org.evolvica.core.init;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;

import org.evolvica.core.IGenotype;
import org.evolvica.core.IntegerArrayGene;
import org.evolvica.util.RandomUtils;

/** Creation of integer arrays.
 * This class creates an individuals containing an array of integers.
 * The array size and the range of values can be specified.
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.10 $ $Date: 2004/05/26 11:58:05 $
 */
public class IntegerArrayInitializer extends AbstractInitializer {
	
	/** Size of the array to generated.
	* 
	*/
	protected int arraySize;
	
	/** Lower bound for the random integers (inclusive).
	* 
	*/
	protected int lowerBound;
	
	/** Upper bound for the random integers (exclusive).
	* 
	*/
	protected int upperBound;

	/**
	 * Constructor.
	 */	
	public IntegerArrayInitializer() {
		this( 10, 0, 10 );
	}
	
	/** Constructor.
	* The number of individuals to generated is set to 1.
	* 
	* @param arraySize size of the array to be generated
	* @param lowerBound lower bound for integers
	* @param upperBound upper bound for integers
	*/
	public IntegerArrayInitializer( int arraySize, int lowerBound, int upperBound ) {
		this.arraySize = arraySize;
		this.lowerBound = lowerBound;
		this.upperBound = upperBound;
	}
	
	/**
	 * @see org.evolvica.core.init.AbstractInitializer#createGenotype()
	 */
	protected IGenotype createGenotype() {
		int[] array = new int[arraySize];
		for ( int i = 0; i < arraySize; i++ ) {
			array[i] = RandomUtils.randomInt( lowerBound, upperBound );
		}
		return new IntegerArrayGene( array );
	}

	/**
	 * @return int
	 */
	public int getArraySize() {
		return arraySize;
	}

	/**
	 * @return int
	 */
	public int getLowerBound() {
		return lowerBound;
	}

	/**
	 * @return int
	 */
	public int getUpperBound() {
		return upperBound;
	}

	/**
	 * Sets the arraySize.
	 * @param arraySize The arraySize to set
	 */
	public void setArraySize( int arraySize ) throws PropertyVetoException {
		if ( arraySize > 0 ) {
			this.arraySize = arraySize;
		} else {
			throw new PropertyVetoException( "array size must be > 0",
				new PropertyChangeEvent( this, "arraySize", new Integer( this.arraySize ), new Integer( arraySize ) )
			);
		}
	}

	/**
	 * Sets the lowerBound.
	 * @param lowerBound The lowerBound to set
	 */
	public void setLowerBound( int lowerBound ) throws PropertyVetoException {
		if ( lowerBound < upperBound ) {
			this.lowerBound = lowerBound;
		} else {
			throw new PropertyVetoException( "lower bound must be smaller than value of upper bound",
			new PropertyChangeEvent( this, "lowerBound", new Integer( this.lowerBound ), new Integer( lowerBound ) )
			);
		}
	}

	/**
	 * Sets the upperBound.
	 * @param upperBound The upperBound to set
	 */
	public void setUpperBound( int upperBound ) throws PropertyVetoException {
		if ( upperBound > lowerBound ) {
			this.upperBound = upperBound;
		} else {
			throw new PropertyVetoException( "upper bound must be greater than value of lower bound",
			new PropertyChangeEvent( this, "upperBound", new Integer( this.upperBound ), new Integer( upperBound ) )
			);
		}
	}

}
