
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:10 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: ReverseMutator.java,v $
 *
 */

package org.evolvica.core.mutate.iarray;

import org.evolvica.util.RandomUtils;

/** Reverse mutation of an integer array.
 * This operator will randomly reverse a section of an array.
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:10 $
 */
public class ReverseMutator extends IntegerArrayMutator {
	
	/**
	* Lower bound.
	*/
	protected int lowerBound = 0;
	
	/**
	* Upper bound.
	*/
	protected int upperBound = Integer.MAX_VALUE;
	
	/**
	* Mutation range.
	*/
	protected float rangePercentage = -1.0f;
	
	/** Constructor.
	* 
	*/
	public ReverseMutator() {
		this.rangePercentage = -1.0f;
	}
	
	/** Constructor with random range.
	* 
	* @param rangePercentage percentage value of the length of a randomly selected range to reverse
	*/
	public ReverseMutator( float rangePercentage ) {
		this.rangePercentage = rangePercentage;
	}
	
	/** Constructor with lower and upper reverse bound.
	* 
	* @param lBound lower bound for reversing
	* @param uBound upper bound for reversing
	*/
	public ReverseMutator( int lBound, int uBound ) {
		this.lowerBound = lBound;
		this.upperBound = uBound;
	}
	
	/** Reverses an array of integers.
	* The reverse is done dependent on lowerBound and upperBound.
	* 
	* @param array array to be reversed
	* @return mutated array
	*/
	public int[] mutate( int[] array ) {
		int lb, ub;
		// check if bounds have been set
		if ( upperBound == Integer.MAX_VALUE ) { // no
			// check if percentage has been set
			if ( rangePercentage > 0.0 ) { // yes
				// calculate a valid range
				int[] b = randomRange( array.length );
				lb = b[0];
				ub = b[1];
			} else {
				// use whole length of list
				lb = 0;
				ub = array.length - 1;
			}
		} else { // yes
			lb = lowerBound;
			ub = upperBound;
		}
		for ( int i = lb, j = ub; i < j; i++, j-- ) {
			int tmp = array[i];
			array[i] = array[j];
			array[j] = tmp;
		}
		return array;
	}
	
	/** Helper for calculating a valid reverse range, dependent from length of array and percentage value.
	* 
	* @param length length of the array
	* @return integer array with lower and upper bound
	*/
	protected int[] randomRange( int length ) {
		int range = (int) Math.round( rangePercentage * (float) length );
		int[] rval = new int[2];
		rval[0] = RandomUtils.randomInt( length - range + 1 );
		rval[1] = rval[0] + range;
		return rval;
	}
	
}
