/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:10 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: ShiftMutator.java,v $
 *
 */

package org.evolvica.core.mutate.iarray;

import org.evolvica.util.RandomUtils;

/** Shift mutation of an integer array.
 * This operator will cut out random elements, shift the remaining elements upwards and append the elements,
 * that were clipped, to the end of the array.
 *
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:10 $
 */
public class ShiftMutator extends IntegerArrayMutator {
	
	/**
	* Number of clipped elements.
	*/
	protected int clipNumber = Integer.MAX_VALUE;

	/**
	* Number of clipped elements (percentage).
	*/
	protected float clipPercentage = -1.0f;
		
	/**
	* Array of clipped elements.
	*/
	protected int[] clipArray = null;
	
	/** Constructor.
	* The mutation propability is set to 0.05.
	*
	*/
	public ShiftMutator() {
		this.clipPercentage = -1.0f;
	}

	/** Constructor with number of clipped elements.
	*
	* @param percentage percentage value of the number of elements to be clipped
	*/
	public ShiftMutator( float percentage ) {
		this.clipPercentage = percentage;
	}

	/** Constructor with number of clipped elements.
	*
	* @param number number of elements to be clipped
	*/
	public ShiftMutator( int number ) {
		this.clipNumber = number;
	}

	/**
	 * @see org.evolvica.core.mutate.iarray.IntegerArrayMutator#mutate(int[])
	 */
	public int[] mutate( int[] array ) {
		int length = array.length;
		int[] cArray = setupClipArray( array, length );
		int[] save = new int[cArray.length];
		int replaceIndex = 0;
		int clipIndex = 0;
		for ( int i = 0; i < length; i++ ) {
			if ( clipIndex < cArray.length && i == cArray[clipIndex] ) {
				save[clipIndex++] = array[i];
			} else {
				if ( i != replaceIndex ) {
					array[replaceIndex] = array[i];
				}
				replaceIndex++;
			}
		}
		for ( int i = 0; i < save.length; i++ ) {
			array[replaceIndex++] = save[i];
		}
		return array;
	}
	
	/**
	 * Set up the clip array
	 * 
	 * @param array clip array
	 * @param arrayLength clip length
	 * @return clipped array
	 */
	protected int[] setupClipArray( int[] array, int arrayLength ) {
		// clipArray has already been set, so do nothing
		if ( this.clipArray != null ) {
			return this.clipArray;
		}
		// setup
		int number;
		// check if rotation distance has been set
		if ( this.clipNumber == Integer.MAX_VALUE ) { // no
			// check if percentage has been set
			if ( clipPercentage >= 0.0 ) { // yes
				// calculate a valid range
				number = (int) Math.round( clipPercentage * (float) arrayLength );
			} else {
				// use half of the length of the array
				number = arrayLength / 2;
			}
		} else { // yes
			if ( clipNumber < arrayLength ) {
				number = clipNumber;
			} else {
				number = arrayLength / 2;
			}
		}
		return RandomUtils.randomIntegerSample( number, arrayLength, true );
	}
	
}
