
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:10 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: SubstitutionMutator.java,v $
 *
 */

package org.evolvica.core.mutate.iarray;

import org.evolvica.util.RandomUtils;

/** Substitution mutation of an array.
 * This operator will replace random elements with elements selected randomly from a given substitution list.
 *
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:10 $
 */
public class SubstitutionMutator extends IntegerArrayMutator {
	
	/**
	* Substitution list.
	*/
	protected Object[] substitutionList = null;
	
	/**
	* Number of substitutions.
	*/
	protected int substNumber = Integer.MAX_VALUE;
	
	/**
	* Number of substitutions (percentage).
	*/
	protected float substPercentage = (float) -1.0;
	
	/**
	* Substitution list helper.
	*/
	protected int[] integerSubstitutionList = null;
	
	/**
	 * Constructor.
	 */
	public SubstitutionMutator() {
		this( null );
	}
	
	/** Constructor.
	* The mutation propability is set to 0.05.
	*
	* @param substList array with objects for substitution
	*/
	public SubstitutionMutator( int[] substList ) {
		this.integerSubstitutionList = substList;
	}

	/** Constructor with number of clipped elements.
	*
	* @param percentage percentage value of the number of elements to be clipped
	* @param substList array with objects for substitution
	*/
	public SubstitutionMutator( float percentage, int[] substList ) {
		this.substPercentage = percentage;
		this.integerSubstitutionList = substList;
	}

	/** Constructor with mutation propability and number of clipped elements.
	*
	* @param number number of elements to be clipped
	* @param substList array with objects for substitution
	*/
	public SubstitutionMutator( int number, int[] substList ) {
		this.substNumber = number;
		this.integerSubstitutionList = substList;
	}

	/**
	 * @see org.evolvica.core.mutate.iarray.IntegerArrayMutator#mutate(int[])
	 */
	public int[] mutate( int[] array ) {
		int arrayLength = array.length;
		int substitutionListLength = integerSubstitutionList.length;
		int[] substIndexArray = setupSubstArray( array, arrayLength );
		for ( int i = 0; i < substIndexArray.length; i++ ) {
			int index = substIndexArray[i];
			int subst = integerSubstitutionList[RandomUtils.randomInt( substitutionListLength )];
			array[index] = subst;
		}
		return array;
	}
	
	/**
	 * Setup substitution.
	 * 
	 * @param o object
	 * @param arrayLength substitution array
	 * @return mutated array
	 */
	protected int[] setupSubstArray( Object o, int arrayLength ) {
		// setup
		int number;
		// check if rotation distance has been set
		if ( this.substNumber == Integer.MAX_VALUE ) { // no
			// check if percentage has been set
			if ( substPercentage >= 0.0 ) { // yes
				// calculate a valid range
				number = (int) Math.round( substPercentage * (float) arrayLength );
			} else {
				// use half of the length of the array
				number = arrayLength / 2;
			}
		} else { // yes
			if ( substNumber < arrayLength ) {
				number = substNumber;
			} else {
				number = arrayLength / 2;
			}
		}
		return RandomUtils.randomIntegerSample( number, arrayLength, true );
	}
	
}
