/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:11 $
 * $Revision: 1.10 $
 * $State: Exp $
 * $RCSfile: AbstractInsertionFilter.java,v $
 *
 */
package org.evolvica.core.population;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;

import org.evolvica.core.IIndividualSet;


/** Abstract base class for an insertion filter.
 * Make sure to read the descriptions of generation gap and insertion rate.
 * These two values define whether a population grows or shrinks. Example:<br><br>
 * population size: 100<br>
 * insertion pool size: 20<br>
 * generation gap: 0.1<br>
 * insertion rate: 0.5<br><br>
 * generation gap size = 100 * 0.1 = 10; 10 individuals will be removed from the population<br>
 * insertion rate size = 20 * 0.5 = 10; 10 individuals from the insertion pool will be inserted into
 * the population.<br>
 * Therefore the new population size is 100 - 10 + 10 = 100 (and stays the same).
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.10 $ $Date: 2004/05/26 11:58:11 $
 */
abstract public class AbstractInsertionFilter implements IInsertionFilter {
	
	/** Generation Gap.
	* The generation gap is defined as the quotient of the number of
	* individuals in a population that are to be replaced and the size of the population.
	* The default value is 1.0, this will replace all individuals.
	*/
	protected float generationGap = (float) 1.0;
	
	/** Insertion Rate.
	* The reinsertion rate is defined as the quotient of the number of individuals that will be
	* inserted and the size of the insertion pool. The default value is 1.0.
	*/
	protected float insertionRate = (float) 1.0;
	
	/** Gets the value of the generation gap.
	* 
	* @return generation gap
	*/
	public float getGenerationGap() {
		return generationGap;
	}
	
	/** Gets the value of the reinsertion rate.
	* 
	* @return reinsertion rate
	*/
	public float getInsertionRate() {
		return insertionRate;
	}
	
	/** Sets the generation gap.
	* The given value must be between 0.0 and 1.0.
	* 
	* @param gap new value of generation gap
	* @exception PropertyVetoException thrown in case the property cannot be set
	*/
	public void setGenerationGap( float gap ) throws PropertyVetoException {
		if ( gap < 0.0 || gap > 1.0 ) {
			throw new PropertyVetoException( "value must fulfill: 0.0 <= value <= 1.0",
				new PropertyChangeEvent( this, "generationGap", new Float( this.generationGap ), new Float( gap ) ) );
		} else {
			generationGap = gap;
		}
	}
	
	/** Sets the insertion rate.
	* The given value must be between 0.0 and 1.0.
	* 
	* @param rate new value of insertion rate
	* @exception PropertyVetoException thrown in case the property cannot be set
	*/
	public void setInsertionRate( float rate ) throws PropertyVetoException {
		if ( rate < 0.0 || rate > 1.0 ) {
			throw new PropertyVetoException( "value must fulfill: 0.0 <= value <= 1.0",
				new PropertyChangeEvent( this, "insertionRate", new Float( this.insertionRate ), new Float( rate ) ) );
		} else {
			insertionRate = rate;
		}
	}
	
	/** Gets the number of individuals to be replaced.
	* This value depends on generation gap and population size and indicates
	* how many individuals will be replaced.
	* 
	* @param population the population used to calculate the generation gap
	* @return number of individuals to be replaced
	*/
	public int generationGapSize( IPopulation population ) {
		return Math.round( generationGap * (float) population.size() );
	}
	
	/** Gets the number of indivdiuals to be inserted.
	* This value depends on reinsertion rate and insertion pool size and indicates how many individuals will be
	* inserted into the population. If the pool is null this method returns 0.
	* 
	* @param pool the insertion pool used to calculate the insertion rate size
	* @return number of individuals to be inserted
	*/
	public int insertionRateSize( IIndividualSet pool ) {
		return Math.round( insertionRate * (float) pool.size() );
	}
	
	/**
	 * @see org.evolvica.core.population.IInsertionFilter#insert(org.evolvica.core.population.IPopulation, org.evolvica.core.IIndividualSet)
	 */
	abstract public void insert( IPopulation population, IIndividualSet pool );
	
}
