/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:11 $
 * $Revision: 1.14 $
 * $State: Exp $
 * $RCSfile: AbstractPopulation.java,v $
 *
 */
package org.evolvica.core.population;

import java.util.Collection;
import java.util.Iterator;

import org.evolvica.core.*;
import org.evolvica.core.common.AbstractGeneticOperator;

/** Abstract base class for a population.
 * This class provides some common methods for a population.
 *
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.14 $ $Date: 2004/05/26 11:58:11 $
 */
abstract public class AbstractPopulation extends AbstractGeneticOperator implements IPopulation {
	
	/** Data structure holding the individuals of this population.
	* This collection may be used in subclasses, but need not. In case another
	* model is used a user must ensure that all necessary methods are overloaded
	* correctly. Beyond this a new individual iterator must be written.
	*/
	protected Collection population;
	
	/** Generation counter.
	* This counter contains the current generation number. The counter is
	* incremented everytime the insert(IIndividualSet) method is called.
	*/
	protected int generation = 0;
	
	/** Individual filter for insertion of individuals into this population.
	*/
	protected IInsertionFilter filter = new DefaultInsertionFilter();
	
	/**
	 * Constructor.
	 */
	public AbstractPopulation() {
		// this.descriptor.setInterface( 1, 1 );
	}
	
	/**
	 * @see org.evolvica.core.IIndividualSet#add(org.evolvica.core.IIndividual)
	 */
	public void add( IIndividual ind ) {
		population.add( ind );
	}
	
	/**
	 * @see org.evolvica.core.IIndividualSet#add(org.evolvica.core.IIndividual[])
	 */
	public void add( IIndividual[] array ) {
		for (int i = 0; i < array.length; i++) {
			population.add( array );
		}
	}
	
	/**
	 * @see org.evolvica.core.IIndividualSet#clear()
	 */
	public void clear() {
		population.clear();
	}
	
	/**
	 * @see org.evolvica.core.IIndividualSet#contains(org.evolvica.core.IIndividual)
	 */
	public boolean contains( IIndividual ind ) {
		return population.contains( ind );
	}
	
	/**
	 * @see org.evolvica.core.IIndividualSet#isEmpty()
	 */
	public boolean isEmpty() {
		return population.isEmpty();
	}
	
	/**
	 * @see org.evolvica.core.IIndividualSet#iterator()
	 */
	public ISetIterator iterator() {
		return new PopulationIterator( population.iterator() );
	}
	
	/**
	 * @see org.evolvica.core.IIndividualSet#remove(org.evolvica.core.IIndividual)
	 */
	public void remove( IIndividual ind ) {
		population.remove( ind );
	}
	
	/**
	 * @see org.evolvica.core.IIndividualSet#remove(org.evolvica.core.IIndividual[])
	 */
	public void remove( IIndividual[] array ) {
		for (int i = 0; i < array.length; i++) {
			population.remove( array[i] );
		}
	}
	
	/**
	 * @see org.evolvica.core.IIndividualSet#size()
	 */
	public int size() {
		return population.size();
	}
	
	/**
	 * @see org.evolvica.core.IIndividualSet#toArray()
	 */
	public IIndividual[] toArray() {
		return (IIndividual[]) population.toArray( new IIndividual[population.size()] );
	}

	/** Gets the generation counter.
	*
	* @return generation counter of this population
	*/
	public int getGeneration() {
		return generation;
	}

	/**
	 * @return returns the insertion filter of this population
	 */
	public IInsertionFilter getFilter() {
		return filter;
	}

	/** Sets a new reinsertion operator (filter).
	*
	* @param filter new reinsertion operator
	*/
	public void setFilter( IInsertionFilter filter ) {
		this.filter = filter;
	}
	
	/**
	 * @see org.evolvica.engine.IProcessor#process(java.lang.Object)
	 */
	public Object process( Object input ) {
		return insert( (IIndividualSet) input );
	}

	/**
	 * @see org.evolvica.core.population.IPopulation#insert(org.evolvica.core.IIndividualSet)
	 */
	public IIndividualSet insert( IIndividualSet iset ) {
		filter.insert( this, iset );
		++generation; // increment generation counter
		IIndividualSet current = new IndividualArray( population.size() );
		current.add( toArray() );
		return current;
	}
	
	/**
	 * @see org.evolvica.core.population.IPopulation#replace(org.evolvica.core.IIndividual, org.evolvica.core.IIndividual)
	 */
	public void replace( IIndividual ind, IIndividual subst ) {
		remove( ind );
		add( subst );
	}
	
	/**
	 * @see org.evolvica.core.population.IPopulation#replace(org.evolvica.core.IIndividual[], org.evolvica.core.IIndividual[])
	 */
	public void replace( IIndividual[] indArray, IIndividual substArray[] ) {
		remove( indArray );
		add( substArray );
	}
	
	private class PopulationIterator implements ISetIterator {
		
		private Iterator iterator;
		
		public PopulationIterator( Iterator iterator ) {
			this.iterator = iterator;
		}
		
		public boolean hasNext() {
			return iterator.hasNext();
		}

		public IIndividual next() {
			return (IIndividual) iterator.next();
		}

	}
		
}
