
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:15 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: CycleCrossover.java,v $
 *
 */

package org.evolvica.core.recombine.iarray;

import java.util.HashMap;

import org.evolvica.util.RandomUtils;

/** Cycle Crossover (CX) for integer arrays.
 * This operator requires two parents and produces two descendants.
 * 
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:15 $
 */
public class CycleCrossover extends IntegerArrayRecombinator {
	
	/** Initial position to start cycle from.
	* 
	*/
	protected int initialPosition = -1;
	
	/** Constructor.
	* 
	*/
	public CycleCrossover() {
		this( -1 );
	}
	
	/** Constructor with initial cycle position.
	* This constructor should only be used for debugging purposes.
	*
	* @param initialPosition position to start cycle from
	*/
	public CycleCrossover( int initialPosition ) {
		this.initialPosition = initialPosition;
	}

	/**
	 * @see org.evolvica.core.recombine.iarray.IntegerArrayRecombinator#recombine(int[][])
	 */
	protected int[][] recombine(int[][] parents) {
		// look for size of chromosomes
		int size = parents[0].length;
		// create children
		int[] child0 = new int[size];
		int[] child1 = new int[size];
		// get interval
		int init; // initial position
		if ( initialPosition < 0 ) {
			init = RandomUtils.randomInt( size );
		} else {
			init = initialPosition;
		}
		// start the algorithm
		// create and fill lookup tables for positions of objects
		HashMap map0 = new HashMap();
		HashMap map1 = new HashMap();
		for ( int i = 0; i < size; i++ ) {
			map0.put( new Integer( parents[0][i] ), new Integer( i ) );
			map1.put( new Integer( parents[1][i] ), new Integer( i ) );
		}
		// cyclic assignment from parent0 -> child0
		int current = init;
		do {
			child0[current] = parents[0][current];
			int other = parents[1][current];
			current = ((Integer) map0.get( new Integer( other ) )).intValue();
			map0.remove( new Integer( other ) ); // remove mapping from hash table
		} while ( current != init );
		// cyclic assignment from parent1 -> child1
		current = init;
		do {
			child1[current] = parents[1][current];
			int other = parents[0][current];
			current = ((Integer) map1.get( new Integer( other ) )).intValue();
			map1.remove( new Integer( other ) ); // remove mapping from hash table
		} while ( current != init );
		// fill the gaps in both children
		for ( int i = 0; i < size; i++ ) {
			int o = parents[1][i]; // handle child 0
			Object mapping = map0.get( new Integer( o ) );
			if ( mapping != null ) {
				child0[i] = o;
			}
			o = parents[0][i]; // handle child 1
			mapping = map1.get( new Integer( o ) );
			if ( mapping != null ) {
				child1[i] = o;
			}
		}
		return new int[][] { child0, child1 };
	}

	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#getProducedChildren()
	 */
	public int getProducedChildren() {
		return 2;
	}

	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#getRequiredParents()
	 */
	public int getRequiredParents() {
		return 2;
	}

}
