
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:16 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: OrderCrossover.java,v $
 *
 */

package org.evolvica.core.recombine.iarray;

import java.util.HashSet;

import org.evolvica.util.RandomUtils;

/** Order Crossover (OX) for arrays.
 * This operator requires two parents and produces two descendants.
 * 
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:16 $
 */
public class OrderCrossover extends IntegerArrayRecombinator {
	
	/** Exchange interval.
	* The interval is defined between lowerBound (inclusive) and upperBound (exclusive).
	* 
	*/
	protected int[] intersectionPoints = null;
	
	/** Constructor.
	* 
	*/
	public OrderCrossover() {
		this( null );
	}
	
	/** Constructor with intersection points.
	* This constructor should only be used for debugging purposes.
	*
	* @param intersectionPoints array with intersection points
	*/
	public OrderCrossover( int[] intersectionPoints ) {
		this.intersectionPoints = intersectionPoints;
	}

	/**
	 * @see org.evolvica.core.recombine.iarray.IntegerArrayRecombinator#recombine(int[][])
	 */
	protected int[][] recombine( int[][] parents ) {
		// look for size of chromosomes
		int size;
		if ( parents[0].length > parents[1].length ) {
			size = parents[0].length;
		} else {
			size = parents[1].length;
		}
		// create children
		int[] child0 = new int[size];
		int[] child1 = new int[size];
		// get interval
		int[] interval; // intersection points
		if ( intersectionPoints == null ) {
			interval = RandomUtils.randomIntegerSample( 2, size, true );
		} else {
			interval = intersectionPoints;
		}
		// start the algorithm
		// mark objects in interval in other parent
		HashSet set0 = new HashSet();
		HashSet set1 = new HashSet();
		for ( int i = interval[0]; i < interval[1]; i++ ) {
			set1.add( new Integer( parents[0][i] ) );
			set0.add( new Integer( parents[1][i] ) );
		}
		// cyclic assignment beginning at starting position of interval (creation of child 0)
		int index = 0;
		for ( int i = interval[0]; i < size; i++ ) { // loop from start of interval to end of parent array
			int o = parents[0][i]; // get element from parent array
			if ( ! set0.contains( new Integer( o ) ) ) { // is element marked ?
				if ( index >= interval[0] && index < interval[1] ) { // check if index is within interval
					index = interval[1]; // it is: so move the index outside the interval
				}
				child0[index++] = o; // assign element from parent to child
			}
		}
		// cyclic assignment from 0 to starting position of interval (creation of child 0)
		for ( int i = 0; i < interval[0]; i++ ) { // loop from position 0 to start of interval
			int o = parents[0][i]; // get element from parent array
			if ( ! set0.contains( new Integer( o ) ) ) { // is element marked ?
				if ( index >= interval[0] && index < interval[1] ) { // check if index is within interval
					index = interval[1]; // it is: so move the index outside the interval
				}
				child0[index++] = o; // assign element from parent to child
			}
		}
		// cyclic assignment beginning at starting position of interval (creation of child 1)
		// same algorithm as creation of child 0
		index = 0;
		for ( int i = interval[0]; i < size; i++ ) {
			int o = parents[1][i];
			if ( ! set1.contains( new Integer( o ) ) ) {
				if ( index >= interval[0] && index < interval[1] ) {
					index = interval[1];
				}
				child1[index++] = o;
			}
		}
		// cyclic assignment from 0 to starting position of interval (creation of child 1)
		for ( int i = 0; i < interval[0]; i++ ) {
			int o = parents[1][i];
			if ( ! set1.contains( new Integer( o ) ) ) {
				if ( index >= interval[0] && index < interval[1] ) {
					index = interval[1];
				}
				child1[index++] = o;
			}
		}
		// exchange of elements within interval
		for ( int i = interval[0]; i < interval[1]; i++ ) {
			child0[i] = parents[1][i]; // assign elements from parent1 to child0
			child1[i] = parents[0][i]; // assign elements from parent0 to child1
		}
		return new int[][] { child0, child1 };
	}
	
	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#getProducedChildren()
	 */
	public int getProducedChildren() {
		return 2;
	}

	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#getRequiredParents()
	 */
	public int getRequiredParents() {
		return 2;
	}

}
