
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:16 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: PartialMatchedCrossover.java,v $
 *
 */

package org.evolvica.core.recombine.iarray;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;

import org.evolvica.util.RandomUtils;

/** Partial Matched Crossover (PMX) for integer arrays.
 * This operator requires two parents and produces two descendants.
 * 
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:16 $
 */
public class PartialMatchedCrossover extends IntegerArrayRecombinator {
	
	/** Exchange interval.
	* The interval is defined between lowerBound (inclusive) and upperBound (exclusive).
	* 
	*/
	protected int[] intersectionPoints = null;
	
	/** Constructor.
	* 
	*/
	public PartialMatchedCrossover() {
		this( null );
	}
	
	/** Constructor with intersection points.
	* This constructor should only be used for debugging purposes.
	*
	* @param intersectionPoints array with intersection points
	*/
	public PartialMatchedCrossover( int[] intersectionPoints ) {
		this.intersectionPoints = intersectionPoints;
	}
	
	/**
	 * @see org.evolvica.core.recombine.iarray.IntegerArrayRecombinator#recombine(int[][])
	 */
	protected int[][] recombine( int[][] parents ) {
		// get length of array
		int length = parents[0].length;
		// create children
		int[] child0 = new int[length];
		int[] child1 = new int[length];
		// look for the exchange interval
		int[] interval; // intersection points
		if ( intersectionPoints == null ) {
			interval = RandomUtils.randomIntegerSample( 2, length, true );
		} else {
			interval = intersectionPoints;
		}
		// create hash maps with mapping from object to index
		HashMap indexMap0 = new HashMap();
		HashMap indexMap1 = new HashMap();
		for ( int i = 0; i < length; i++ ) {
			indexMap0.put( new Integer( parents[0][i] ), new Integer( i ) );
			indexMap1.put( new Integer( parents[1][i] ), new Integer( i ) );
		}
		// create hash sets for exchanged elements
		HashSet exchangeSet0 = new HashSet();
		HashSet exchangeSet1 = new HashSet();
		// create hash maps for elements outside of exchange interval; mapped to index
		HashMap outsideMap0 = new HashMap();
		HashMap outsideMap1 = new HashMap();
		// copy elements from left side of interval
		for ( int i = 0; i < interval[0]; i++ ) {
			int o = parents[0][i];
			child0[i] = o;
			outsideMap0.put( new Integer( o ), new Integer( i ) );
			o = parents[1][i];
			child1[i] = o;
			outsideMap1.put( new Integer( o ), new Integer( i ) );
		}
		// copy elements from right side of interval
		for ( int i = interval[1]; i < length; i++ ) {
			int o = parents[0][i];
			child0[i] = o;
			outsideMap0.put( new Integer( o ), new Integer( i ) );
			o = parents[1][i];
			child1[i] = o;
			outsideMap1.put( new Integer( o ), new Integer( i ) );
		}
		// exchange the objects in the interval and add them to the exchange set
		for ( int i = interval[0]; i < interval[1]; i++ ) {
			int ex0 = parents[0][i];
			int ex1 = parents[1][i];
			child0[i] = ex1;
			child1[i] = ex0;
			exchangeSet0.add( new Integer( ex1 ) );
			exchangeSet1.add( new Integer( ex0 ) );
		}
		for ( Iterator it = exchangeSet0.iterator(); it.hasNext(); ) {
			Object o = it.next();
			if ( outsideMap0.containsKey( o ) ) {
				int index = ((Integer) indexMap1.get( o )).intValue();
				int replaceIndex = ((Integer) outsideMap0.get( o )).intValue();
				child0[replaceIndex] = parents[0][index];
			}
		}
		for ( Iterator it = exchangeSet1.iterator(); it.hasNext(); ) {
			Object o = it.next();
			if ( outsideMap1.containsKey( o ) ) {
				int index = ((Integer) indexMap0.get( o )).intValue();
				int replaceIndex = ((Integer) outsideMap1.get( o )).intValue();
				child1[replaceIndex] = parents[1][index];
			}
		}
		return new int[][] { child0, child1 };
	}
	
	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#getProducedChildren()
	 */
	public int getProducedChildren() {
		return 2;
	}

	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#getRequiredParents()
	 */
	public int getRequiredParents() {
		return 2;
	}
}
