
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:18 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: TournamentSelector.java,v $
 *
 */

package org.evolvica.core.select;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;

import org.evolvica.core.IIndividual;
import org.evolvica.core.IIndividualComparator;
import org.evolvica.util.RandomUtils;

/** Tournament Selection.
 * This class implements a tournament selection. In this variant tournaments are executed between random
 * individuals. The best individual wins the tournament and is selected.
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:18 $
 */
public class TournamentSelector extends AbstractSelector {
	
	/** Number of individuals attending a tournament.
	 */
	protected int tournamentSize = 2;
	
	/** Individual comparator for use in tournament.
	 */
	protected IIndividualComparator comparator = null;
	
	/**
	 * Constructor.
	 */
	public TournamentSelector() {
		this( -1, 2, null );
	}
	
	/** Constructor with number per selection, tournament size and individual comparator.
	 * 
	 * @param numberPerSelection number how many individuals are selected
	 * @param tournamentSize size of the tournament
	 * @param comparator individual comparator to use for comparisons
	 */
	public TournamentSelector( int numberPerSelection, int tournamentSize, IIndividualComparator comparator ) {
		super( numberPerSelection );
		this.tournamentSize = tournamentSize;
		this.comparator = comparator;
	}

	/**
	 * @see org.evolvica.core.select.AbstractSelector#select(org.evolvica.core.IIndividual[])
	 */
	public IIndividual[] select( IIndividual[] pool ) {
		IIndividual[] rval = null;
		if ( numberPerSelection < 0 ) {
			rval = new IIndividual[pool.length];
		} else {
			rval = new IIndividual[numberPerSelection];
		}
		for ( int i = 0; i < rval.length; i++ ) {
			rval[i] = selectWinner( pool );
		}
		return rval;
	}
	
	/**
	 * Selects a winner.
	 * 
	 * @param pool tournament attendees
	 * @return winner
	 */
	protected IIndividual selectWinner( IIndividual[] pool ) {
		int[] selectionIndex = RandomUtils.randomIntegerSample( tournamentSize, pool.length, true );
		IIndividual winner = pool[selectionIndex[0]];
		for ( int j = 1; j < selectionIndex.length; j++ ) {
			IIndividual opponent = pool[selectionIndex[j]];
			if ( comparator.compare( winner, opponent ) > 0 ) {
				winner = opponent;
			}
		}
		return winner;
	}
	
	/** Sets a new tournament size.
	 * 
	 * @param tournamentSize new tournament size
	 * @exception PropertyVetoException thrown in case the property cannot be set
	 */
	public void setTournamentSize( int tournamentSize ) throws PropertyVetoException {
		if ( tournamentSize > 1 ) {
			this.tournamentSize = tournamentSize;
		} else {
			throw new PropertyVetoException( "tournament size must be bigger than 1",
				new PropertyChangeEvent(
					this,
					"tournamentSize",
					new Integer( this.tournamentSize ),
					new Integer( tournamentSize )
				)
			);
		}
	}

	/** Gets the current tournament size.
	 * 
	 * @return current tournament size
	 */
	public int getTournamentSize() {
		return tournamentSize;
	}
	
	/**
	 * Returns the comparator.
	 * 
	 * @return comparator
	 */
	public IIndividualComparator getComparator() {
		return comparator;
	}

	/**
	 * Sets the comparator.
	 * 
	 * @param comparator comparator
	 */
	public void setComparator( IIndividualComparator comparator ) {
		this.comparator = comparator;
	}

}
