/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:18 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: AbsoluteTimeCondition.java,v $
 *
 */
package org.evolvica.core.terminate;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;
import java.util.Calendar;
import java.util.GregorianCalendar;

import org.evolvica.core.IIndividualSet;
import org.evolvica.util.EVLog;

/** Termination at specified time.
 * The condition is fulfilled in case of (currentTime >= targetTime). The target time
 * can be specified in the constructor.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:58:18 $
 */
public class AbsoluteTimeCondition implements ICondition {
	
	/** Current time.
	*
	*/
	protected GregorianCalendar current = new GregorianCalendar();

	/** Target time.
	*
	*/
	protected GregorianCalendar target = new GregorianCalendar();
	
	/**
	 * Year.
	 */
	protected int year = 0;
	
	/**
	 * Month.
	 */
	protected int month = 0;
	
	/**
	 * Day.
	 */
	protected int day = 0;
	
	/**
	 * Hour.
	 */
	protected int hour = 0;
	
	/**
	 * Minute.
	 */
	protected int minute = 0;
	
	/**
	 * Constructor.
	 */
	public AbsoluteTimeCondition() {
		target.add( Calendar.HOUR_OF_DAY, 1 );
		year = target.get( Calendar.YEAR );
		month = target.get( Calendar.MONTH );
		day = target.get( Calendar.DAY_OF_MONTH );
		hour = target.get( Calendar.HOUR_OF_DAY );
		minute = target.get( Calendar.MINUTE );
	}
	
	/** Constructor with target time.
	*
	* @param year year
	* @param month month
	* @param day day
	* @param hour hour
	* @param minute minute
	*/
	public AbsoluteTimeCondition( int year, int month, int day, int hour, int minute ) {
		target = new GregorianCalendar( year, month, day, hour, minute );
	}

	/** Test of condition.
	*
	* @param set individual set
	* @return result of condition test
	*/
	public boolean isTrue( IIndividualSet set ) {
		current = new GregorianCalendar();
		if ( current.after( target ) ) {
			EVLog.info( "absolute time condition met at " +
				current.get( Calendar.YEAR ) + "/" + (current.get( Calendar.MONTH ) + 1) + "/" + current.get( Calendar.DAY_OF_MONTH ) + " " +
				current.get( Calendar.HOUR_OF_DAY ) + ":" + current.get( Calendar.MINUTE ) + ":" + current.get( Calendar.SECOND )
			);
			return true;
		} else {
			return false;
		}
	}
	/**
	 * Returns the day
	 * 
	 * @return day
	 */
	public int getDay() {
		return day;
	}

	/**
	 * Returns the hour.
	 * 
	 * @return hour
	 */
	public int getHour() {
		return hour;
	}

	/**
	 * Returns the minute.
	 * 
	 * @return minute
	 */
	public int getMinute() {
		return minute;
	}

	/**
	 * Returns the month.
	 * 
	 * @return month
	 */
	public int getMonth() {
		return month;
	}

	/**
	 * Returns the year.
	 * 
	 * @return year
	 */
	public int getYear() {
		return year;
	}

	/**
	 * Sets the day.
	 * 
	 * @param value day
	 * @exception PropertyVetoException thrown in case the property cannot be set
	 */
	public void setDay( int value ) throws PropertyVetoException {
		if ( value >= target.getMinimum( Calendar.DAY_OF_MONTH ) && value <= target.getMaximum( Calendar.DAY_OF_MONTH ) ) {
			target.set( year, month, value, hour, minute, 0 );
			day = value;
		} else {
			throw new PropertyVetoException( "cannot set day", new PropertyChangeEvent(
				this, "target change", new Integer( day ), new Integer( value )
			) );
		}
	}

	/**
	 * Sets the hour.
	 * 
	 * @param value hour
	 * @exception PropertyVetoException thrown in case the property cannot be set
	 */
	public void setHour( int value ) throws PropertyVetoException {
		if ( value >= target.getMinimum( Calendar.HOUR_OF_DAY ) && value <= target.getMaximum( Calendar.HOUR_OF_DAY ) ) {
			target.set( year, month, day, value, minute, 0 );
			hour = value;
		} else {
			throw new PropertyVetoException( "cannot set hour", new PropertyChangeEvent(
				this, "target change", new Integer( hour ), new Integer( value )
			) );
		}
	}

	/**
	 * Sets the minute.
	 * 
	 * @param value minute
	 * @exception PropertyVetoException thrown in case the property cannot be set
	 */
	public void setMinute( int value ) throws PropertyVetoException {
		if ( value >= target.getMinimum( Calendar.MINUTE ) && value <= target.getMaximum( Calendar.MINUTE ) ) {
			target.set( year, month, day, hour, value, 0 );
			minute = value;
		} else {
			throw new PropertyVetoException( "cannot set minute", new PropertyChangeEvent(
				this, "target change", new Integer( minute ), new Integer( value )
			) );
		}
	}

	/**
	 * Sets the month.
	 * 
	 * @param value month
	 * @exception PropertyVetoException thrown in case the property cannot be set
	 */
	public void setMonth( int value ) throws PropertyVetoException {
		if ( value >= target.getMinimum( Calendar.MONTH ) && value <= target.getMaximum( Calendar.MONTH ) ) {
			target.set( year, value, day, hour, minute, 0 );
			month = value;
		} else {
			throw new PropertyVetoException( "cannot set month", new PropertyChangeEvent(
				this, "target change", new Integer( month ), new Integer( value )
			) );
		}
	}

	/**
	 * Sets the year.
	 * 
	 * @param value year
	 * @exception PropertyVetoException thrown in case the property cannot be set
	 */
	public void setYear( int value ) throws PropertyVetoException {
		if ( value >= target.getMinimum( Calendar.YEAR ) && value <= target.getMaximum( Calendar.YEAR ) ) {
			target.set( value, month, day, hour, minute, 0 );
			year = value;
		} else {
			throw new PropertyVetoException( "cannot set year", new PropertyChangeEvent(
				this, "target change", new Integer( year ), new Integer( value )
			) );
		}
	}

}
