/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:19 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: Messages.java,v $
 *
 */
package org.evolvica.core.terminate;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.apache.commons.lang.StringUtils;
import org.evolvica.util.ClassUtils;

/**
 * Provides access to string resources.
 * All string resources are held in a resource file and can be accessed via their
 * appropriate key. A valid key looks like that: CallingClass.key. The calling class
 * can be optionally omitted and is determined automatically if missing. Therefore
 * getString( "MyClass.myKey" ) and getString( myKey ) will return the same values.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:58:19 $
 *
 * @since 0.4.0
 */
public class Messages {
	
	// the name of the resource bundle to be used
	private static final String BUNDLE_NAME = "org.evolvica.resources.terminate"; //$NON-NLS-1$
	
	// the resource bundle to be used
	private static final ResourceBundle RESOURCE_BUNDLE = ResourceBundle.getBundle( BUNDLE_NAME );
	
	private Messages() {
		// do not instantiate
	}

	/**
	 * Returns the value of the given key from the resource bundle.
	 *
	 * @param key key
	 *
	 * @return value
	 */
	public static final String getString( String key ) {
		try {
			return RESOURCE_BUNDLE.getString( getResourceKey( key ) );
		} catch ( MissingResourceException ex ) {
			return "!$RESOURCE KEY " + key + " MISSING$!"; //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	/**
	 * Returns the value of the given key from the resource bundle and substitutes %0 occurences.
	 *
	 * @param key key
	 * @param subst0 substitution for %0
	 *
	 * @return value
	 */
	public static final String getString( String key, String subst0 ) {
		try {
			String message = RESOURCE_BUNDLE.getString( getResourceKey( key ) );
			return StringUtils.replace( message, "%0", subst0 ); //$NON-NLS-1$
		} catch ( MissingResourceException ex ) {
			return "!$RESOURCE KEY " + key + " MISSING$!"; //$NON-NLS-1$ //$NON-NLS-2$
		}
	}

	/**
	 * Returns the value of the given key from the resource bundle and substitutes %0 and %1 occurences.
	 *
	 * @param key key
	 * @param subst0 substitution for %0
	 * @param subst1 substitution for %1
	 *
	 * @return value
	 */
	public static final String getString( String key, String subst0, String subst1 ) {
		try {
			String message = RESOURCE_BUNDLE.getString( getResourceKey( key ) );
			message = StringUtils.replace( message, "%0", subst0 ); //$NON-NLS-1$
			return StringUtils.replace( message, "%1", subst1 ); //$NON-NLS-1$
		} catch ( MissingResourceException ex ) {
			return "!$RESOURCE KEY " + key + " MISSING$!"; //$NON-NLS-1$ //$NON-NLS-2$
		}
	}

	/**
	 * Returns the value of the given key from the resource bundle and substitutes %0, %1 and %2 occurences.
	 *
	 * @param key key
	 * @param subst0 substitution for %0
	 * @param subst1 substitution for %1
	 * @param subst2 substitution for %2
	 *
	 * @return value
	 */
	public static final String getString( String key, String subst0, String subst1, String subst2 ) {
		try {
			String message = RESOURCE_BUNDLE.getString( getResourceKey( key ) );
			message = StringUtils.replace( message, "%0", subst0 ); //$NON-NLS-1$
			message = StringUtils.replace( message, "%1", subst1 ); //$NON-NLS-1$
			return StringUtils.replace( message, "%2", subst2 ); //$NON-NLS-1$
		} catch ( MissingResourceException ex ) {
			return "!$RESOURCE KEY " + key + " MISSING$!"; //$NON-NLS-1$ //$NON-NLS-2$
		}
	}

	/**
	 * Returns the value of the given key from the resource bundle and substitutes %index occurences.
	 *
	 * @param key key
	 * @param subst substitutions for %0, %1, and so on
	 *
	 * @return value
	 */
	public static final String getString( String key, String[] subst ) {
		try {
			String message = RESOURCE_BUNDLE.getString( getResourceKey( key ) );
			for ( int i = 0; i < subst.length; i++ ) {
				message = StringUtils.replace( message, "%" + i, subst[i] ); //$NON-NLS-1$
			}
			return message;
		} catch ( MissingResourceException ex ) {
			return "!$RESOURCE KEY " + key + " MISSING$!"; //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	// helper method
	private static final String getResourceKey( String key ) {
		int index = key.lastIndexOf( '.' );
		String resourceKey = null;
		if ( index > 0 ) { // the calling class is specified
			resourceKey = key;
		} else {
			Class clazz = ClassUtils.callingClass( 3 );
			String className = clazz.getName();
			resourceKey = className.substring( className.lastIndexOf( '.' ) + 1 ) + "." + key; //$NON-NLS-1$
		}
		return resourceKey;
	}
	
}
