/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:19 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: RelativeErrorCondition.java,v $
 *
 */

package org.evolvica.core.terminate;

import org.evolvica.core.*;

/**
 * Termination at an relative error to the target value.
 * The break condition is fulfilled in case that current relative
 * error < given relative error. The relative error is calculated as:
 * Abs( targetValue - bestScore) / targetValue ). Note that the calculation
 * fails, if the target value is 0.0.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:58:19 $
 */
public class RelativeErrorCondition extends ScoreComparingCondition {
	
	/**
	 * Target value.
	 */
	protected double target;
	
	/**
	 * Target error.
	 */
	protected double targetError; 
	
	/**
	 * Constructor.
	 */
	public RelativeErrorCondition() {
		this( 0.0, 0.0, null );
	}
	
	/**
	 * Constructor.
	 * 
	 * @param target target value
	 * @param targetError target error
	 * @param comparator score comparator
	 */
	public RelativeErrorCondition( double target, double targetError, IScoreComparator comparator ) {
		super( comparator );
		this.target = target;
		this.targetError = targetError;
	}
	
	/**
	 * @see org.evolvica.core.terminate.ICondition#isTrue(org.evolvica.core.IIndividualSet)
	 */
	public boolean isTrue( IIndividualSet iset ) {
		IIndividual best = IndividualSetUtils.findBest( iset, comparator );
		double bestScore = ((Number) best.getScore().getValueAsObject()).doubleValue();
		float currentError = (float) Math.abs( ( target - bestScore) / target );
		if ( currentError <= targetError ) {
			return true;
		} else {
			return false;
		}
		
	}

	/**
	 * Returns the target value.
	 * 
	 * @return target value
	 */
	public double getTarget() {
		return target;
	}

	/**
	 * Returns the target error.
	 * 
	 * @return target error
	 */
	public double getTargetError() {
		return targetError;
	}

	/**
	 * Sets the target value.
	 * 
	 * @param d target value
	 */
	public void setTarget( double d ) {
		target = d;
	}

	/**
	 * Stes the target error.
	 * 
	 * @param d target error
	 */
	public void setTargetError( double d ) {
		targetError = d;
	}

}
