/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:20 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: RunningBestScoreCondition.java,v $
 *
 */

package org.evolvica.core.terminate;

import org.evolvica.core.IIndividualSet;
import org.evolvica.core.IScoreComparator;
import org.evolvica.core.IndividualSetUtils;

/**
 * Termination dependent on the running best score.
 * The termination condition is fulfilled, if current best
 * score - average value of best scores of last n generations falls
 * under a particular threshold.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:58:20 $
 */
public class RunningBestScoreCondition extends ScoreComparingCondition {
	
	/**
	 * Threshold.
	 */
	protected double threshold;
	
	/**
	 * Stack.
	 */
	protected double[] stack;
	
	/**
	 * Next index.
	 */
	protected int nextIndex = 0;
	
	/**
	 * Indicator if stack is full.
	 */
	protected boolean stackFull = false;
	
	/**
	 * Constructor.
	 */
	public RunningBestScoreCondition() {
		this( 0.0, 0, null );
	}
	
	/**
	 * Constructor.
	 * 
	 * @param threshold threshold
	 * @param generations generations
	 * @param comparator score comparator
	 */
	public RunningBestScoreCondition( double threshold, int generations, IScoreComparator comparator ) {
		super( comparator );
		this.threshold = threshold;
		this.stack = new double[generations];
	}
	
	/**
	 * @see org.evolvica.core.terminate.ICondition#isTrue(org.evolvica.core.IIndividualSet)
	 */
	public boolean isTrue( IIndividualSet iset ) {
		double bestScore = ((Number) IndividualSetUtils.findBest( iset, comparator ).getScore().getValueAsObject()).doubleValue();
		stack[nextIndex++] = bestScore;
		if ( nextIndex >= stack.length ) {
			nextIndex = 0;
			stackFull = true;
		}
		if ( stackFull == true ) {
			double average = 0.0;
			for ( int i = 0; i < stack.length; i++ ) {
				average += stack[i];
			}
			average /= (double) stack.length;
			double diff = Math.abs( bestScore - average );
			if ( diff < threshold ) {
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Returns the number of generations.
	 * 
	 * @return number of generations
	 */
	public int getGenerations() {
		return stack.length;
	}

	/**
	 * Returns the threshold.
	 * 
	 * @return threshold
	 */
	public double getThreshold() {
		return threshold;
	}

	/**
	 * Sets the number of generations.
	 * 
	 * @param i number of generations
	 */
	public void setGenerations( int i ) {
		this.stack = new double[i];
	}

	/**
	 * Sets the threshold.
	 * 
	 * @param d threshold
	 */
	public void setThreshold( double d ) {
		threshold = d;
	}

}
