/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:21 $
 * $Revision: 1.14 $
 * $State: Exp $
 * $RCSfile: AbstractAlgorithm.java,v $
 *
 */
package org.evolvica.engine;

import gnu.trove.THashSet;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyVetoException;
import java.util.Iterator;

import org.evolvica.util.*;

/**
 * Abstract base class for an algorithm implementation.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.14 $ $Date: 2004/05/26 11:58:21 $
 */
abstract public class AbstractAlgorithm implements IAlgorithm {
	
	private THashSet propertyChangeListeners = new THashSet();
	
	/**
	 * Algorithm name.
	 * 
	 */
	protected String name = "default algorithm";
	
	/**
	 * Algorithm elements.
	 * 
	 */
	protected THashSet elements = new THashSet();
	
	/**
	 * Connections.
	 * 
	 */
	protected THashSet connections = new THashSet();
	
	/**
	 * Algorithm runner.
	 * 
	 */
	protected IRunner runner = new DefaultAlgorithmRunner();
	
	/**
	 * Algorithm validator.
	 * 
	 */
	protected IValidator validator = new DefaultValidator();
	
	/**
	 * The metadata store of this algorithm.
	 */
	protected MetadataStore metadata = new MetadataStore();
	
	/**
	 * @see org.evolvica.engine.IAlgorithm#addElement(org.evolvica.engine.IElement)
	 */
	public void addElement( IElement element ) {
		if ( element != null ) {
			elements.add( element );
			element.setAlgorithm( this );
			firePropertyChangeEvent( new PropertyChangeEvent( this, Events.ELEMENT_ADDED, null, element ) );
		}
	}
	
	/**
	 * Adds an operator to this algorithm.
	 * An algorithm element is created for the given operator and added to this algorithm.
	 * 
	 * @param operator operator to be added
	 */
	public void addOperator( IOperator operator ) {
		addElement( ElementFactory.newElement( operator ) );
	}
	
	/**
	 * Return the algorithm element for a given operator.
	 * 
	 * @param operator operator
	 * @return appropriate algorithm element of null
	 */
	public IElement getElement( IOperator operator ) {
		for (Iterator it = elements.iterator(); it.hasNext(); ) {
			IElement element = (IElement) it.next();
			if ( element.getOperator() == operator ) {
				return element;
			}
		}
		return null;
	}
	
	/**
	 * @see org.evolvica.engine.IAlgorithm#getElementById(java.lang.String)
	 */
	public IElement getElementById( String id ) {
		for (Iterator it = elements.iterator(); it.hasNext(); ) {
			IElement element = (IElement) it.next();
			if ( element.getOperator().getId().equals( id ) ) {
				return element;
			}
		}
		return null;
	}
	
	/**
	 * @see org.evolvica.engine.IAlgorithm#iterateElements()
	 */
	public Iterator iterateElements() {
		return elements.iterator();
	}
	
	/**
	 * @see org.evolvica.engine.IAlgorithm#removeElement(org.evolvica.engine.IElement)
	 */
	public void removeElement( IElement element ) {
		// remove all incoming connections
		for (int i = 0; i < element.incomingSize(); i++) {
			removeConnection( element.getIncoming( i ) );
		}
		// remove all outgoing connections
		for (int i = 0; i < element.outgoingSize(); i++) {
			removeConnection( element.getOutgoing( i ) );
		}
		// and finally remove the element itself
		elements.remove( element );
		element.setAlgorithm( null );
		firePropertyChangeEvent( new PropertyChangeEvent( this, Events.ELEMENT_REMOVED, null, element ) );
	}
	
	/**
	 * Removes the element for a given operator from this algorithm.
	 * 
	 * @param operator operator to be removed from this algorithm
	 */
	public void removeOperator( IOperator operator ) {
		IElement element = getElement( operator );
		if ( element != null ) {
			removeElement( element );
		}
	}
	
	/**
	 * @see org.evolvica.engine.IAlgorithm#elementSize()
	 */
	public int elementSize() {
		return elements.size();
	}
	
	/**
	 * Connects two operator.
	 * This is a short cut for connect( source, 0, target, 0 ).
	 * 
	 * @param source source operator
	 * @param target target operator
	 */
	public void connect( IOperator source, IOperator target ) {
		connect( source, 0, target, 0 );
	}
	
	/**
	 * Connects two operators.
	 * This is a short cut for connect( source, sourceSlot, target, 0 ).
	 * 
	 * @param source source operator
	 * @param output output slot
	 * @param target target operator
	 */
	public void connect( IOperator source, int output, IOperator target ) {
		connect( source, output, target, 0 );
	}
	
	/**
	 * Connects two operators.
	 * This is a short cut for connect( source, 0, target, targetSlot ).
	 * 
	 * @param source source operator
	 * @param target target operator
	 * @param input input slot
	 */
	public void connect( IOperator source, IOperator target, int input ) {
		connect( source, 0, target, input );
	}
	
	/**
	 * @see org.evolvica.engine.IAlgorithm#connect(org.evolvica.engine.IOperator, int, org.evolvica.engine.IOperator, int)
	 */
	public void connect( IOperator source, int output, IOperator target, int input ) {
		IElement sourceElement = getElement( source );
		if ( sourceElement == null ) {
			sourceElement = ElementFactory.newElement( source );
			addElement( sourceElement );
		}
		IElement targetElement = getElement( target );
		if ( targetElement == null ) {
			targetElement = ElementFactory.newElement( target );
			addElement( targetElement );
		}
		connect( sourceElement, output, targetElement, input );
	}
	
	/**
	 * @see org.evolvica.engine.IAlgorithm#connect(org.evolvica.engine.IElement, int, org.evolvica.engine.IElement, int)
	 */
	public void connect( IElement source, int output, IElement target, int input ) {
		Connection connection = new Connection( source, target );
		source.setOutgoing( connection, output );
		target.setIncoming( connection, input );
		connections.add( connection );
		firePropertyChangeEvent( new PropertyChangeEvent( this, Events.CONNECTION_ADDED, null, connection ) );
	}
	
	/**
	 * @see org.evolvica.engine.IAlgorithm#getConnection(org.evolvica.engine.IElement, int, org.evolvica.engine.IElement, int)
	 */
	public IConnection getConnection( IElement source, int output, IElement target, int input ) {
		IConnection sourceConnection = source.getOutgoing( output );
		IConnection targetConnection = target.getIncoming( input );
		if ( sourceConnection == targetConnection ) {
			return sourceConnection;
		} else {
			return null;
		}
	}

	/**
	 * @see org.evolvica.engine.IAlgorithm#getConnection(org.evolvica.engine.IOperator, int, org.evolvica.engine.IOperator, int)
	 */
	public IConnection getConnection( IOperator source, int output, IOperator target, int input ) {
		return getConnection( getElement( source ), output, getElement( target ), input );
	}

	/**
	 * @see org.evolvica.engine.IAlgorithm#iterateConnections()
	 */
	public Iterator iterateConnections() {
		return connections.iterator();
	}
	
	/**
	 * @see org.evolvica.engine.IAlgorithm#removeConnection(org.evolvica.engine.IConnection)
	 */
	public void removeConnection( IConnection connection ) {
		if ( connection != null ) {
			connection.getSource().removeConnection( connection );
			connection.getTarget().removeConnection( connection );
			connections.remove( connection );
			firePropertyChangeEvent( new PropertyChangeEvent( this, Events.CONNECTION_REMOVED, null, connection ) );
		}
	}
	
	/**
	 * @see org.evolvica.engine.IAlgorithm#connectionSize()
	 */
	public int connectionSize() {
		return connections.size();
	}
	
	/**
	 * @see org.evolvica.engine.IAlgorithm#setup()
	 */
	abstract public void setup() throws Exception;
	
	/**
	 * @see org.evolvica.engine.IAlgorithm#teardown()
	 */
	abstract public void teardown() throws Exception;
	
	/**
	 * @see org.evolvica.engine.IAlgorithm#execute()
	 */
	public void execute() {
		runner.start( this );
	}

	/**
	 * @see org.evolvica.engine.IAlgorithm#getName()
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * @see org.evolvica.engine.IAlgorithm#getRunner()
	 */
	public IRunner getRunner() {
		return runner;
	}

	/**
	 * @see org.evolvica.engine.IAlgorithm#getValidator()
	 */
	public IValidator getValidator() {
		return validator;
	}

	/**
	 * @see org.evolvica.engine.IAlgorithm#setName(java.lang.String)
	 */
	public void setName( String string ) {
		name = string;
	}

	/**
	 * @see org.evolvica.engine.IAlgorithm#setRunner(org.evolvica.engine.IRunner)
	 */
	public void setRunner( IRunner runner ) throws PropertyVetoException {
		if ( runner != null ) {
			this.runner = runner;
		} else {
			throw new PropertyVetoException( "algorithm runner can't be null",
				new PropertyChangeEvent( this, "runner", this.runner, runner )
			);
		}
	}

	/**
	 * @see org.evolvica.engine.IAlgorithm#setValidator(org.evolvica.engine.IValidator)
	 */
	public void setValidator( IValidator validator ) {
		this.validator = validator;
	}

	/**
	 * @see org.evolvica.engine.IAlgorithm#getMetadata(java.lang.Object, java.lang.String)
	 */
	public Metadata getMetadata( Object source, String path ) {
		return metadata.getValue( source, path );
	}

	/**
	 * @see org.evolvica.engine.IAlgorithm#restoreMetadata(org.evolvica.util.IMetadataProvider)
	 */
	public void restoreMetadata( IMetadataProvider provider ) {
		metadata.restore( provider );
	}

	/**
	 * @see org.evolvica.engine.IAlgorithm#serializeMetadata(org.evolvica.util.IMetadataSerializer)
	 */
	public void serializeMetadata( IMetadataSerializer serializer ) {
		metadata.serialize( serializer );
	}

	/**
	 * @see org.evolvica.engine.IAlgorithm#setMetadata(java.lang.Object, org.evolvica.util.Metadata)
	 */
	public void setMetadata( Object source, Metadata data ) {
		metadata.setValue( source, data );
	}

	/**
	 * @see org.evolvica.engine.IAlgorithm#addPropertyChangeListener(java.beans.PropertyChangeListener)
	 */
	public void addPropertyChangeListener( PropertyChangeListener listener ) {
		if ( listener != null ) {
			propertyChangeListeners.add( listener );
		}
	}
	
	/**
	 * @see org.evolvica.engine.IAlgorithm#addMetadataChangeListener(java.beans.PropertyChangeListener)
	 */
	public void addMetadataChangeListener( PropertyChangeListener listener ) {
		if ( listener != null ) {
			metadata.addChangeListener( listener );
		}
	}
	
	/**
	 * @see org.evolvica.engine.IAlgorithm#removePropertyChangeListener(java.beans.PropertyChangeListener)
	 */
	public void removePropertyChangeListener( PropertyChangeListener listener ) {
		if ( listener != null ) {
			propertyChangeListeners.remove( listener );
		}
	}
	
	/**
	 * @see org.evolvica.engine.IAlgorithm#removeMetadataChangeListener(java.beans.PropertyChangeListener)
	 */
	public void removeMetadataChangeListener( PropertyChangeListener listener ) {
		if ( listener != null ) {
			metadata.removeChangeListener( listener );
		}
	}
	
	/**
	 * @see org.evolvica.engine.IAlgorithm#firePropertyChangeEvent(java.beans.PropertyChangeEvent)
	 */
	public void firePropertyChangeEvent( PropertyChangeEvent event ) {
		for ( Iterator it = propertyChangeListeners.iterator(); it.hasNext(); ) {
			((PropertyChangeListener) it.next()).propertyChange( event );
		}
	}
	
}
