/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:21 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: AbstractElement.java,v $
 *
 */
package org.evolvica.engine;

import gnu.trove.THashSet;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;

import org.evolvica.util.Metadata;


/** Abstract algorithm element.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:58:21 $
 */
abstract public class AbstractElement implements IElement {
	
	private THashSet propertyChangeListeners = new THashSet();
	
	/**
	 * The algorithm this element belongs to.
	 */
	protected IAlgorithm algorithm = null;
	
	/**
	 * Incoming connections.
	 * 
	 */
	protected IConnection[] incoming = null;
	
	/**
	 * Outgoing connections.
	 * 
	 */
	protected IConnection[] outgoing = null;
	
	/**
	 * Input data.
	 * 
	 */
	protected Object input[] = null;
	
	/**
	 * Output data.
	 * 
	 */
	protected Object output[] = null;
	
	/**
	 * Constructor.
	 * 
	 * @param incomingSlots number of input slots
	 * @param outgoingSlots number of output slots
	 */
	public AbstractElement( int incomingSlots, int outgoingSlots ) {
		this.incoming = new IConnection[incomingSlots];
		this.outgoing = new IConnection[outgoingSlots];
		this.input = new Object[incomingSlots];
		this.output = new Object[outgoingSlots];
	}
	
	/**
	 * @see org.evolvica.engine.IElement#getAlgorithm()
	 */
	public IAlgorithm getAlgorithm() {
		return algorithm;
	}

	/**
	 * @see org.evolvica.engine.IElement#setAlgorithm(org.evolvica.engine.IAlgorithm)
	 */
	public void setAlgorithm( IAlgorithm algorithm ) {
		this.algorithm = algorithm;
	}
	
	/**
	 * @see org.evolvica.engine.IElement#getMetadata(java.lang.String)
	 */
	public Metadata getMetadata( String path ) {
		return algorithm.getMetadata( this, path );
	}

	/**
	 * @see org.evolvica.engine.IElement#setMetadata(org.evolvica.util.Metadata)
	 */
	public void setMetadata( Metadata data ) {
		if ( data != null ) {
			algorithm.setMetadata( this, data );
		}
	}

	/**
	 * @see org.evolvica.engine.IElement#incomingSize()
	 */
	public int incomingSize() {
		return incoming.length;
	}
	
	/**
	 * @see org.evolvica.engine.IElement#outgoingSize()
	 */
	public int outgoingSize() {
		return outgoing.length;
	}

	/**
	 * @see org.evolvica.engine.IElement#getIncoming(int)
	 */
	public IConnection getIncoming( int index ) {
		return incoming[index];
	}
	
	/**
	 * @see org.evolvica.engine.IElement#getIncomingIndex(org.evolvica.engine.IConnection)
	 */
	public int getIncomingIndex( IConnection connection ) {
		for ( int i = 0; i < incoming.length; i++ ) {
			if ( incoming[i] == connection ) {
				return i;
			}
		}
		return -1;
	}
	
	/**
	 * @see org.evolvica.engine.IElement#setIncoming(org.evolvica.engine.IConnection, int)
	 */
	public void setIncoming( IConnection connection, int index ) {
		if ( connection != null && index > -1 && index < incoming.length ) {
			if ( incoming[index] != null ) {
				algorithm.removeConnection( incoming[index] );
			}
			incoming[index] = connection;
			connection.setTarget( this );
			firePropertyChangeEvent( new PropertyChangeEvent( this, Events.INCOMING_CONNECTED, null, connection ) );
		}
	}
	
	/**
	 * @see org.evolvica.engine.IElement#getOutgoing(int)
	 */
	public IConnection getOutgoing( int index ) {
		return outgoing[index];
	}
	
	/**
	 * @see org.evolvica.engine.IElement#getOutgoingIndex(org.evolvica.engine.IConnection)
	 */
	public int getOutgoingIndex( IConnection connection ) {
		for ( int i = 0; i < outgoing.length; i++ ) {
			if ( outgoing[i] == connection ) {
				return i;
			}
		}
		return -1;
	}
	
	/**
	 * @see org.evolvica.engine.IElement#removeConnection(org.evolvica.engine.IConnection)
	 */
	public void removeConnection( IConnection connection ) {
		if ( connection != null ) {
			for ( int i = 0; i < incoming.length; i++ ) {
				if ( incoming[i] == connection ) {
					incoming[i] = null;
					connection.setTarget( null );
					// IMPROVE: workaround for the GUI: send the old slot number as the "oldValue" of the event
					firePropertyChangeEvent( new PropertyChangeEvent( this, Events.INCOMING_DISCONNECTED, new Integer( i ), connection ) );
					return;
				}
			}
			for ( int i = 0; i < outgoing.length; i++ ) {
				if ( outgoing[i] == connection ) {
					outgoing[i] = null;
					connection.setSource( null );
					// IMPROVE: workaround for the GUI: send the old slot number as the "oldValue" of the event
					firePropertyChangeEvent( new PropertyChangeEvent( this, Events.OUTGOING_DISCONNECTED, new Integer( i ), connection ) );
					return;
				}
			}
		}
	}
	
	/**
	 * @see org.evolvica.engine.IElement#setOutgoing(org.evolvica.engine.IConnection, int)
	 */
	public void setOutgoing( IConnection connection, int index ) {
		if ( connection != null && index > -1 && index < outgoing.length ) {
			if ( outgoing[index] != null ) {
				algorithm.removeConnection( outgoing[index] );
			}
			outgoing[index] = connection;
			connection.setSource( this );
			firePropertyChangeEvent( new PropertyChangeEvent( this, Events.OUTGOING_CONNECTED, null, connection ) );
		}
	}
	
	/**
	 * Sets the input data.
	 * 
	 * @param input input data
	 */
	public void setInput( Object[] input ) {
		// hmm?
	}
	
	/**
	 * Gets the output data
	 * 
	 * @return output data
	 */
	public Object[] getOutput() {
		return output;
	}

	/**
	 * @see org.evolvica.engine.IElement#addPropertyChangeListener(java.beans.PropertyChangeListener)
	 */
	public void addPropertyChangeListener( PropertyChangeListener listener ) {
		if ( listener != null ) {
			propertyChangeListeners.add( listener );
		}
	}
	
	/**
	 * @see org.evolvica.engine.IElement#removePropertyChangeListener(java.beans.PropertyChangeListener)
	 */
	public void removePropertyChangeListener( PropertyChangeListener listener ) {
		if ( listener != null ) {
			propertyChangeListeners.remove( listener );
		}
	}
	
	/**
	 * @see org.evolvica.engine.IElement#firePropertyChangeEvent(java.beans.PropertyChangeEvent)
	 */
	public void firePropertyChangeEvent( PropertyChangeEvent event ) {
		for ( Iterator it = propertyChangeListeners.iterator(); it.hasNext(); ) {
			((PropertyChangeListener) it.next()).propertyChange( event );
		}
	}
	
}
