/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:21 $
 * $Revision: 1.11 $
 * $State: Exp $
 * $RCSfile: AlgorithmBuilder.java,v $
 *
 */
 
package org.evolvica.engine;

import java.beans.PropertyVetoException;
import java.io.*;
import java.util.LinkedList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.beanutils.ConvertUtils;
import org.apache.commons.beanutils.Converter;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang.StringUtils;
import org.evolvica.util.*;
import org.w3c.dom.*;
import org.xml.sax.SAXException;

/** Algorithm builder.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.11 $ $Date: 2004/05/26 11:58:21 $
 */
public final class AlgorithmBuilder {
	
	private DocumentBuilder builder = null;
	
	private IAlgorithm algorithm = null;
	
	private ClassLoader loader = ClassLoader.getSystemClassLoader();
	
	private XMLMetadataProvider provider = new XMLMetadataProvider();
	
	/**
	 * Constructor.
	 * 
	 */
	public AlgorithmBuilder() {
		try {
			builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		} catch ( ParserConfigurationException pcx ) {
			EVLog.error( "error creating XML document builder", pcx );
		}
	}
	
	/**
	 * Builds an algorithm from the resource at the given URI.
	 * 
	 * @param uri URI pointing to an algorithm description.
	 * @return algorithm built from the given description
	 * @throws IOException thrown if access to the given description fails
	 * @throws SAXException thrown if the given description is invalid
	 * @throws SerializationException thrown in case the deserialization of the XML document fails
	 */
	public IAlgorithm build( String uri ) throws SerializationException, SAXException, IOException {
		return build( builder.parse( uri ), null );
	}
	
	/**
	 * Builds an algorithm from the resource at the given URI.
	 * 
	 * @param uri URI pointing to an algorithm description
	 * @param loader classloader to use for instantiating the classes
	 * @return algorithm built from the given description
	 * @throws IOException thrown if access to the given description fails
	 * @throws SAXException thrown if the given description is invalid
	 * @throws SerializationException thrown in case the deserialization of the XML document fails
	 */
	public IAlgorithm build( String uri, ClassLoader loader ) throws SerializationException, SAXException, IOException {
		return build( builder.parse( uri ), loader );
	}
	
	/**
	 * Builds an algorithm from the given inputstream.
	 * 
	 * @param stream input stream
	 * @return algorithm built from the given input stream
	 * @throws IOException thrown if access to the given description fails
	 * @throws SAXException thrown if the given description is invalid
	 * @throws SerializationException thrown in case the deserialization of the XML document fails
	 */
	public IAlgorithm build( InputStream stream ) throws SerializationException, SAXException, IOException {
		return build( builder.parse( stream ) );
	}
	
	/**
	 * Builds an algorithm from the given inputstream.
	 * 
	 * @param stream input stream
	 * @param loader classloader to use for instantiating the classes
	 * @return algorithm built from the given input stream
	 * @throws IOException thrown if access to the given description fails
	 * @throws SAXException thrown if the given description is invalid
	 * @throws SerializationException thrown in case the deserialization of the XML document fails
	 */
	public IAlgorithm build( InputStream stream, ClassLoader loader ) throws SerializationException, SAXException, IOException {
		return build( builder.parse( stream ), loader );
	}
	
	/**
	 * Reads an algorithm description but does not build the algorithm.
	 * 
	 * @param stream input stream to read the description from
	 * @return XML document containing the algorithm description
	 * @throws IOException thrown if access to the given description fails
	 * @throws SAXException thrown if the given description is invalid
	 */
	public Document buildDocument( InputStream stream ) throws IOException, SAXException {
		return builder.parse( stream );
	}
	
	/**
	 * Builds an algorithm from a XML document.
	 * 
	 * @param document XML document containing an algorithm description
	 * @return built algorithm
	 * @throws SerializationException thrown in case the deserialization of the XML document fails
	 */
	public IAlgorithm build( Document document ) throws SerializationException {
		return build( document, null );
	}
	
	/**
	 * Builds an algorithm from a XML document.
	 * 
	 * @param document XML document containing an algorithm description
	 * @param loader class loader to use for instantiating the classes
	 * @return built algorithm
	 * @throws SerializationException thrown in case the deserialization of the XML document fails
	 */
	public IAlgorithm build( Document document, ClassLoader loader ) throws SerializationException {
		Element element = document.getDocumentElement();
		if ( element.getNodeName().equals( "algorithm" ) ) {
			String algorithmVersion = element.getAttribute( "version" );
			if ( algorithmVersion.equals( "0.1" ) ) {
				return build_0_1( document, loader );
			} else {
				return build_0_2( document, loader );
			}
		}
		return null;
	}
	
	private IAlgorithm build_0_1( Document document, ClassLoader loader ) {
		if ( loader != null ) {
			this.loader = loader;
		}
		Element element = document.getDocumentElement();
		if ( element.getNodeName().equals( "algorithm" ) ) {
			String algorithmName = element.getAttribute( "name" );
			String algorithmVersion = element.getAttribute( "version" ); 
			algorithm = new TemplateAlgorithm( algorithmName );
			scanStructureNode( (Element) element.getElementsByTagName( "structure" ).item( 0 ) );
			scanConnectivityNode( (Element) element.getElementsByTagName( "connectivity" ).item( 0 ) );
			NodeList children = element.getChildNodes();
			for ( int i = 0; i < children.getLength(); i++ ) {
				Node n = children.item( i );
				if ( n.getNodeType() == Node.ELEMENT_NODE ) {
					if ( ! (n.getNodeName().equals( "engine" ) || n.getNodeName().equals( "structure" ) || n.getNodeName().equals( "connectivity" )) ) {
						scanMetadataNode( (Element) n );
					}
				}
			}
		}
		return algorithm;
	}
	
	private IAlgorithm build_0_2( Document document, ClassLoader loader ) throws SerializationException {
		if ( loader != null ) {
			this.loader = loader;
		}
		Element element = document.getDocumentElement();
		if ( element.getNodeName().equals( "algorithm" ) ) {
			String algorithmName = element.getAttribute( "name" );
			String algorithmVersion = element.getAttribute( "version" ); 
			algorithm = new TemplateAlgorithm( algorithmName );
			scanEngineNode( (Element) element.getElementsByTagName( "engine" ).item( 0 ) );
			scanStructureNode_0_2( (Element) element.getElementsByTagName( "structure" ).item( 0 ) );
			scanConnectivityNode( (Element) element.getElementsByTagName( "connectivity" ).item( 0 ) );
			NodeList children = element.getChildNodes();
			for ( int i = 0; i < children.getLength(); i++ ) {
				Node n = children.item( i );
				if ( n.getNodeType() == Node.ELEMENT_NODE ) {
					if ( ! (n.getNodeName().equals( "engine" ) || n.getNodeName().equals( "structure" ) || n.getNodeName().equals( "connectivity" )) ) {
						scanMetadataNode( (Element) n );
					}
				}
			}
		}
		return algorithm;
	}
	
	private void scanEngineNode( Element engineNode ) throws SerializationException {
		NodeList children = engineNode.getChildNodes();
		for ( int i = 0; i < children.getLength(); i++ ) {
			Node child = children.item( i );
			if ( child.getNodeType() == Node.ELEMENT_NODE && child.getNodeName().equals( "runner" ) ) {
				Object runner = SerialUtils.deserialize( (Element) child, loader );
				try {
					algorithm.setRunner( (IRunner) runner );
				} catch ( PropertyVetoException pvx ) {
					pvx.printStackTrace();
				}
			}
			if ( child.getNodeType() == Node.ELEMENT_NODE && child.getNodeName().equals( "validator" ) ) {
				Object validator = SerialUtils.deserialize( (Element) child, loader );
				algorithm.setValidator( (IValidator) validator );
			}
		}
	}
	
	private void scanStructureNode_0_2( Element structureNode ) throws SerializationException {
		NodeList children = structureNode.getChildNodes();
		for ( int i = 0; i < children.getLength(); i++ ) {
			Node child = children.item( i );
			if ( child.getNodeType() == Node.ELEMENT_NODE && child.getNodeName().equals( "operator" ) ) {
				Element operatorNode = (Element) child;
				IOperator operator = (IOperator) SerialUtils.deserialize( operatorNode, loader );
				algorithm.addElement( ElementFactory.newElement( operator ) );
			}
		}
	}
	
	private void scanStructureNode( Element structureNode ) {
		NodeList children = structureNode.getChildNodes();
		for ( int i = 0; i < children.getLength(); i++ ) {
			Node child = children.item( i );
			if ( child.getNodeType() == Node.ELEMENT_NODE && child.getNodeName().equals( "operator" ) ) {
				scanOperatorNode( (Element) child );
			}
		}
	}
	
	private void scanOperatorNode( Element operatorNode ) {
		String name = operatorNode.getAttribute( "name" );
		String id = operatorNode.getAttribute( "id" );
		String clazz = operatorNode.getAttribute( "class" );
		try {
			IOperator operator = (IOperator) loader.loadClass( clazz ).newInstance();
			operator.setName( name );
			operator.setId( id );
			NodeList children = operatorNode.getChildNodes();
			for ( int i = 0; i < children.getLength(); i++ ) {
				Node child = children.item( i );
				if ( child.getNodeType() == Node.ELEMENT_NODE && child.getNodeName().equals( "property" ) ) {
					scanPropertyNode( operator, (Element) child );
				}
			}
			algorithm.addElement( ElementFactory.newElement( operator ) );
		} catch ( Exception ex ) {
			ex.printStackTrace();
		}
	}
	
	private void scanPropertyNode( IOperator operator, Element propertyNode ) {
		String name = propertyNode.getAttribute( "name" );
		String value = propertyNode.getAttribute( "value" );
		String convertible = propertyNode.getAttribute( "convertible" );
		if ( value != null ) {
			try {
				Class type = PropertyUtils.getPropertyType( operator, name );
				if ( convertible.equals( "string" ) ) {
					Converter converter = ConvertUtils.lookup( type );
					Object propertyValue = converter.convert( type, value );
					PropertyUtils.setProperty( operator, name, propertyValue );
				} else if ( convertible.equals( "file") ) {
					if ( ! value.equals( "null" ) ) {
						File file = new File( value );
						PropertyUtils.setProperty( operator, name, file );
					}
				} else if ( convertible.equals( "class" ) ) {
					if ( StringUtils.isNotEmpty( value ) ) {
						Object object = loader.loadClass( value ).newInstance();
						PropertyUtils.setProperty( operator, name, object );
					}
				} else {
					EVLog.warn( "ignoring non-convertible property " + name );
				}
			} catch ( Exception ex ) {
				EVLog.error( "error accessing property " + name + " in operator " + operator.getName(), ex );
			}
		}
	}
	
	private void scanConnectivityNode( Element networkNode ) {
		NodeList children = networkNode.getChildNodes();
		for ( int i = 0; i < children.getLength(); i++ ) {
			Node child = children.item( i );
			if ( child.getNodeType() == Node.ELEMENT_NODE && child.getNodeName().equals( "connection" ) ) {
				scanConnectionNode( (Element) child );
			}
		}
	}
	
	private void scanConnectionNode( Element connectionNode ) {
		String sourceID = connectionNode.getAttribute( "sourceid" );
		String sourceSlot = connectionNode.getAttribute( "sourceslot" );
		String targetID = connectionNode.getAttribute( "targetid" );
		String targetSlot = connectionNode.getAttribute( "targetslot" );
		IElement source = algorithm.getElementById( sourceID );
		IElement target = algorithm.getElementById( targetID );
		algorithm.connect( source, Integer.parseInt( sourceSlot ), target, Integer.parseInt( targetSlot) );
	}
	
	private void scanMetadataNode( Element metadataNode ) {
		provider.storeMetadata( metadataNode );
		algorithm.restoreMetadata( provider );
		provider.clear();
	}
	
	private class XMLMetadataProvider implements IMetadataProvider {
		
		private LinkedList sources = new LinkedList();
		
		private LinkedList values = new LinkedList();
		
		private void clear() {
			sources.clear();
			values.clear();
		}
		
		public Object[] getSources() {
			return sources.toArray();
		}

		public Metadata[] getValues() {
			return (Metadata[]) values.toArray( new Metadata[values.size()] );
		}
		
		// IMPROVE: only handles the three-level (view) stuff at the moment
		private void storeMetadata( Element node ) {
			String level0 = node.getNodeName();
			NodeList level1List = node.getChildNodes();
			for ( int i = 0; i < level1List.getLength(); i++ ) {
				if ( level1List.item( i ).getNodeType() == Node.ELEMENT_NODE ) {
					Element level1Element = (Element) level1List.item( i );
					String level1 = level1Element.getNodeName();
					String id = level1Element.getAttribute( "id" );
					if ( StringUtils.isNotEmpty( id ) ) {
						IElement e = algorithm.getElementById( id );
						if ( e != null ) {
							NamedNodeMap map = level1Element.getAttributes();
							for ( int j = 0; j < map.getLength(); j++ ) {
								Attr att = (Attr) map.item( j );
								if ( ! att.getName().equals( "id" ) ) {
									sources.add( e );
									values.add( new Metadata( level0 + Metadata.SEPARATOR + level1 + Metadata.SEPARATOR + att.getName(), att.getValue() ) );
								}
							}
						}
					}
				}
			}
		}
		
	}
	
}
