
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:22 $
 * $Revision: 1.13 $
 * $State: Exp $
 * $RCSfile: AlgorithmWriter.java,v $
 *
 */
 
package org.evolvica.engine;

import java.io.OutputStream;
import java.util.Iterator;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.commons.lang.StringUtils;
import org.evolvica.util.*;
import org.evolvica.util.Convert;
import org.evolvica.util.SerialUtils;
import org.evolvica.util.SerializationException;
import org.w3c.dom.*;

/** Algorithm writer.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.13 $ $Date: 2004/05/26 11:58:22 $
 */
public class AlgorithmWriter {
	
	private IAlgorithm algorithm = null;
	
	private Document document = null;
	
	private XMLMetadataSerializer serializer = new XMLMetadataSerializer();
		
	/**
	 * Writes an XML algorithm document to an output stream.
	 * 
	 * @param document document to write
	 * @param stream output stream to write to
	 * @throws TransformerConfigurationException thrown if XML transformation fails
	 * @throws TransformerException thrown if XML transformation fails
	 */
	public void write( Document document, OutputStream stream ) throws TransformerConfigurationException, TransformerException {
		Transformer transformer = TransformerFactory.newInstance().newTransformer();
		transformer.setOutputProperty( OutputKeys.INDENT, "yes" );
		Source source = new DOMSource( document.getDocumentElement() );
		Result result = new StreamResult( stream );
		transformer.transform( source, result );
	}
	
	/**
	 * Writes an algorithm to an output stream.
	 * 
	 * @param algorithm algorithm to write
	 * @param stream output stream to write to
	 * @throws SerializationException thrown in case the XML document could not be created
	 * @throws TransformerConfigurationException thrown if XML transformation fails
	 * @throws TransformerException thrown if XML transformation fails
	 */
	public void write( IAlgorithm algorithm, OutputStream stream ) throws SerializationException, TransformerConfigurationException, TransformerException {
		write( createDocument( algorithm ), stream );
	}
	
	/**
	 * Creates a XML document from a given algorithm.
	 * 
	 * @param algorithm algorithm to be transformed
	 * @return XML document containing the algorithm description
	 * @throws SerializationException thrown in case the document could not be created
	 */
	public Document createDocument( IAlgorithm algorithm ) throws SerializationException {
		this.algorithm = algorithm;
		try {
			DOMImplementation impl = DocumentBuilderFactory.newInstance().newDocumentBuilder().getDOMImplementation();
			this.document = impl.createDocument( "http://www.evolvica.org/namespace", "algorithm", null );
		} catch ( ParserConfigurationException pcx ) {
			pcx.printStackTrace();
		}
		document.getDocumentElement().setAttribute( "name", algorithm.getName() );
		document.getDocumentElement().setAttribute( "version", "0.2" );
		Element engineNode = document.createElement( "engine" );
		Element structureNode = document.createElement( "structure" );
		Element connectivityNode = document.createElement( "connectivity" );
		document.getDocumentElement().appendChild( engineNode );
		document.getDocumentElement().appendChild( structureNode );
		document.getDocumentElement().appendChild( connectivityNode );
		createEngine( engineNode );
		createStructure( structureNode );
		createConnectivity( connectivityNode );
		createMetadata( document.getDocumentElement() );
		return document;
	}
	
	private void createEngine( Element engineNode ) throws SerializationException {
		// save runner
		IRunner runner = algorithm.getRunner();
		Element runnerNode = document.createElement( "runner" );
		if ( runner != null ) {
			runnerNode.setAttribute( "class", Convert.toClassname( runner ) );
			SerialUtils.serialize( runnerNode, runner );
			engineNode.appendChild( runnerNode );
		}
		// save validator
		IValidator validator = algorithm.getValidator();
		Element validatorNode = document.createElement( "validator" );
		if ( validator != null ) {
			validatorNode.setAttribute( "class", Convert.toClassname( validator ) );
			SerialUtils.serialize( validatorNode, validator );
			engineNode.appendChild( validatorNode );
		}
	}
	
	private void createStructure( Element structureNode ) throws SerializationException {
		for (Iterator it = algorithm.iterateElements(); it.hasNext(); ) {
			IOperator operator = ((IElement) it.next()).getOperator();
			Element operatorNode = document.createElement( "operator" );
			operatorNode.setAttribute( "class", Convert.toClassname( operator ) );
			structureNode.appendChild( operatorNode );
			SerialUtils.serialize( operatorNode, operator );
		}
	}
	
	private void createConnectivity( Element connectivityNode ) {
		for (Iterator it = algorithm.iterateConnections(); it.hasNext(); ) {
			IConnection connection = (IConnection) it.next();
			IElement source = connection.getSource();
			IElement target = connection.getTarget();
			int sourceSlot = findSourceSlot( source, connection );
			int targetSlot = findTargetSlot( target, connection );
			Element connectionNode = document.createElement( "connection" );
			connectionNode.setAttribute( "sourceid", source.getOperator().getId() );
			connectionNode.setAttribute( "sourceslot", String.valueOf( sourceSlot ) );
			connectionNode.setAttribute( "targetid", target.getOperator().getId() );
			connectionNode.setAttribute( "targetslot", String.valueOf( targetSlot ) );
			connectivityNode.appendChild( connectionNode );
		}
	}
	
	private void createMetadata( Element documentNode ) {
		algorithm.serializeMetadata( serializer );
	}
	
	private int findSourceSlot( IElement element, IConnection connection ) {
		for ( int i = 0; i < element.outgoingSize(); i++ ) {
			if ( element.getOutgoing( i ) == connection ) {
				return i;
			}
		}
		return -1; // something is wrong, returning -1 should raise an exception later
	}
	
	private int findTargetSlot( IElement element, IConnection connection ) {
		for ( int i = 0; i < element.incomingSize(); i++ ) {
			if ( element.getIncoming( i ) == connection ) {
				return i;
			}
		}
		return -1; // something is wrong, returning -1 should raise an exception later
	}
	
	private class XMLMetadataSerializer implements IMetadataSerializer {
		
		// IMPROVE: the following serializer is not bullet-proof, it just secures backward compatibility
		public void serialize( Object source, Metadata value ) {
			if ( source instanceof IElement ) {
				Element docElement = AlgorithmWriter.this.document.getDocumentElement();
				String[] path = StringUtils.split( value.getPath(), Metadata.SEPARATOR );
				// only treat 3-element paths
				if ( path.length == 3 ) {
					// check for element level 0
					NodeList list = docElement.getElementsByTagName( path[0] );
					Element level0Element = null;
					if ( list.getLength() > 0 ) {
						level0Element = (Element) list.item( 0 );
					} else {
						level0Element = AlgorithmWriter.this.document.createElement( path[0] );
						docElement.appendChild( level0Element );
					}
					// check for element level 1
					list = level0Element.getElementsByTagName( path[1] );
					Element level1Element = null;
					for ( int i = 0; i < list.getLength(); i++ ) {
						String id = ((IElement) source).getOperator().getId();
						Element e = (Element) list.item( i );
						if ( e.getNodeName().equals( path[1] ) && e.getAttribute( "id" ).equals( id ) ) {
							level1Element = (Element) list.item( i );
							break;
						}
					}
					if ( level1Element == null ) {
						level1Element = AlgorithmWriter.this.document.createElement( path[1] );
						level1Element.setAttribute( "id", ((IElement) source).getOperator().getId() );
						level0Element.appendChild( level1Element );
					}
					// finally set the attribute
					level1Element.setAttribute( path[2], value.getValue() );
				}
			}
		}
		
	}
	
}
