/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:22 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: AttributeDescriptor.java,v $
 *
 */
package org.evolvica.engine;

import java.beans.IndexedPropertyDescriptor;
import java.beans.PropertyDescriptor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.apache.commons.beanutils.PropertyUtils;
import org.evolvica.util.Convert;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Descriptor of an attribute/property of an operator.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:58:22 $
 * @since 0.5.0
 */
public class AttributeDescriptor extends AbstractDescriptor {
	
	private boolean bound = false;
	private boolean constrained = false;
	
	// transient properties;: these need to be calculated at runtime
	private transient Class indexedPropertyType = null;
	private transient Method indexedReadMethod = null;
	private transient Method indexedWriteMethod = null;
	private transient Class propertyType = null;
	private transient Method readMethod = null;
	private transient Method writeMethod = null;
	
	/**
	 * Constructor.
	 * 
	 * @param name name of the attribute
	 */
	public AttributeDescriptor( String name ) {
		super( name );
	}
	
	/**
	 * Constructor.
	 * Creates a new descriptor describing the given attribute/property.
	 * 
	 * @param name name of the attribute
	 * @param displayName display name of the attribute
	 * @param bean bean from which to retrieve the property
	 * @throws IllegalAccessException thrown in case the property cannot be retrieve
	 * @throws InvocationTargetException thrown in case the property cannot be retrieve
	 * @throws NoSuchMethodException thrown in case the property cannot be retrieve
	 */
	public AttributeDescriptor( String name, String displayName, Object bean ) throws IllegalAccessException, InvocationTargetException, NoSuchMethodException {
		this( PropertyUtils.getPropertyDescriptor( bean, name ) );
		setDisplayName( displayName );
	}
	
	/**
	 * Copy Constructor.
	 * Creates a new descriptor from an existing property descriptor.
	 * 
	 * @param descriptor property descriptor
	 */
	public AttributeDescriptor( PropertyDescriptor descriptor ) {
		super( descriptor.getName() );
		setBound( descriptor.isBound() );
		setConstrained( descriptor.isConstrained() );
		setDisplayName( descriptor.getDisplayName() );
		setExpert( descriptor.isExpert() );
		setHidden( descriptor.isHidden() );
		setPreferred( descriptor.isPreferred() );
		// add transient information
		this.propertyType = descriptor.getPropertyType();
		this.readMethod = descriptor.getReadMethod();
		this.writeMethod = descriptor.getWriteMethod();
		if ( descriptor instanceof IndexedPropertyDescriptor ) {
			IndexedPropertyDescriptor ipd = (IndexedPropertyDescriptor) descriptor;
			this.indexedPropertyType = ipd.getIndexedPropertyType();
			this.indexedReadMethod = ipd.getIndexedReadMethod();
			this.indexedWriteMethod = ipd.getIndexedWriteMethod();
		}
	}
	
	/**
	 * Clones this attribute descriptor.
	 * 
	 * @return cloned descriptor
	 */
	public Object clone() {
		AttributeDescriptor clone = new AttributeDescriptor( getName() );
		clone.setBound( isBound() );
		clone.setConstrained( isConstrained() );
		clone.setDisplayName( getDisplayName() );
		clone.setExpert( isExpert() );
		clone.setHidden( isHidden() );
		clone.setPreferred( isPreferred() );
		clone.setShortDescription( getShortDescription() );
		// set transient fields
		clone.indexedPropertyType = indexedPropertyType;
		clone.indexedReadMethod = indexedReadMethod;
		clone.indexedWriteMethod = indexedWriteMethod;
		clone.propertyType = propertyType;
		clone.readMethod = readMethod;
		clone.writeMethod = writeMethod;
		return clone;
	}
	
	/**
	 * Returns the indexed property type (if there is any)
	 * 
	 * @return indexed property type or null
	 */
	public Class getIndexedPropertyType() {
		return indexedPropertyType;
	}

	/**
	 * Returns the read method for indexed properties.
	 * 
	 * @return read method or null
	 */
	public Method getIndexedReadMethod() {
		return indexedReadMethod;
	}

	/**
	 * Returns the write method for indexed properties.
	 * 
	 * @return write method or null
	 */
	public Method getIndexedWriteMethod() {
		return indexedWriteMethod;
	}

	/**
	 * Returns the property type.
	 * 
	 * @return property type
	 */
	public Class getPropertyType() {
		return propertyType;
	}

	/**
	 * Returns the read method for the property.
	 * 
	 * @return read method
	 */
	public Method getReadMethod() {
		return readMethod;
	}

	/**
	 * Returns the write method for this property.
	 * 
	 * @return write method or null
	 */
	public Method getWriteMethod() {
		return writeMethod;
	}

	/**
	 * Returns whether this property is bound.
	 * 
	 * @return bound flag
	 */
	public boolean isBound() {
		return bound;
	}

	/**
	 * Returns whether this property is constrained.
	 * 
	 * @return contrained flag
	 */
	public boolean isConstrained() {
		return constrained;
	}

	/**
	 * Returns whether this property has already been resolved.
	 * 
	 * @return true if the property has been resolved, false instead
	 */
	public boolean isResolved() {
		if ( propertyType != null && readMethod != null ) {
			return true;
		}
		return false;
	}
	
	/**
	 * Attempts to resolve this property.
	 * 
	 * @param parent parent class
	 * @deprecated use DescriptorResolver instead of this method
	 */
	public void resolve( Class parent ) {
		DescriptorResolver.resolve( this, parent );
	}
	
	/**
	 * Loads the descriptor from an XML element.
	 * 
	 * @param element XML element
	 * @throws IllegalArgumentException thrown in case the descriptor cannot be loaded
	 * @deprecated use DescriptorBuilder to create XML elements from attribute descriptors
	 */
	public void load( Element element ) throws IllegalArgumentException {
		DescriptorBuilder.updateDescriptor( element, this );
	}
	
	/**
	 * Sets the bound flag.
	 * 
	 * @param b bound flag
	 */
	public void setBound( boolean b ) {
		bound = b;
	}

	/**
	 * Sets the constrained flag.
	 * 
	 * @param b constrained flag
	 */
	public void setConstrained( boolean b ) {
		constrained = b;
	}
	
	/**
	 * Sets the indexed property type.
	 * 
	 * @param class1 indexed property type
	 */
	public void setIndexedPropertyType( Class class1 ) {
		indexedPropertyType = class1;
	}

	/**
	 * Sets the read method for indexed properties.
	 * 
	 * @param method read method
	 */
	public void setIndexedReadMethod( Method method ) {
		indexedReadMethod = method;
	}

	/**
	 * Sets the write method for index porperties.
	 * 
	 * @param method write method
	 */
	public void setIndexedWriteMethod( Method method ) {
		indexedWriteMethod = method;
	}

	/**
	 * Sets the property type.
	 * 
	 * @param class1 property type
	 */
	public void setPropertyType( Class class1 ) {
		propertyType = class1;
	}

	/**
	 * Sets the read method for the property.
	 * 
	 * @param method read method
	 */
	public void setReadMethod( Method method ) {
		readMethod = method;
	}

	/**
	 * Sets the write method for the property.
	 * 
	 * @param method write method
	 */
	public void setWriteMethod(Method method) {
		writeMethod = method;
	}
	
	/**
	 * Saves the descriptor into an XML element.
	 * 
	 * @param doc XML document to use for element creation
	 * @return created XML element
	 * @deprecated use DescriptorWriter for saving
	 */
	public Element save( Document doc ) {
		return DescriptorWriter.createAttributeNode( doc, this );
	}
	
	/**
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo( Object o ) {
		return getName().compareTo( ((AttributeDescriptor) o).getName() );
	}
	
	/**
	 * Returns a string representation.
	 * 
	 * @return string representation
	 */
	public String toString() {
		return getDisplayName() + ", type " + Convert.toClassname( getPropertyType() );
	}
	
}
