/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:22 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: DefaultAlgorithmRunner.java,v $
 *
 */
package org.evolvica.engine;

import java.util.*;
import java.util.Date;
import java.util.HashSet;
import java.util.LinkedList;

import org.evolvica.util.EVLog;
import org.evolvica.util.NotImplementedException;

/** Default implementation of an algorithm runner.
 * This runner executes an algorithm inside a single thread.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:22 $
 */
public class DefaultAlgorithmRunner implements IRunner {
	
	/**
	 * The algorithm to be executed.
	 * 
	 */
	protected IAlgorithm algorithm;
	
	/**
	 * Algorithm thread.
	 * 
	 */
	protected Thread algorithmThread;
	
	/**
	 * Set containing the sources of the algorithm.
	 * 
	 */
	protected HashSet sourceSet = new HashSet();
	
	/**
	 * Set containing the operators of the algorithm.
	 * 
	 */
	protected HashSet operatorSet = new HashSet();
	
	/**
	 * Set containing the sinks of the algorithm.
	 * 
	 */
	protected HashSet sinkSet = new HashSet();
	
	/**
	 * Queue with active elements.
	 * 
	 */
	protected LinkedList activeQueue = new LinkedList();
	
	/**
	 * Runs the algorithm.
	 */
	public void run() {
		setupActiveQueue();
		traverseElements();
	}
	
	/**
	 * @see org.evolvica.engine.IRunner#start(org.evolvica.engine.IAlgorithm)
	 */
	public void start( IAlgorithm algorithm ) {
		this.algorithm = algorithm;
		try {
			EVLog.engine.info( "setting up algorithm" );
			algorithm.setup();
			if ( algorithm.getValidator() != null ) {
				IValidator validator = algorithm.getValidator();
				boolean isValid = validator.validate( algorithm );
				if ( isValid == false ) {
					EVLog.engine.error( "invalid algorithm: " + validator.getValidationResult()[0] );
					return;
				}
			} else {
				EVLog.engine.warn( "no algorithm validator set: cannot ensure correctness of algorithm structure" );
			}
		} catch ( Exception ex ) {
			ex.printStackTrace(); // TODO: handle algorithm setup exception
			return;
		}
		EVLog.engine.info( "starting algorithm at " + new Date() );
		algorithmThread = new Thread( this );
		algorithmThread.start();
		try {
			algorithmThread.join();
		} catch ( Exception ex ) {
			ex.printStackTrace(); // TODO: handle algorithm join exception
		}
		EVLog.engine.info( "algorithm finished at " + new Date() );
		try {
			algorithm.teardown();
		} catch ( Exception ex ) {
			ex.printStackTrace(); // TODO: handle algorithm teardown exception
		}
	}
	
	/**
	 * @see org.evolvica.engine.IRunner#stop()
	 */
	public void stop() {
		throw new NotImplementedException();
	}
	
	/**
	 * @see org.evolvica.engine.IRunner#resume()
	 */
	public void resume() {
		throw new NotImplementedException();
	}
	
	/**
	 * Sets up the queue.
	 */
	protected void setupActiveQueue() {
		// fill sets with elements
		for (Iterator it = algorithm.iterateElements(); it.hasNext(); ) {
			IElement element = (IElement) it.next();
			if ( element.getOperator() instanceof ISource ) {
				sourceSet.add( element );
				activeQueue.add( element ); // put sources in active queue
			} else if ( element.getOperator() instanceof ISink ) {
				sinkSet.add( element );
			} else {
				operatorSet.add( element );
			}
		}
	}

	/**
	 * Traverses active elements and activates them.
	 */
	protected void traverseElements() {
		while ( activeQueue.size() > 0 ) {
			IElement element = (IElement) activeQueue.getFirst();
			element.activate();
			for ( int i = 0; i < element.outgoingSize(); i++ ) {
				IConnection outgoing = element.getOutgoing( i );
				IElement target = outgoing.getTarget();
				if ( target.isActivatable() ) {
					activeQueue.add( target );
				}
			}
			activeQueue.removeFirst();
		}
	}
	
}
