/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:22 $
 * $Revision: 1.3 $
 * $State: Exp $
 * $RCSfile: DefaultValidator.java,v $
 * 
 */
package org.evolvica.engine;

import java.util.Iterator;
import java.util.LinkedList;


/** A default algorithm validator.
 *
 * @author Andreas Rummler
 * @since 0.6.0
 * @version $Revision: 1.3 $ $Date: 2004/05/26 11:58:22 $
 */
public class DefaultValidator implements IValidator {
	
	private LinkedList messages = new LinkedList();
	
	private boolean isValid = true;
	
	/**
	 * @see org.evolvica.engine.IValidator#getValidationResult()
	 */
	public String[] getValidationResult() {
		return (String[]) messages.toArray( new String[messages.size()] );
	}
	
	/**
	 * @see org.evolvica.engine.IValidator#validate(org.evolvica.engine.IAlgorithm)
	 */
	public boolean validate( IAlgorithm algorithm ) {
		// init
		isValid = true;
		messages.clear();
		// perform checks
		checkForSourceExistance( algorithm );
		checkForSinkExistance( algorithm );
		checkForOpenInputs( algorithm );
		checkForOpenOutputs( algorithm );
		return isValid;
	}
	
	private void checkForSourceExistance( IAlgorithm algorithm ) {
		int sources = 0;
		for ( Iterator it = algorithm.iterateElements(); it.hasNext(); ) {
			IOperator op = ((IElement) it.next()).getOperator();
			if ( op.descriptor().getInputSize() == 0 && op.descriptor().getOutputSize() > 0 ) {
				++sources;
			}
		}
		if ( sources == 0 ) {
			isValid = false;
			messages.add( "algorithm does not contain any source operators" );
		}
	}
	
	private void checkForSinkExistance( IAlgorithm algorithm ) {
		int sinks = 0;
		for ( Iterator it = algorithm.iterateElements(); it.hasNext(); ) {
			IOperator op = ((IElement) it.next()).getOperator();
			if ( op.descriptor().getInputSize() > 0 && op.descriptor().getOutputSize() == 0 ) {
				++sinks;
			}
		}
		if ( sinks == 0 ) {
			isValid = false;
			messages.add( "algorithm does not contain any sink operators" );
		}
	}
	
	private void checkForOpenInputs( IAlgorithm algorithm ) {
		for ( Iterator it = algorithm.iterateElements(); it.hasNext(); ) {
			IElement element = (IElement) it.next();
			for ( int i = 0; i < element.incomingSize(); i++ ) {
				if ( element.getIncoming( i ) == null ) {
					messages.add( "algorithm contains operators with open inputs" );
					isValid = false;
				}
			}
		}
	}
	
	private void checkForOpenOutputs( IAlgorithm algorithm ) {
		for ( Iterator it = algorithm.iterateElements(); it.hasNext(); ) {
			IElement element = (IElement) it.next();
			for ( int i = 0; i < element.outgoingSize(); i++ ) {
				if ( element.getOutgoing( i ) == null ) {
					messages.add( "algorithm contains operators with open outputs" );
					isValid = false;
				}
			}
		}
	}
	
}
