/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:23 $
 * $Revision: 1.3 $
 * $State: Exp $
 * $RCSfile: DescriptorBuilder.java,v $
 * 
 */
package org.evolvica.engine;

import java.io.IOException;
import java.io.InputStream;
import java.net.URI;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.evolvica.util.SerializationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/** Builder for loading operator descriptors.
 *
 * @author Andreas Rummler
 * @since 0.6.0
 * @version $Revision: 1.3 $ $Date: 2004/05/26 11:58:23 $
 */
public class DescriptorBuilder {
	
	/**
	 * Loads a descriptor from the given URI.
	 * 
	 * @param uri URI to load descriptor from
	 * @return created descriptor
	 * @throws SerializationException thrown in case the loading of the descriptor fails
	 */
	public static OperatorDescriptor build( URI uri ) throws SerializationException {
		try{
			InputStream stream = uri.toURL().openStream();
			OperatorDescriptor desc = build( stream );
			stream.close();
			return desc;
		} catch ( IOException iox ) {
			throw new SerializationException( "unable to open stream to given URL" );
		}
	}
	
	/**
	 * Loads a descriptor from the given input stream.
	 * 
	 * @param stream input stream to load descriptor from
	 * @return created descriptor
	 * @throws SerializationException thrown in case the loading of the descriptor fails
	 */
	public static OperatorDescriptor build( InputStream stream ) throws SerializationException {
		try {
			DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
			return build( builder.parse( stream ) );
		} catch ( Exception ex ) {
			throw new SerializationException( "unable to build descriptor from XML document", ex );
		}
	}
	
	/**
	 * Builds a descriptor from the given XML document.
	 * 
	 * @param document XML document to create the descriptor from
	 * @return created descriptor
	 * @throws SerializationException thrown in case the descriptor building fails
	 */
	public static OperatorDescriptor build( Document document ) throws SerializationException {
		Element docElement = document.getDocumentElement();
		if ( docElement.getNodeName().equals( "operator" ) ) {
			OperatorDescriptor descriptor = new OperatorDescriptor( docElement.getAttribute( "beanClass" ), null, null );
			descriptor.setCategory( docElement.getAttribute( "category" ) );
			descriptor.setDatatypeName( docElement.getAttribute( "datatype" ) );
			descriptor.setDisplayName( docElement.getAttribute( "displayName" ) );
			descriptor.setExpert( Boolean.valueOf( docElement.getAttribute( "expert" ) ).booleanValue() );
			descriptor.setHidden( Boolean.valueOf( docElement.getAttribute( "hidden" ) ).booleanValue() );
			descriptor.setInputSize( Integer.parseInt( docElement.getAttribute( "inputSize" ) ) );
			descriptor.setOutputSize( Integer.parseInt( docElement.getAttribute( "outputSize" ) ) );
			descriptor.setPreferred( Boolean.valueOf( docElement.getAttribute( "preferred" ) ).booleanValue() );
			descriptor.setShortDescription( docElement.getAttribute( "shortDescription" ) );
			descriptor.setVendor( docElement.getAttribute( "vendor" ) );
			for ( int i = 0; i < docElement.getChildNodes().getLength(); i++ ) {
				Node child = docElement.getChildNodes().item( i );
				if ( child.getNodeType() == Node.ELEMENT_NODE && child.getNodeName().equals( "property" ) ) {
					Element element = (Element) child;
					descriptor.addAttribute( buildAttributeDescriptor( element ) ); 
				}
			}
			return descriptor;
		} else {
			throw new SerializationException( "cannot create descriptor: invalid document type " + docElement.getNodeName() );
		}
	}
	
	private static AttributeDescriptor buildAttributeDescriptor( Element element ) {
		AttributeDescriptor descriptor = new AttributeDescriptor( element.getAttribute( "name" ) );
		descriptor.setDisplayName( element.getAttribute( "displayName" ) );
		descriptor.setExpert( Boolean.valueOf( element.getAttribute( "expert" ) ).booleanValue() );
		descriptor.setHidden( Boolean.valueOf( element.getAttribute( "hidden" ) ).booleanValue() );
		descriptor.setPreferred( Boolean.valueOf( element.getAttribute( "preferred" ) ).booleanValue() );
		descriptor.setShortDescription( element.getAttribute( "shortDescription" ) );
		return descriptor;
	}
	
	/**
	 * Updates an existing attribute descriptor with the information from the given XML element.
	 * This method is just for backward compatibility and will be removed in the future.
	 * 
	 * @param element XML element
	 * @param descriptor attribute descriptor to be updated
	 */
	public static void updateDescriptor( Element element, AttributeDescriptor descriptor ) {
		if ( element.getNodeName().equals( "property" ) ) {
			descriptor.setDisplayName( element.getAttribute( "displayName" ) );
			descriptor.setExpert( Boolean.valueOf( element.getAttribute( "expert" ) ).booleanValue() );
			descriptor.setHidden( Boolean.valueOf( element.getAttribute( "hidden" ) ).booleanValue() );
			descriptor.setPreferred( Boolean.valueOf( element.getAttribute( "preferred" ) ).booleanValue() );
			descriptor.setShortDescription( element.getAttribute( "shortDescription" ) );
		}
	}
	
}
