/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:23 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: DescriptorResolver.java,v $
 * 
 */
package org.evolvica.engine;

import java.beans.IndexedPropertyDescriptor;
import java.beans.PropertyDescriptor;
import java.io.File;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.SystemUtils;
import org.evolvica.core.GenotypeRepository;
import org.evolvica.util.ClassUtils;

/** Resolver for descriptors.
 *
 * @author Andreas Rummler
 * @since 0.6.0
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:58:23 $
 */
public class DescriptorResolver {
	
	private static PropertyDescriptor findDescriptor( Class clazz, String name ) {
		PropertyDescriptor[] pd = PropertyUtils.getPropertyDescriptors( clazz );
		for ( int i = 0; i < pd.length; i++ ) {
			if ( pd[i].getName().equals( name ) ) {
				return pd[i];
			}
		}
		return null;
	}
	
	/**
	 * Creates a complete copy of a descriptor.
	 * 
	 * @param desc original descriptor
	 * @return copy of the given descriptor
	 */
	public static OperatorDescriptor copy( OperatorDescriptor desc ) {
		OperatorDescriptor copy = new OperatorDescriptor( desc.getBeanClassName(), desc.getLocation(), desc.getSourceLocation() );
		copy.setBeanClass( desc.getBeanClass() );
		copy.setCategory( desc.getCategory() );
		copy.setDatatype( desc.getDatatype() );
		copy.setDatatypeName( desc.getDatatypeName() );
		copy.setDisplayName( desc.getDisplayName() );
		copy.setExpert( desc.isExpert() );
		copy.setHidden( desc.isHidden() );
		copy.setInputSize( desc.getInputSize() );
		copy.setOutputSize( desc.getOutputSize() );
		copy.setPreferred( desc.isPreferred() );
		copy.setShortDescription( desc.getShortDescription() );
		copy.setVendor( desc.getVendor() );
		for ( Iterator it = desc.iterateProperties(); it.hasNext(); ) {
			AttributeDescriptor ad = (AttributeDescriptor) it.next();
			copy.addAttribute( (AttributeDescriptor) ad.clone() );
		}
		return copy;
	}
	
	/**
	 * Checks if the given descriptor has an associated class file.
	 * 
	 * @param descriptor operator descriptor
	 * @return true, if the descriptor is valid, false instead
	 */
	public static boolean isValid( OperatorDescriptor descriptor ) {
		boolean isValid = false;
		try {
			File parent = new File( descriptor.getLocation() );
			if ( parent.exists() ) {
				if ( parent.isFile() && ( parent.getName().endsWith( "zip" ) || parent.getName().endsWith( "jar" ) ) ) {
					ZipFile zipfile = new ZipFile( parent );
					ZipEntry entry = zipfile.getEntry( StringUtils.replace( descriptor.getBeanClassName(), ".", "/" ) + ".class" );
					if ( entry != null ) {
						isValid = true;
					}
					zipfile.close();
				} else {
					String sep = SystemUtils.FILE_SEPARATOR;
					File file = new File( parent.getAbsolutePath() + sep + StringUtils.replace( descriptor.getBeanClassName(), ".", sep ) + ".class" );
					if ( file.exists() && file.canRead() ) {
						isValid = true;
					}
				}
			}
		} catch ( Exception ex ) {
			// be quiet
		}
		return isValid;
	}
	
	/**
	 * Locates a descriptor for the given class.
	 * 
	 * @param clazz class
	 * @return URI poiting to the descriptor or null if no descriptor has been found
	 */
	public static URI locateDescriptor( Class clazz ) {
		URI uri = null;
		if ( IOperator.class.isAssignableFrom( clazz ) ) {
			String descriptor = ClassUtils.getShortClassname( clazz ) + ".descriptor";
			try {
				uri = new URI( clazz.getResource( descriptor ).toString() );
			} catch ( URISyntaxException usx ) {
				// 
			}
		}
		return uri;
	}
	
	/**
	 * Locates a descriptor for the given class.
	 * 
	 * @param className name of the class
	 * @return URI poiting to the descriptor or null if no descriptor has been found
	 */
	public static URI locateDescriptor( String className ) {
		return locateDescriptor( className, ClassLoader.getSystemClassLoader() );
	}
	
	/**
	 * Locates a descriptor for a given class by using the provided class loader.
	 * 
	 * @param className name of the class
	 * @param loader class loader to be used
	 * @return URI poiting to the descriptor or null if no descriptor has been found
	 */
	public static URI locateDescriptor( String className, ClassLoader loader ) {
		String path = StringUtils.replace( className, ".", "/" ) + ".descriptor";
		URI uri = null;
		try {
			uri = new URI( loader.getResource( path ).toString() );
		} catch ( URISyntaxException usx ) {
			// 
		}
		return uri;
	}
	
	/**
	 * Locates a descriptor for a given class at the given location.
	 * 
	 * @param className name of the class
	 * @param location location of the class
	 * @return URI poiting to the descriptor or null if no descriptor has been found
	 */
	public static URI locateDescriptor( String className, URI location ) {
		URI uri = null;
		try {
			File file = new File( location );
			if ( file.exists() && file.canRead() ) {
				if ( file.isFile() ) { // assume it's a zipfile
					ZipFile zipfile = new ZipFile( file );
					ZipEntry zipentry = zipfile.getEntry( StringUtils.replace( className, ".", "/" ) + ".descriptor" );
					if ( zipentry != null ) {
						String scheme = "jar";
						String specific = file.toURI().toString() + "!/" + zipentry.getName();
						uri = new URI( scheme, specific, null );
					}
				} else { // assume it's a class folder
					String sep = SystemUtils.FILE_SEPARATOR;
					File descLocation = new File( file.getAbsolutePath() + sep + StringUtils.replace( className, ".", sep ) + ".descriptor" );
					if ( descLocation.exists() && descLocation.canRead() ) {
						uri = descLocation.toURI();
					}
				}
			}
		} catch ( Exception ex ) {
			// be quiet ...
		}
		return uri;
	}
	
	/**
	 * Resolves the given attribute descriptor.
	 * 
	 * @param ad descriptor to be resolved
	 * @param clazz class that is the parent of the property to be resolved
	 */
	public static void resolve( AttributeDescriptor ad, Class clazz ) {
		PropertyDescriptor pd = findDescriptor( clazz, ad.getName() );
		if ( pd != null ) {
			ad.setBound( pd.isBound() );
			ad.setConstrained( pd.isConstrained() );
			ad.setPropertyType( pd.getPropertyType() );
			ad.setReadMethod( pd.getReadMethod() );
			ad.setWriteMethod( pd.getWriteMethod() );
			if ( pd instanceof IndexedPropertyDescriptor ) {
				ad.setIndexedPropertyType( ((IndexedPropertyDescriptor) pd).getIndexedPropertyType() );
				ad.setIndexedReadMethod( ((IndexedPropertyDescriptor) pd).getIndexedReadMethod() );
				ad.setIndexedWriteMethod( ((IndexedPropertyDescriptor) pd).getIndexedWriteMethod() );
			}
		}
	}
	
	/**
	 * Resolves the given attribute descriptor.
	 * 
	 * @param ad descriptor to be resolved
	 * @param obj object that is the parent of the property to be resolved
	 */
	public static void resolve( AttributeDescriptor ad, Object obj ) {
		resolve( ad, obj.getClass() );
	}
	
	/**
	 * Attempts to resolve this descriptor.
	 * This method tries to retrieve the classes for beanClass and datatype from the beanClassName
	 * and datatypeName properties. It uses the current system class loader.
	 * 
	 * @param descriptor descriptor to be resolved
	 */
	public static void resolve( OperatorDescriptor descriptor ) {
		resolve( descriptor, ClassLoader.getSystemClassLoader() );
	}
	
	/**
	 * Attempts to resolve this descriptor.
	 * This method tries to retrieve the classes for beanClass and datatype from the beanClassName
	 * and datatypeName properties by use of the given class loader.
	 * 
	 * @param descriptor operator descriptor to be resolved
	 * @param loader class loader to be used for resolving
	 */
	public static void resolve( OperatorDescriptor descriptor, ClassLoader loader ) {
		try {
			Class beanClass = loader.loadClass( descriptor.getBeanClassName() );
			descriptor.setBeanClass( beanClass );
			if ( descriptor.getDatatypeName().equals( "universal" ) ) {
				descriptor.setDatatype( IDatatype.UNIVERSAL );
			} else {
				descriptor.setDatatype( GenotypeRepository.getID( loader.loadClass( descriptor.getDatatypeName() ) ) ); // FIX: remove dependency from evolvica.core
			}
			PropertyDescriptor[] beanDesc = PropertyUtils.getPropertyDescriptors( beanClass );
			LinkedList retain = new LinkedList();
			for ( int i = 0; i < beanDesc.length; i++ ) {
				AttributeDescriptor ad = descriptor.getAttribute( beanDesc[i].getName() );
				if ( ad == null ) {
					ad = new AttributeDescriptor( beanDesc[i] );
					descriptor.addAttribute( ad );
				} else {
					ad.setBound( beanDesc[i].isBound() );
					ad.setConstrained( beanDesc[i].isConstrained() );
					ad.setPropertyType( beanDesc[i].getPropertyType() );
					ad.setReadMethod( beanDesc[i].getReadMethod() );
					ad.setWriteMethod( beanDesc[i].getWriteMethod() );
					if ( beanDesc[i] instanceof IndexedPropertyDescriptor ) {
						ad.setIndexedPropertyType( ((IndexedPropertyDescriptor) beanDesc[i]).getIndexedPropertyType() );
						ad.setIndexedReadMethod( ((IndexedPropertyDescriptor) beanDesc[i]).getIndexedReadMethod() );
						ad.setIndexedWriteMethod( ((IndexedPropertyDescriptor) beanDesc[i]).getIndexedWriteMethod() );
					}
				}
				retain.add( ad );
			}
			descriptor.retainAttributes( retain );
		} catch ( ClassNotFoundException cnfx ) {
			// be quiet
		}
	}
	
	public static void main( String[] args ) throws Exception {
//		System.out.println( locateDescriptor( "org.evolvica.core.select.RandomSelector" ) );
//		System.out.println( locateDescriptor( org.evolvica.core.select.RandomSelector.class ) );
		URI uri;
		//uri = new URI( ClassLoader.getSystemResource( "java/util/Vector.class" ).toString() );
		//uri = new URI( "file:/c:/java/evolvica/workspace-2.1.1/reference/classes/" );
		uri = new URI( "file:/c:/java/evolvica/workspace-2.1.1/reference/com.jar" );
		//uri = new URI( "file:/c:/java/evolvica/workspace-2.1.1/reference/com.zip" );
		uri = locateDescriptor( "com.ReferenceEvaluator", uri );
		System.out.println( uri );
		OperatorDescriptor d = new OperatorDescriptor( "com.ReferenceEvaluator", new URI( "file:/c:/java/evolvica/workspace-2.1.1/reference/com.jar" ), null );
		System.out.println( isValid( d ) );
	}
	
}
