/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:23 $
 * $Revision: 1.4 $
 * $State: Exp $
 * $RCSfile: DescriptorWriter.java,v $
 * 
 */
package org.evolvica.engine;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.net.URI;
import java.util.Iterator;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.evolvica.util.SerializationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/** Writer for operator descriptors.
 *
 * @author Andreas Rummler
 * @since 0.6.0
 * @version $Revision: 1.4 $ $Date: 2004/05/26 11:58:23 $
 */
public class DescriptorWriter {
	
	/**
	 * Writes an XML document to the given URI.
	 * 
	 * @param document XML document
	 * @param uri URI to write to (must be a directory, but is not checked!)
	 * @throws SerializationException thrown in case the serialization fails
	 */
	public static void write( Document document, URI uri ) throws SerializationException {
		try{
			OutputStream stream = new FileOutputStream( new File( uri ) );
			write( document, stream );
			stream.close();
		} catch ( Exception ex ) {
			throw new SerializationException( "unable to write document to " + uri, ex );
		}
	}
	
	/**
	 * Writes an XML algorithm document to an output stream.
	 * 
	 * @param document document to write
	 * @param stream output stream to write to
	 * @throws SerializationException thrown in case the serialization fails
	 */
	public static void write( Document document, OutputStream stream ) throws SerializationException {
		try {
			Transformer transformer = TransformerFactory.newInstance().newTransformer();
			transformer.setOutputProperty( OutputKeys.INDENT, "yes" );
			Source source = new DOMSource( document.getDocumentElement() );
			Result result = new StreamResult( stream );
			transformer.transform( source, result );
		} catch ( Exception ex ) {
			throw new SerializationException( "unable to transform the XML document and write it to output stream" );
		}
	}
	
	/**
	 * Writes an operator descriptor to an output stream.
	 * 
	 * @param descriptor descriptor to be written
	 * @param stream output stream to write to
	 * @throws SerializationException thrown in case the XML document could not be created
	 */
	public static void write( OperatorDescriptor descriptor, OutputStream stream ) throws SerializationException {
		try {
			write( createDocument( descriptor ), stream );
		} catch ( Exception ex ) {
			throw new SerializationException( "unable to create XML document from descriptor " + descriptor, ex );
		}
	}
	
	/**
	 * Writes an operator descriptor to the given URI.
	 * 
	 * @param descriptor descriptor to be written
	 * @param uri URI to write to
	 * @throws SerializationException thrown in case the XML document could not be created
	 */
	public static void write( OperatorDescriptor descriptor, URI uri ) throws SerializationException {
		try {
			write( createDocument( descriptor ), uri );
		} catch ( Exception ex ) {
			throw new SerializationException( "unable to create XML document from descriptor " + descriptor, ex );
		}
	}
	
	/**
	 * Creates a XML document from a given algorithm.
	 * 
	 * @param descriptor operator descriptor to be converted into an XML document
	 * @return XML document containing the algorithm description
	 * @throws SerializationException thrown in case the document could not be created
	 */
	public static Document createDocument( OperatorDescriptor descriptor ) throws ParserConfigurationException, FactoryConfigurationError {
		Document document = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
		Element docElement = document.createElement( "operator" );
		document.appendChild( docElement );
		docElement.setAttribute( "beanClass", descriptor.getBeanClassName() );
		docElement.setAttribute( "category", descriptor.getCategory() );
		docElement.setAttribute( "datatype", descriptor.getDatatypeName() );
		docElement.setAttribute( "expert", String.valueOf( descriptor.isExpert() ) );
		docElement.setAttribute( "displayName", descriptor.getDisplayName() );
		docElement.setAttribute( "hidden", String.valueOf( descriptor.isHidden() ) );
		docElement.setAttribute( "inputSize", String.valueOf( descriptor.getInputSize() ) );
		docElement.setAttribute( "outputSize", String.valueOf( descriptor.getOutputSize() ) );
		docElement.setAttribute( "preferred", String.valueOf( descriptor.isPreferred() ) );
		docElement.setAttribute( "shortDescription", descriptor.getShortDescription() );
		docElement.setAttribute( "vendor", descriptor.getVendor() );
		for ( Iterator it = descriptor.iterateProperties(); it.hasNext(); ) {
			docElement.appendChild( createAttributeNode( document, (AttributeDescriptor) it.next() ) );
		}
		return document;
	}
	
	/**
	 * Creates an XML element from a given attribute descriptor.
	 * 
	 * @param document XML document used for element creation
	 * @param descriptor attribute descriptor
	 * @return created XML element
	 */
	public static Element createAttributeNode( Document document, AttributeDescriptor descriptor ) {
		Element element = document.createElement( "property" );
		element.setAttribute( "displayName", descriptor.getDisplayName() );
		element.setAttribute( "expert", String.valueOf( descriptor.isExpert() ) );
		element.setAttribute( "hidden", String.valueOf( descriptor.isHidden() ) );
		element.setAttribute( "name", descriptor.getName() );
		element.setAttribute( "preferred", String.valueOf( descriptor.isPreferred() ) );
		element.setAttribute( "shortDescription", descriptor.getShortDescription() );
		return element;
	}
	
}
