/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:23 $
 * $Revision: 1.10 $
 * $State: Exp $
 * $RCSfile: IAlgorithm.java,v $
 *
 */
package org.evolvica.engine;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyVetoException;
import java.util.Iterator;

import org.evolvica.util.IMetadataProvider;
import org.evolvica.util.IMetadataSerializer;
import org.evolvica.util.Metadata;

/** Interface for an algorithm.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.10 $ $Date: 2004/05/26 11:58:23 $
 */
public interface IAlgorithm {
	
	/**
	 * Adds a property change listener to this algorithm.
	 * 
	 * @param listener property change listener to be added
	 */
	public void addPropertyChangeListener( PropertyChangeListener listener );
	
	/**
	 * Adds a property change listener that reacts on metadata changes to this algorithm.
	 * 
	 * @param listener property change listener to be added
	 */
	public void addMetadataChangeListener( PropertyChangeListener listener );
	
	/**
	 * Removes a property change listener from this algorithm.
	 * 
	 * @param listener property change listener to be removed
	 */
	public void removePropertyChangeListener( PropertyChangeListener listener );
	
	/**
	 * Removes a metadata property change listener from this algorithm.
	 * 
	 * @param listener property change listener to be removed
	 */
	public void removeMetadataChangeListener( PropertyChangeListener listener );
	
	/**
	 * Fires a property change event.
	 * 
	 * @param event event to be fired
	 */
	public void firePropertyChangeEvent( PropertyChangeEvent event );
	
	/**
	 * Sets up the algorithm.
	 * This method is called before the algorithm is started.
	 * 
	 * @throws Exception thrown if the setup stage fails
	 */
	public void setup() throws Exception;
	
	/**
	 * Executes the algorithm.
	 * 
	 */
	public void execute();
	
	/**
	 * Teardown method.
	 * This method is called after an algorithm has finished its execution.
	 * 
	 * @throws Exception thrown if teardown stage fails
	 */
	public void teardown() throws Exception;
	
	/**
	 * Returns the name of the algorithm.
	 * 
	 * @return name of the algorithm
	 */
	public String getName();
	
	/**
	 * Sets the name of the algorithm.
	 * 
	 * @param name name of the algorithm
	 */
	public void setName( String name );
	
	/**
	 * Returns a metadata object from the associated metadata store.
	 * 
	 * @param source source object
	 * @param path metadata path
	 * @return metadata object or null if no such object exists
	 */
	public Metadata getMetadata( Object source, String path );
	
	/**
	 * Restores the metadata from the given metadata provider.
	 * 
	 * @param provider metadata provider
	 */
	public void restoreMetadata( IMetadataProvider provider );
	
	/**
	 * Serializes this algorithm's metadata by using the given serializer.
	 * 
	 * @param serializer metadata serializer
	 */
	public void serializeMetadata( IMetadataSerializer serializer );
	
	/**
	 * Sets a metadata object.
	 * 
	 * @param source source object
	 * @param data metadata object
	 */
	public void setMetadata( Object source, Metadata data );
	
	/**
	 * Adds an element to this algorithm.
	 * 
	 * @param element alement to add
	 */
	public void addElement( IElement element );
	
	/**
	 * Gets an element by its ID.
	 * 
	 * @param id id of the element
	 * @return element with the given ID or null
	 */
	public IElement getElementById( String id );
	
	/**
	 * Iterates all elements in this algorithm.
	 * 
	 * @return iterator over all algorithm elements.
	 */
	public Iterator iterateElements();
	
	/**
	 * Removes an element from this algorithm.
	 * 
	 * @param element element to be removed
	 */
	public void removeElement( IElement element );
	
	/**
	 * Returns the number of elements contained in this algorithm.
	 * 
	 * @return number of elements in this algorithm
	 */
	public int elementSize();
	
	/**
	 * Connects two elements.
	 * 
	 * @param source source element
	 * @param output source slot
	 * @param target target element
	 * @param input target slot
	 */
	public void connect( IElement source, int output, IElement target, int input );
	
	/**
	 * Connects two operators.
	 * 
	 * @param source source operator
	 * @param output source slot
	 * @param target target operator
	 * @param input target slot
	 */
	public void connect( IOperator source, int output, IOperator target, int input );
	
	/**
	 * Returns the connection between two elements.
	 * 
	 * @param source source element
	 * @param output source slot
	 * @param target target element
	 * @param input target slot
	 * @return connection between the given elements or null if no such connection exists
	 */
	public IConnection getConnection( IElement source, int output, IElement target, int input );
	
	/**
	 * Returns the connection between two operators.
	 * 
	 * @param source source operator
	 * @param output source slot
	 * @param target target operator
	 * @param input target slot
	 * @return connection between the given operators or null if no such connection exists
	 */
	public IConnection getConnection( IOperator source, int output, IOperator target, int input );
	
	/**
	 * Iterates all connections in this algorithm.
	 * 
	 * @return iterator over all connections
	 */
	public Iterator iterateConnections();
	
	/**
	 * Removes a connection from this algorithm.
	 * 
	 * @param connection connection to be removed.
	 */
	public void removeConnection( IConnection connection );
	
	/**
	 * Returns the number of connections in this algorithm.
	 * 
	 * @return number of connections
	 */
	public int connectionSize();
	
	/**
	 * Gets the algorithm runner.
	 * 
	 * @return algorithm runner
	 */
	public IRunner getRunner();
	
	/**
	 * Sets a new algorithm runner.
	 * 
	 * @param runner new algorithm runner
	 * @throws PropertyVetoException thrown if algorithm runner is null
	 */
	public void setRunner( IRunner runner ) throws PropertyVetoException;
	
	/**
	 * Gets the algorithm validator.
	 * 
	 * @return the current validator.
	 */
	public IValidator getValidator();
	
	/**
	 * Sets the algorithm validator.
	 * 
	 * @param validator new algorithm validator
	 */
	public void setValidator( IValidator validator );
	
}
