/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:24 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: IDatatype.java,v $
 *
 */

package org.evolvica.engine;

/**
 * An identifier for datatypes.
 * Identifiers are used to recognize the type of a data. Validity checks
 * in operators rely on these type identifiers.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:58:24 $
 *
 * @since 0.4.0
 */
public interface IDatatype {
	
	/**
	 * A universal datatype.
	 * Use this datatype to indicate that an operator works on all kinds
	 * of datatypes.
	 *
	 */
	public static final IDatatype UNIVERSAL = new UniversalDatatype();
	
	/**
	 * Return the name of this identifier.
	 *
	 * @return name of this identifier
	 */
	public String getName();

	/**
	 * Returns the class of the referenced datatype.
	 *
	 * @return class of the referenced datatype
	 */
	public Class getReferencedClass();
	
	/**
	 * Checks if this ID equals the given (other) ID.
	 * 
	 * @param otherType data type to check
	 * @return true if both IDs equal, false instead
	 */
	public boolean isEqualTo( IDatatype otherType );
	
	/**
	 * Checks if this datatype is compatible to another datatype.
	 * This datatype is compatible if the referenced class is a subclass of the
	 * referenced class of the other datatype. Example: Given are the two classes:
	 * <blockcode><pre>
	 * public class DefaultInteger { ... }
	 * public class ExtendedInteger extends DefaultInteger { ... }
	 * </pre></blockcode>
	 * Calls to the isCompatibleTo()-methods:
	 * <blockcode><pre>
	 * DefaultInteger integer = new DefaultInteger();
	 * ExtendedInteger extInteger = new ExtendedInteger();
	 * System.out.println( integer.getType().isCompatibleTo( extInteger.getType() ) );
	 * System.out.println( extInteger.getType().isCompatibleTo( integer.getType() ) );
	 * </pre></blockcode>
	 * return the following results:
	 * <blockcode><pre>
	 * false
	 * true
	 * </pre></blockcode>
	 * That means an ExtendedInteger can be used as a DefaultInteger but the
	 * opposite case (using a DefaultInteger as an ExtendedInteger) is not
	 * possible.
	 *
	 * @param otherID data type to check
	 * @return true if this datatype is compatible to the other datatype, false instead
	 */
	public boolean isCompatibleTo( IDatatype otherID );
	
	/**
	 * A universal datatype.
	 * This class cannot be instatiated, use IDatatype.UNIVERSAL to refer to
	 * this datatype.
	 *
	 */
	public final class UniversalDatatype implements IDatatype {
		
		/**
		 * Constructor.
		 */
		public UniversalDatatype() {
			// do nothing
		}
		
		/**
		 * @see org.evolvica.engine.IDatatype#getName()
		 */
		public String getName() {
			return "universal";
		}
		
		/**
		 * @see org.evolvica.engine.IDatatype#getReferencedClass()
		 */
		public Class getReferencedClass() {
			return Object.class;
		}
		
		/**
		 * @see org.evolvica.engine.IDatatype#isEqualTo(org.evolvica.engine.IDatatype)
		 */
		public boolean isEqualTo( IDatatype otherDatatype ) {
			if ( Object.class == otherDatatype.getReferencedClass() ) {
				return true;
			} else {
				return false;
			}
		}

		/**
		 * @see org.evolvica.engine.IDatatype#isCompatibleTo(org.evolvica.engine.IDatatype)
		 */
		public boolean isCompatibleTo( IDatatype otherDatatype ) {
			if ( otherDatatype.getReferencedClass().isAssignableFrom( Object.class ) ) {
				return true;
			} else {
				return false;
			}
		}
	}
	
}
