/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:24 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: IElement.java,v $
 *
 */

package org.evolvica.engine;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.evolvica.util.Metadata;


/** Interface for an algorithmic element used in algorithms.
 * Actually this is used as a wrapper for IOperator instances.
 * Algorithmic elements are connected by IConnection objects.
 * These connections transport data between algorithmic elements.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:58:24 $
 */
public interface IElement {
	
	/**
	 * Adds a property change listener to this element.
	 * 
	 * @param listener property change listener to be added
	 */
	public void addPropertyChangeListener( PropertyChangeListener listener );
	
	/**
	 * Removes a property change listener from this element.
	 * 
	 * @param listener property change listener to be removed
	 */
	public void removePropertyChangeListener( PropertyChangeListener listener ); 
	
	/**
	 * Fires a property change event.
	 * 
	 * @param event event to be fired
	 */
	public void firePropertyChangeEvent( PropertyChangeEvent event );
	
	/**
	 * Returns the algorithm this element is contained in.
	 * 
	 * @return algorithm this element is contained in
	 */
	public IAlgorithm getAlgorithm();
	
	/**
	 * Sets the algorithm this element belongs to.
	 * 
	 * @param algorithm the algorithm this element belongs to
	 */
	public void setAlgorithm( IAlgorithm algorithm );
	
	/**
	 * Returns the operator that is assigned to this element.
	 * 
	 * @return assigned operator
	 */
	public IOperator getOperator();
	
	/**
	 * Returns a metadata object for this element from the associated metadata store.
	 * The metadata is stored in the associated algorithms metadata store.
	 * 
	 * @param path metadata path
	 * @return metadata object or null if no such object exists
	 */
	public Metadata getMetadata( String path );
	
	/**
	 * Sets a metadata object for this element.
	 * The metadata is stored in the associated algorithms metadata store.
	 * 
	 * @param data metadata object
	 */
	public void setMetadata( Metadata data );
	
	/**
	 * Returns the number of incoming connections.
	 * 
	 * @return number of incoming connections
	 */
	public int incomingSize();
	
	/**
	 * Returns the number of outgoing connections.
	 * 
	 * @return number of outgoing connections
	 */
	public int outgoingSize();
	
	/**
	 * Gets an incoming connection.
	 * 
	 * @param index index of the connection
	 * @return incoming connection with the specified index
	 */
	public IConnection getIncoming( int index );
	
	/**
	 * Returns the index of the given incoming connection.
	 * 
	 * @param connection incoming connection
	 * @return index of the connection or -1 if the connection is
	 * not an incoming connection for this element
	 */
	public int getIncomingIndex( IConnection connection );
	
	/**
	 * Sets an incoming connection.
	 * 
	 * @param connection connection to set
	 * @param index index of the coonection
	 */
	public void setIncoming( IConnection connection, int index );
	
	/**
	 * Sets a new value for the number of incoming slots.
	 * 
	 * @param size number of incoming slots 
	 */
	public void setIncomingSize( int size );
	
	/**
	 * Gets an outgoing connection.
	 * 
	 * @param index index of the connection
	 * @return outgoing connection with the specified index
	 */
	public IConnection getOutgoing( int index );
	
	/**
	 * Returns the index of the given outgoing connection.
	 * 
	 * @param connection outgoing connection
	 * @return index of the connection or -1 if the connection is
	 * not an outgoing connection for this element
	 */
	public int getOutgoingIndex( IConnection connection );
	
	/**
	 * Sets an outgoing connection.
	 * 
	 * @param connection connection to set
	 * @param index index of the coonection
	 */
	public void setOutgoing( IConnection connection, int index );
	
	/**
	 * Sets a new value for the number of outgoing slots.
	 * 
	 * @param size number of outgoing slots 
	 */
	public void setOutgoingSize( int size );
	
	/**
	 * Removes a connection from this element.
	 * 
	 * @param connection connection to be removed
	 */
	public void removeConnection( IConnection connection );
	
	/**
	 * Activates this element.
	 */
	public void activate();
	
	/**
	 * Checks if this element can activated.
	 * 
	 * @return true if this element can be activated, false instead
	 */
	public boolean isActivatable();
	
}
