/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:26 $
 * $Revision: 1.16 $
 * $State: Exp $
 * $RCSfile: OperatorDescriptor.java,v $
 *
 */
package org.evolvica.engine;

import java.beans.BeanDescriptor;
import java.io.File;
import java.io.IOException;
import java.io.InvalidObjectException;
import java.net.*;
import java.util.*;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.apache.commons.beanutils.ConversionException;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.SystemUtils;
import org.evolvica.util.ClassUtils;
import org.evolvica.util.Convert;
import org.evolvica.util.SerializationException;
import org.xml.sax.SAXException;

/**
 * Descriptor for an algorithmic operator.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.16 $ $Date: 2004/05/26 11:58:26 $
 */
public class OperatorDescriptor extends AbstractDescriptor {
	
	// properties of operator
	private String category = "misc";
	private String datatypeName = null;
	private int inputSize = 0;
	private int outputSize = 0;
	private String vendor = "no vendor information available";
	
	// additional properties for internal use
	// they may be serialized and restored but must be calculated on the client system
	private String beanClassName = null;
	private URI classLocation = null;
	private URI sourceLocation = null;
	
	// set of attribute descriptors
	private Set properties = new TreeSet();
	
	// transient properties
	private transient Class beanClass = null;
	private transient IDatatype datatype = null;
	
	/**
	 * Constructor.
	 * Creates an operator descriptor describing the given operator class. The class must be
	 * located at the given location. The source location is optional.
	 * 
	 * @param beanClassName operator class name
	 * @param classLocation class location
	 * @param sourceLocation source location (optional)
	 */
	public OperatorDescriptor( String beanClassName, URI classLocation, URI sourceLocation ) {
		this( beanClassName, classLocation, sourceLocation, Convert.toClassname( IDatatype.UNIVERSAL.getReferencedClass() ) );
	}
	
	/**
	 * Constructor.
	 * Creates an operator descriptor describing the given operator class. The class must be
	 * located at the given location. The source location is optional.
	 * 
	 * @param beanClassName operator class name
	 * @param classLocation class location
	 * @param sourceLocation source location (optional)
	 * @param datatypeName datatype class name
	 */
	public OperatorDescriptor( String beanClassName, URI classLocation, URI sourceLocation, String datatypeName ) {
		super( beanClassName.substring( beanClassName.lastIndexOf( '.' ) + 1 ) );
		this.beanClassName = beanClassName;
		this.classLocation = classLocation;
		this.sourceLocation = sourceLocation;
		this.datatypeName = datatypeName;
	}
	
	/**
	 * Constructor.
	 * Creates an operator descriptor which describes the given class. Note that no check is
	 * performed if the class is really an evolutionary operator.
	 * 
	 * @param beanClass operator bean class
	 */
	public OperatorDescriptor( Class beanClass ) {
		this( beanClass, IDatatype.UNIVERSAL );
	}
	
	/**
	 * Constructor.
	 * Creates an operator descriptor which describes the given class. Note that no check is
	 * performed if the class is really an evolutionary operator.
	 * 
	 * @param beanClass operator bean class
	 * @param datatype datatype for this operator
	 */
	public OperatorDescriptor( Class beanClass, IDatatype datatype ) {
		super( ClassUtils.getShortClassname( beanClass ) );
		this.beanClass = beanClass;
		this.beanClassName = beanClass.getName();
		this.datatype = datatype;
		this.datatypeName = Convert.toClassname( datatype.getReferencedClass() );
		// inititailiz properties
		java.beans.PropertyDescriptor[] props = PropertyUtils.getPropertyDescriptors( this.beanClass );
		for ( int i = 0; i < props.length; i++ ) {
			properties.add( new AttributeDescriptor( props[i]) );
		}
	}
	
	/**
	 * Constructor.
	 * Creates an operator descriptor as a copy of a bean descriptor.
	 * 
	 * @param descriptor bean descriptor
	 */
	public OperatorDescriptor( BeanDescriptor descriptor ) {
		super( descriptor.getName() );
		// set common field
		setDisplayName( descriptor.getDisplayName() );
		setExpert( descriptor.isExpert() );
		setHidden( descriptor.isHidden() );
		setPreferred( descriptor.isPreferred() );
		setShortDescription( descriptor.getShortDescription() );
		// set operator related fields
		this.beanClass = descriptor.getBeanClass();
		this.beanClassName = descriptor.getBeanClass().getName();
		this.datatype = IDatatype.UNIVERSAL;
		this.datatypeName = Convert.toClassname( datatype.getReferencedClass() );
		// initialize properties
		java.beans.PropertyDescriptor[] props = PropertyUtils.getPropertyDescriptors( this.beanClass );
		for ( int i = 0; i < props.length; i++ ) {
			properties.add( new AttributeDescriptor( props[i]) );
		}
	}
	
	/**
	 * Adds an attribute descriptor to this operator descriptor.
	 * 
	 * @param ad attribute descriptor
	 */
	public void addAttribute( AttributeDescriptor ad ) {
		properties.add( ad );
	}
	
	/**
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo( Object o ) {
		return getBeanClassName().compareTo( ((OperatorDescriptor) o).getBeanClassName() );
	}
	
	/**
	 * Returns the operator class.
	 * 
	 * @return operator class
	 */
	public Class getBeanClass() {
		if ( beanClass == null ) {
			try {
				beanClass = Class.forName( beanClassName );
			} catch ( ClassNotFoundException cnfx ) {
				// try custom classloader
				try {
					ClassLoader loader = ClassLoader.getSystemClassLoader();
					URLClassLoader urlLoader = URLClassLoader.newInstance( new URL[] { classLocation.toURL() }, loader );
					beanClass = urlLoader.loadClass( beanClassName );
				} catch ( Exception ex ) {
					// be quiet
				} catch ( NoClassDefFoundError error ) {
					// be also quiet
				}
			}
		}
		return beanClass;
	}

	/**
	 * Returns the operator class name.
	 * 
	 * @return operator class name
	 */
	public String getBeanClassName() {
		return beanClassName;
	}

	/**
	 * Returns the category.
	 * 
	 * @return category
	 */
	public String getCategory() {
		return category;
	}

	/**
	 * Returns the datatype.
	 * 
	 * @return datatype
	 */
	public IDatatype getDatatype() {
		return datatype;
	}

	/**
	 * Returns the datatype name.
	 * 
	 * @return datatype name
	 */
	public String getDatatypeName() {
		return datatypeName;
	}

	/**
	 * Returns the input size.
	 * 
	 * @return input size
	 */
	public int getInputSize() {
		return inputSize;
	}
	
	/**
	 * Returns the class location
	 * 
	 * @return class location or null if location is unknown
	 */
	public URI getLocation() {
		return classLocation;
	}

	/**
	 * Returns the output size
	 * 
	 * @return output size
	 */
	public int getOutputSize() {
		return outputSize;
	}

	/**
	 * Returns the property descriptor with the given name.
	 * 
	 * @param property property name
	 * @return property descriptor with the given name or null
	 */
	public AttributeDescriptor getAttribute( String property ) {
		for ( Iterator it = properties.iterator(); it.hasNext(); ) {
			AttributeDescriptor element = (AttributeDescriptor) it.next();
			if ( element.getName().equals( property ) ) {
				return element;
			}
		}
		return null;
	}
	
	/**
	 * Returns the property descriptor with the given name.
	 * 
	 * @param property property name
	 * @return property descriptor with the given name or null
	 * @deprecated use getAttribute( String )
	 */
	public AttributeDescriptor getPropertyDescriptor( String property ) {
		return getAttribute( property );
	}
	
	/**
	 * Returns the source location.
	 * 
	 * @return source location or null if location is unknown
	 */
	public URI getSourceLocation() {
		return sourceLocation;
	}

	/**
	 * Returns the vendor.
	 * 
	 * @return vendor
	 */
	public String getVendor() {
		return vendor;
	}
	
	/**
	 * Prints some information to the console.
	 */
	public void info() {
		StringBuffer buffer = new StringBuffer();
		if ( isResolved() ) {
			buffer.append( "name: " + getName() + "\n" );
			buffer.append( "class: " + getBeanClass().getName() + "\n" );
			buffer.append( "category: " + getCategory() + "\n" );
			buffer.append( "vendor: " + getVendor() + "\n" );
			buffer.append( "required datatype: " + datatype.getName() + "\n" );
			buffer.append( "interface: " + getInputSize() + " inputs, " + getOutputSize() + " outputs" );
			buffer.append( "properties:\n" );
			for ( Iterator it = iterateProperties(); it.hasNext(); ) {
				AttributeDescriptor desc = (AttributeDescriptor) it.next();
				buffer.append( "\t" + desc.getName() + " <" );
				if ( desc.isConstrained() ) {
					buffer.append( "constrained" );
				} else {
					buffer.append( "unconstrained" );
				}
				if ( desc.getReadMethod() != null ) {
					buffer.append( "/readable" );
				}
				if ( desc.getWriteMethod() != null ) {
					buffer.append( "/writeable" );
				}
				buffer.append( "> (" );
				buffer.append( desc.getShortDescription() );
				buffer.append( ")\n" );
			}
			buffer.append( "description: " + getShortDescription() );
		} else {
			buffer.append( "name: " + beanClassName + " (UNRESOLVED)\n" );
			buffer.append( "class location: " + classLocation + "\n" );
			buffer.append( "source location: " + sourceLocation + "\n" );
			long lastModified = lastModified();
			if ( lastModified == Long.MAX_VALUE ) {
				buffer.append( "last modified: ???" );
			} else {
				buffer.append( "last modified: " + new Date( lastModified() ).toString() );
			}
		}
		System.out.println( buffer );
	}

	/**
	 * Checks if this descriptor has been resolved.
	 * Resolved means the classes for beanClass and the datatype properties have been retrieved
	 * and are not null.
	 * 
	 * @return true if this descriptor has already been resolved
	 */
	public boolean isResolved() {
		if ( beanClass != null && datatype != null && properties.size() > 0 ) {
			return true;
		}
		return false;
	}

	/**
	 * Checks if this descriptor points to a valid class file.
	 * 
	 * @return true if this descriptor is valid
	 * @deprecated use DescriptorResolver to check for validity
	 */
	public boolean isValid() {
		return DescriptorResolver.isValid( this );
	}

	/**
	 * Iterate all properties descriptors.
	 * 
	 * @return iterator
	 */
	public Iterator iterateProperties() {
		return properties.iterator();
	}
	
	/**
	 * Checks for the modification time of the resource this descriptor points to.
	 * 
	 * @return modification time stamp
	 */
	public long lastModified() {
		File parent = new File( classLocation );
		if ( parent.exists() ) {
			if ( parent.isFile() ) {
				return parent.lastModified();
			} else {
				File file = new File( parent.getAbsolutePath()
					+ SystemUtils.FILE_SEPARATOR
					+ StringUtils.replace( beanClassName, ".", SystemUtils.FILE_SEPARATOR ) + ".class" );
				if ( file.exists() ) {
					return file.lastModified();
				}
			}
		}
		return Long.MAX_VALUE;
	}
	
	/**
	 * Loads the descriptor.
	 * Loading means loading necessary information from a descriptor file.
	 * This method searches the source location/class location for a file named
	 * &lt;classname&gt;.descriptor and attempts to load it.
	 * 
	 * @throws IOException thrown if reading the descriptor file fails
	 * @deprecated use DescriptorBuilder to read descriptors
	 */
	public void load() throws IOException, ParserConfigurationException, SAXException {
		URI uri = DescriptorResolver.locateDescriptor( getBeanClassName(), classLocation );
		if ( uri == null ) {
			uri = DescriptorResolver.locateDescriptor( getBeanClassName(), sourceLocation );
		}
		OperatorDescriptor other = null;
		try {
			other = DescriptorBuilder.build( uri );
		} catch ( SerializationException sx ) {
			throw new IOException( "unable to load descriptor for operator " + getBeanClassName() );
		}
		if ( ! beanClassName.equals( other.getBeanClassName() ) ) {
			throw new InvalidObjectException( "bean class does not match specified bean class in serialized descriptor" );
		} else {
			setCategory( other.getCategory() );
			setDatatypeName( other.getDatatypeName() );
			setDisplayName( other.getDisplayName() );
			setExpert( other.isExpert() );
			setHidden( other.isHidden() );
			setInputSize( other.getInputSize() );
			setOutputSize( other.getOutputSize() );
			setPreferred( other.isPreferred() );
			setShortDescription( other.getShortDescription() );
			setVendor( other.getVendor() );
			for ( Iterator it = other.iterateProperties(); it.hasNext(); ) {
				properties.add( it.next() );
			}
		}
	}
	
	void retainAttributes( Collection retain ) {
		properties.retainAll( retain );
	}
	
	/**
	 * Attempts to resolve this descriptor.
	 * This method tries to retrieve the classes for beanClass and datatype from the beanClassName
	 * and datatypeName properties by use of the given classpath.
	 * 
	 * @param classpath classpath to use
	 * @throws ConversionException thrown if the internal type names couldn't be converted
	 * @throws MalformedURLException thrown if the classpath contains an invalid URI
	 * @deprecated use DescriptorResolver to perform the resolution
	 */
	public void resolve( URI[] classpath ) throws ConversionException, MalformedURLException {
		URL[] cp = null;
		if ( classpath == null ) {
			cp = new URL[0];
		} else {
			cp = new URL[classpath.length];
			for ( int i = 0; i < classpath.length; i++ ) {
				cp[i] = classpath[i].toURL();
			}
		}
	}
	
	/**
	 * Tries to resolve this descriptor and returns a copy of it.
	 * 
	 * @param classpath classpath to use
	 * @return copy of the resolved descriptor
	 * @throws ConversionException thrown if the internal type names couldn't be converted
	 * @throws MalformedURLException thrown if the classpath contains an invalid URI
	 * @deprecated use DescriptorResolver for copying
	 */
	public OperatorDescriptor getResolved( URI[] classpath ) throws ConversionException, MalformedURLException {
		resolve( classpath );
		OperatorDescriptor clone = new OperatorDescriptor( beanClassName, classLocation, sourceLocation, datatypeName );
		clone.beanClass = beanClass;
		clone.category = category;
		clone.datatype = datatype;
		clone.setDisplayName( getDisplayName() );
		clone.setExpert( isExpert() );
		clone.setHidden( isHidden() );
		clone.inputSize = inputSize;
		clone.outputSize = outputSize;
		clone.setPreferred( isPreferred() );
		clone.properties = new TreeSet();
		if ( properties != null ) {
			for ( Iterator it = properties.iterator(); it.hasNext(); ) {
				clone.properties.add( ((AttributeDescriptor) it.next()).clone() );
			}
		}
		clone.setShortDescription( getShortDescription() );
		clone.vendor = vendor;
		return clone;
	}
	
	/**
	 * Saves this descriptor.
	 * This method tries to save the descriptor information into an XML file at the source location
	 * of the operator class. If the source location does not exist it attempts to write the XML file
	 * to the class location.
	 * 
	 * @throws ParserConfigurationException thrown if the XML parser couldn't be configured
	 * @throws TransformerException thrown if the internal XML file couldn't be transformed into a XML file
	 * @throws URISyntaxException thrown in case the source and/or class location URI is invalid
	 * @deprecated use DescriptorWriter and DescriptorResolver to store descriptors to disk
	 */
	public void save() throws ParserConfigurationException, TransformerException, URISyntaxException {
		URI out = null;
		File tmpFile = null;
		URI tmpURI = null;
		if ( sourceLocation != null ) {
			String fullPath = sourceLocation.toString() + beanClassName.replace( '.', '/' );
			tmpURI = new URI( fullPath.substring( 0, fullPath.lastIndexOf( "/" ) + 1 ) );
			tmpFile = new File( tmpURI );
			if ( tmpFile.exists() && tmpFile.isDirectory() ) {
				out = new URI( tmpURI.toString() + getName() + ".descriptor" );
			}
		}
		if ( out == null ) { // source location not accessible, try it with the class location
			if ( classLocation != null ) {
				String fullPath = classLocation.toString() + beanClassName.replace( '.', '/' );
				tmpURI = new URI( fullPath.substring( 0, fullPath.lastIndexOf( "/" ) + 1 ) );
				tmpFile = new File( classLocation );
				if ( tmpFile.exists() && tmpFile.isDirectory() ) {
					out = new URI( tmpURI.toString() + getName() + ".descriptor" );
				}
			}
		}
		try {
			DescriptorWriter.write( this, out );
		} catch ( SerializationException sx ) {
			throw new TransformerException( "unable to write descriptor " + beanClassName, sx );
		}
	}
	
	void setBeanClass( Class beanClass ) {
		this.beanClass = beanClass;
	}
	
	/**
	 * Sets the category.
	 * 
	 * @param string category
	 */
	public void setCategory( String string ) {
		category = string;
	}

	/**
	 * Sets the datatype.
	 * 
	 * @param type datatype
	 */
	public void setDatatype( IDatatype type ) {
		this.datatype = type;
		if ( type == null ) {
			this.datatypeName = "UNKNOWN";
		} else {
			this.datatypeName = type.getName();
		}
	}
	
	/**
	 * Sets the datatype name.
	 * 
	 * @param name datatype name
	 */
	public void setDatatypeName( String name ) {
		this.datatypeName = name;
	}
	
	/**
	 * Sets the input size.
	 * 
	 * @param i input size
	 */
	public void setInputSize( int i ) {
		inputSize = i;
	}

	/**
	 * Sets the interface (input- and output size).
	 * 
	 * @param inputs number of inputs
	 * @param outputs number of outputs
	 */
	public void setInterface( int inputs, int outputs ) {
		inputSize = inputs;
		outputSize = outputs;
	}
	
	/**
	 * Sets the class and source location of the operator.
	 * 
	 * @param classLocation class location
	 * @param sourceLocation source location
	 */
	public void setLocation( URI classLocation, URI sourceLocation ) {
		this.classLocation = classLocation;
		this.sourceLocation = sourceLocation;
	}
	
	/**
	 * Sets the output size.
	 * 
	 * @param i output size
	 */
	public void setOutputSize( int i ) {
		outputSize = i;
	}

	/**
	 * Sets the vendor
	 * 
	 * @param string new vendor string
	 */
	public void setVendor( String string ) {
		vendor = string;
	}
	
	/**
	 * Returns a string representation.
	 * @return string representation
	 */
	public String toString() {
		return getName();
	}
	
}
