
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:33 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: ResourceUtils.java,v $
 *
 */

package org.evolvica.resources;

import java.util.Locale;
import java.util.ResourceBundle;

/** Resource handling utilities.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:58:33 $
 */
public class ResourceUtils {
	
	/**
	 * Current locale.
	 */
	public final static Locale LOCALE = Locale.getDefault();
	
	/**
	 * Default locale.
	 */
	public final static Locale DEFAULT = Locale.ENGLISH;
	
	private static StringBuffer pathBuffer = new StringBuffer();
	
	private static StringBuffer keyBuffer = new StringBuffer();
	
	private ResourceUtils() {
		// do not instantiate class
	}
	
	private static String getPath( Class clazz ) {
		pathBuffer.delete( 0, pathBuffer.length() );
		pathBuffer.append( clazz.getName() );
		pathBuffer.delete( pathBuffer.lastIndexOf( "." ), pathBuffer.length() ); //$NON-NLS-1$
		pathBuffer.append( ".resource" ); //$NON-NLS-1$
		String packageName = clazz.getPackage().getName();
		packageName = packageName.substring( packageName.lastIndexOf( '.' ) );
		pathBuffer.append( packageName );
		return pathBuffer.toString();
	}
	
	private static String getKey( Class clazz, String key ) {
		keyBuffer.delete( 0, keyBuffer.length() );
		keyBuffer.append( clazz.getName() );
		keyBuffer.delete( 0, keyBuffer.lastIndexOf( "." ) + 1 ); //$NON-NLS-1$
		keyBuffer.append( "." ); //$NON-NLS-1$
		keyBuffer.append( key );
		return keyBuffer.toString();
	}
	
	/**
	 * Returns the default resource string for the given key.
	 * 
	 * @param key resource key
	 * @return resource string
	 */
	public static String getDefault( String key ) {
		Class clazz = getCallingClass();
		return ResourceBundle.getBundle( getPath( clazz ), DEFAULT ).getString( getKey( clazz, key ) );
	}
	
	/**
	 * Returns the locale resource string for the given key.
	 * 
	 * @param key resource key
	 * @return resource string
	 */
	public static String locale( String key ) {
		Class clazz = getCallingClass();
		return ResourceBundle.getBundle( getPath( clazz ), LOCALE ).getString( getKey( clazz, key ) );
	}
	
	/**
	 * Returns the locale resource string for the given key and substitutes %0.
	 * 
	 * @param key resource key
	 * @param subst0 substitution 0
	 * @return resource string
	 */
	public static String locale( String key, String subst0 ) {
		Class clazz = getCallingClass();
		String message = ResourceBundle.getBundle( getPath( clazz ), LOCALE ).getString( getKey( clazz, key ) );
		return substitute( message, 0, subst0 );
	}
	
	/**
	 * Returns the locale resource string for the given key and substitutes %0 and %1.
	 * 
	 * @param key resource key
	 * @param subst0 substitution 0
	 * @param subst1 substitution 1
	 * @return resource string
	 */
	public static String locale( String key, String subst0, String subst1 ) {
		Class clazz = getCallingClass();
		String message = ResourceBundle.getBundle( getPath( clazz ), LOCALE ).getString( getKey( clazz, key ) );
		message = substitute( message, 0, subst0 );
		return substitute( message, 1, subst1 );
	}
	
	/**
	 * Returns the locale resource string for the given key and substitutes %0, %1 and %2.
	 * 
	 * @param key resource key
	 * @param subst0 substitution 0
	 * @param subst1 substitution 1
	 * @param subst2 substitution 2
	 * @return resource string
	 */
	public static String locale( String key, String subst0, String subst1, String subst2 ) {
		Class clazz = getCallingClass();
		String message = ResourceBundle.getBundle( getPath( clazz ), LOCALE ).getString( getKey( clazz, key ) );
		message = substitute( message, 0, subst0 );
		message = substitute( message, 1, subst1 );
		return substitute( message, 2, subst2 );
	}
	
	/**
	 * Returns the locale resource string for the given key and substitutes %0 to %n.
	 * 
	 * @param key resource key
	 * @param subst array with substitution string
	 * @return resource string
	 */
	public static String locale( String key, String[] subst ) {
		Class clazz = getCallingClass();
		String message = ResourceBundle.getBundle( getPath( clazz ), LOCALE ).getString( getKey( clazz, key ) );
		for ( int i = 0; i < subst.length; i++ ) {
			message = substitute( message, i, subst[i] );
		}
		return message;
	}
	
	private static final Class getCallingClass( ) {
		StackTraceElement caller = new Throwable().getStackTrace()[2];
		try {
			return Class.forName( caller.getClassName().toString() );
		} catch ( Exception ex ) {
			return ResourceUtils.class;
		}
	}
	
	private static final String substitute( String message, int index, String substitution ) {
		StringBuffer buffer = new StringBuffer( message );
		String searchString = "%" + index; //$NON-NLS-1$
		int sIndex = buffer.indexOf( searchString );
		while ( sIndex > 0 ) {
			buffer.replace( sIndex, sIndex + searchString.length(), substitution );
			sIndex = buffer.indexOf( searchString );
		}
		return buffer.toString();
	}
	
}
