/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:08 $
 * $Revision: 1.25 $
 * $State: Exp $
 * $RCSfile: AlgorithmEditor.java,v $
 *
 */

package org.evolvica.ui;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.net.URI;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.draw2d.FigureCanvas;
import org.eclipse.gef.*;
import org.eclipse.gef.dnd.AbstractTransferDropTargetListener;
import org.eclipse.gef.editparts.FreeformGraphicalRootEditPart;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.parts.GraphicalEditorWithPalette;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
import org.eclipse.ui.views.navigator.LocalSelectionTransfer;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.evolvica.engine.*;
import org.evolvica.ui.action.*;
import org.evolvica.ui.parts.GraphicalPartFactory;
import org.evolvica.ui.parts.OperatorEditPart;
import org.evolvica.ui.views.navigator.OperatorItem;
import org.evolvica.ui.views.outline.AlgorithmOutlinePage;
import org.evolvica.ui.views.properties.AlgorithmPropertiesSheetPage;
import org.evolvica.util.Metadata;

/** Documentation missing.
 *
 * @author rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.25 $ $Date: 2004/05/26 11:53:08 $
 */
public class AlgorithmEditor extends GraphicalEditorWithPalette implements ISelectionChangedListener {
	
	protected IAlgorithm algorithm = null;
	protected AlgorithmEditorPalette palette = null;
	protected AlgorithmOutlinePage outline = null;
	protected AlgorithmPropertiesSheetPage propertiesPage = null;
	protected IJavaProject project = null;
	protected IFile file = null;
	protected boolean dirty = false;
	
	public AlgorithmEditor() {
  	setEditDomain( new DefaultEditDomain( this ) );
  }

  /**
   * @see org.eclipse.ui.IEditorPart#init(IEditorSite, IEditorInput)
   **/
  public void init( IEditorSite site, IEditorInput input ) throws PartInitException {
		setSite( site );
		setInput( input );
		createActions();
		try {
			// get file and java project
			file = ((FileEditorInput) input).getFile();
			project = JavaCore.create( file.getProject() );
			// prepare a classloader for loading the algorithm
			URI[] classpath = PathManager.getClassPath( project );
			URL[] urlClasspath = new URL[classpath.length];
			for ( int i = 0; i < classpath.length; i++ ) {
				urlClasspath[i] = classpath[i].toURL();
			}
			ClassLoader parentLoader = AlgorithmEditorPlugin.getDefault().getDescriptor().getPluginClassLoader();
			URLClassLoader loader = URLClassLoader.newInstance( urlClasspath, parentLoader );
			// load the algorithm
			InputStream stream = file.getContents();
			algorithm = new AlgorithmBuilder().build( stream, loader );
			stream.close();
		} catch ( Exception ex ) {
			UILog.error( "error loading algorithm", ex );
			algorithm = new TemplateAlgorithm( "name" );
			MessageDialog.openError( getSite().getShell(), "Error", "unable to read algorithm, creating empty algorithm" );
		}
		setTitle( algorithm.getName() );
	}
	
	/**
	 * @see org.eclipse.gef.ui.parts.GraphicalEditor#configureGraphicalViewer()
	 **/
	protected void configureGraphicalViewer() {
		super.configureGraphicalViewer();
		// create outline
		outline = new AlgorithmOutlinePage( algorithm );
		// create properties sheet page
		propertiesPage = new AlgorithmPropertiesSheetPage();
		// viewer
		GraphicalViewer viewer = getGraphicalViewer();
		viewer.setRootEditPart( new FreeformGraphicalRootEditPart() );
		viewer.setEditPartFactory( GraphicalPartFactory.instance );
		viewer.addDropTargetListener( new OperatorDropTargetListener( viewer ) );
		viewer.addSelectionChangedListener( this );
		viewer.addSelectionChangedListener( outline );
		ContextMenuProvider provider = new OperatorContextMenuProvider( viewer, getActionRegistry() );
		viewer.setContextMenu( provider );
//		  getSite().registerContextMenu( "org.evolvica.ui.editor.contextmenu", provider, viewer );
		// If you don't put this line, then moving figures by drag & drop
		// above the left or top limit of the editor window will lead to
		// an infinite loop!
		((FigureCanvas)viewer.getControl()).setScrollBarVisibility( FigureCanvas.ALWAYS );
	}
  
  /**
   * @see org.eclipse.gef.ui.parts.GraphicalEditor#initializeGraphicalViewer()
   **/
  protected void initializeGraphicalViewer() {
		getGraphicalViewer().setContents( algorithm );
  }
  
  /**
   * @see org.eclipse.ui.IEditorPart#isSaveAsAllowed()
   **/
  public boolean isSaveAsAllowed() {
		return true;
  }

  /**
   * @see org.eclipse.ui.IEditorPart#doSave(IProgressMonitor)
   **/
  public void doSave( IProgressMonitor iMonitor ) {
  	try {
			ByteArrayOutputStream output = new ByteArrayOutputStream();
			new AlgorithmWriter().write( algorithm, output );
			ByteArrayInputStream input = new ByteArrayInputStream( output.toByteArray() );
			if ( file.exists() ) {
				file.setContents( input, true, false, null );
			} else {
				file.create( input, true, null );
			}
			file.refreshLocal( IResource.DEPTH_ONE, null );
			output.close();
			dirty = false;
			firePropertyChange( PROP_DIRTY );
  	} catch ( Exception ex ) {
  		ex.printStackTrace();
  	}
  }

  /**
   * @see org.eclipse.ui.IEditorPart#doSaveAs()
   **/
  public void doSaveAs() {
		InputDialog dialog = new InputDialog(
			getSite().getShell(),
			"Save Algorithm as ...",
			"Enter a new name for the algorithm",
			algorithm.getName() + "1",
			AlgorithmNameValidator.instance );
		int open = dialog.open();
		if ( open == InputDialog.OK ) {
			String newName = dialog.getValue();
			IPath newPath = file.getFullPath().removeFirstSegments( 1 ).removeLastSegments( 1 ).append( newName ).addFileExtension( "eva" );
			IFile newFile = project.getProject().getFile( newPath );
			boolean write = true;
			if ( newFile.exists() ) {
				write = MessageDialog.openQuestion( getSite().getShell(), "Question", "Algorithm file already exists, overwrite?" );
			}
			if ( write == true ) {
				setInput( new FileEditorInput( newFile ) );
				setTitle( newName );
				algorithm.setName( newName );
				file = newFile;
				doSave( null );
			}
		}
  }

  /**
   * @see org.eclipse.ui.IEditorPart#gotoMarker(IMarker)
   **/
  public void gotoMarker( IMarker iMarker ) {
		System.out.println( "called gotoMarker( " + iMarker + " )" );
  }

  /**
   * @see org.eclipse.ui.IEditorPart#isDirty()
	 **/
  public boolean isDirty() {
  	return dirty;
	}
	
	public void setDirty( boolean dirty ) {
		this.dirty = dirty;
		outline.refresh();
		firePropertyChange( PROP_DIRTY );
	}
	
	/**
	 * @see org.eclipse.gef.ui.parts.GraphicalEditorWithPalette#getPaletteRoot()
	 */
	protected PaletteRoot getPaletteRoot() {
		if ( palette == null ) {
			palette = new AlgorithmEditorPalette( project );
		}
		return palette;
	}
	
	public AlgorithmEditorPalette getPalette() {
		return (AlgorithmEditorPalette) getPaletteRoot();
	}
	
	public IJavaProject getJavaProject() {
		return project;
	}
	
	public IAlgorithm getAlgorithm() {
		return algorithm;
	}
	
	protected void createActions() {
		super.createActions();
		ActionRegistry registry = getActionRegistry();
		// add all actions to registry
		registry.registerAction( new DeleteAction( this ) );
		registry.registerAction( new MirrorShapeAction( this ) );
		registry.registerAction( new RenameAction( this ) );
		registry.registerAction( new SetColorAction( this ) );
		registry.registerAction( new SetInputAction( this ) );
		registry.registerAction( new SetOutputAction( this ) );
		// add actions to selection actions
		getSelectionActions().add( MirrorShapeAction.ID );
		getSelectionActions().add( DeleteAction.ID );
		getSelectionActions().add( RenameAction.ID );
		getSelectionActions().add( SetColorAction.ID );
		getSelectionActions().add( SetInputAction.ID );
		getSelectionActions().add( SetOutputAction.ID );
  }
	
	public void selectionChanged( SelectionChangedEvent event ) {
		// bring the properties view to front
		IViewPart viewPart = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().findView( IPageLayout.ID_PROP_SHEET );
		if ( viewPart != null ) {
			PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().bringToTop( viewPart );
		}
		// update actions
		ActionRegistry registry = getActionRegistry();
		List actionList = new LinkedList();
		for ( Iterator it = registry.getActions(); it.hasNext(); ) {
			actionList.add( ((IAction) it.next()).getId() );
		}
		updateActions( actionList );
	}
	
	public void select( Object object ) {
		if ( object instanceof IAlgorithm ) {
			getGraphicalViewer().select( getGraphicalViewer().getRootEditPart().getContents() );
		} else if ( object instanceof IElement ) {
			List children = getGraphicalViewer().getRootEditPart().getContents().getChildren();
			for ( Iterator it = children.iterator(); it.hasNext(); ) {
				OperatorEditPart part = (OperatorEditPart) it.next();
				if ( part.getElement() == object ) {
					getGraphicalViewer().select( part );
					return;
				}
			}
		}
	}
	
	/**
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter( Class type ) {
		if ( type.equals( IContentOutlinePage.class ) ) {
			return outline;
		} else if ( type.equals( IPropertySheetPage.class ) ) {
			return propertiesPage;
		}
		return super.getAdapter( type );
	}

	protected class OperatorDropTargetListener extends AbstractTransferDropTargetListener {
		
		public OperatorDropTargetListener( EditPartViewer viewer ) {
			super( viewer, LocalSelectionTransfer.getInstance() );
		}
		
		protected Request createTargetRequest() {
			return new CreateRequest();
		}
		
		protected void handleDragOver() {
			getCurrentEvent().detail = DND.DROP_COPY;
		}
		
		protected void handleDrop() {
			StructuredSelection selection = (StructuredSelection) LocalSelectionTransfer.getInstance().getSelection();
			OperatorItem item = (OperatorItem) selection.getFirstElement();
			OperatorDescriptor descriptor = ((OperatorDescriptor) item.getItem());
			IJavaProject project = AlgorithmEditor.this.project;
			try {
				IOperator operator = OperatorCreationFactory.createOperator( descriptor, project );
				IElement element = ElementFactory.newElement( operator );
				AlgorithmEditor.this.getAlgorithm().addElement( element );
				element.setMetadata( new Metadata( UIEvents.ELEMENT_X, getDropLocation().x ) );
				element.setMetadata( new Metadata( UIEvents.ELEMENT_Y, getDropLocation().y ) );
			} catch ( Exception ex ) {
				UILog.error( "error instantiating operator " + descriptor.getBeanClassName() + " in project " + project.getElementName(), ex );
				String message = ex.getMessage();
				if ( message == null ) {
					message = "unknown, please consult the logfile for more information";
				}
				MessageDialog.openError( AlgorithmEditor.this.getSite().getShell(), "Error",
				"error instantiating operator " + descriptor.getBeanClassName() + ", reason: " + ex.getMessage() );
			}
			/*
			((CreateRequest) getTargetRequest()).setLocation( getDropLocation() );
			((CreateRequest) getTargetRequest()).setFactory( new OperatorCreationFactory( descriptor, AlgorithmEditor.this.project ) );
			*/
		}
		
		public boolean isEnabled( DropTargetEvent event ) {
			return true;
		}
		
		protected void updateTargetRequest() {
			((CreateRequest) getTargetRequest()).setLocation( getDropLocation() );
		}

	}
	
	private static class AlgorithmNameValidator implements IInputValidator {
		
		private static final AlgorithmNameValidator instance = new AlgorithmNameValidator();

		public String isValid( String newText ) {
			if ( StringUtils.isEmpty( newText ) ) {
				return "algorithm name may not be empty";
			} else if ( ! StringUtils.isAlphanumeric( newText ) ) {
				return "algorithm name may only contain alpha-numeric characters";
			}
			return null;
		}
		
	}
	
}

