/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:11 $
 * $Revision: 1.9 $
 * $State: Exp $
 * $RCSfile: DiskCache.java,v $
 * 
 */
package org.evolvica.ui;

import java.io.*;
import java.net.URI;
import java.util.*;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.SystemUtils;
import org.eclipse.core.runtime.IPath;
import org.evolvica.engine.DescriptorBuilder;
import org.evolvica.engine.OperatorDescriptor;
import org.evolvica.ui.prefs.PreferenceDefaults;
import org.evolvica.util.Digester;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.9 $ $Date: 2004/05/26 11:53:11 $
 */
public class DiskCache {
	
	private static final HashMap cache = new HashMap(); // map from unique key to CacheEntry
	
	private static final HashSet updatedKeys = new HashSet();
	
	private static HashMap locations = new HashMap(); // map from a URI to a unique key
	
	public static final void load() {
		IPath stateLocation = AlgorithmEditorPlugin.getDefault().getStateLocation();
		// load all the cachefiles
		File stateDirectory = stateLocation.toFile();
		File[] cacheFiles = stateDirectory.listFiles( new FileFilter() {
			public boolean accept( File f ) {
				if ( f.getName().endsWith( ".cache" ) && StringUtils.countMatches( f.getName(), "-" ) == 4 ) {
					return true;
				} else {
					return false;
				}
			}
		} );
		for ( int i = 0; i < cacheFiles.length; i++ ) {
			IPath cacheLocation = stateLocation.append( cacheFiles[i].getName() );
			File cacheFile = cacheLocation.toFile();
			String key = StringUtils.chomp( cacheFile.getName(), "." );
			try {
				ObjectInputStream stream = new ObjectInputStream( new BufferedInputStream( new FileInputStream( cacheFile ) ) );
				CacheEntry cacheEntry = (CacheEntry) stream.readObject();
				cache.put( key, cacheEntry );
				locations.put( cacheEntry.location, key );
				stream.close();
			} catch ( Exception ex ) {
				UILog.error( "error reading cachefile " + cacheFile.getName() );
				// outdatedKeys.add( key ); // schedule corrupt file for removal
			}
		}
	}
	
	public static final void save() {
		IPath stateLocation = AlgorithmEditorPlugin.getDefault().getStateLocation();
		// save updated keys
		for ( Iterator it = updatedKeys.iterator(); it.hasNext(); ) {
			String key = (String) it.next();
			CacheEntry cacheEntry = (CacheEntry) cache.get( key );
			IPath cacheLocation = stateLocation.append( key + ".cache" );
			File cachefile = cacheLocation.toFile();
			try {
				ObjectOutputStream stream = new ObjectOutputStream( new BufferedOutputStream( new FileOutputStream( cachefile ) ) );
				stream.writeObject( cacheEntry );
				stream.close();
				UILog.debug( "updated cachefile " + cachefile.getName() );
			} catch ( IOException iox ) {
				UILog.error( "error writing cachefile " + cachefile.getName() );
			}
		}
	}
	
	public static final void clear() {
		cache.clear(); // throw away everything
		updatedKeys.clear();
		locations.clear();
	}
	
	public static final void clear( URI location ) {
		if ( locations.containsKey( location ) ) { // throw away the given location
			String key = (String) locations.get( location );
			cache.remove( key );
			updatedKeys.remove( key );
			locations.remove( location );
		}
	}
	
	public static final void refresh( URI location, URI[] srcLocations ) {
		clear( location );
		findDescriptors( location, srcLocations, true );
	}
	
	public static final OperatorDescriptor[] findDescriptors( URI location, URI[] srcLocations ) {
		return findDescriptors( location, srcLocations, false );
	}
	
	public static final OperatorDescriptor[] findDescriptors( URI location, URI[] srcLocations, boolean forceRefresh ) {
		if ( forceRefresh == false ) { // no refresh, we use the data from the cache
			if ( locations.containsKey( location ) ) { // get data from cache and return it
				UILog.debug( "cache hit for " + location );
				return ((CacheEntry) cache.get( locations.get( location ) )).toArray();
			} else {
				UILog.debug( "cache miss for " + location );
				return findDescriptors( location, srcLocations, true ); // there is no data in the cache for the given location -> restart
			}
		} else {
			String path = location.toString();
			if ( path.endsWith( "/" ) ) { // location is a directory
				UILog.debug( "updating cache for " + location );
				updateFromDirectory( location, srcLocations );
				return ((CacheEntry) cache.get( locations.get( location ) )).toArray();
			} else if ( path.endsWith( ".jar" ) || path.endsWith( ".zip" ) ) { // location is a zipfile
				UILog.debug( "updating cache for " + location );
				updateFromZipfile( location, srcLocations );
				return ((CacheEntry) cache.get( locations.get( location ) )).toArray();
			} else { // location is something else, ignore it!
				UILog.debug( "ignoring unknown descriptor location " + location );
				return new OperatorDescriptor[0];
			}
		}
	}
	
	private static final void updateFromDirectory( URI location, URI[] srcLocations ) {
		File toplevel = new File( location );
		LinkedList queue = new LinkedList();
		HashSet set = new HashSet();
		queue.add( toplevel );
		while( ! queue.isEmpty() ) {
			File next = (File) queue.getFirst();
			if ( next.isDirectory() ) {
				File[] children = next.listFiles();
				for ( int i = 0; i < children.length; i++ ) {
					queue.add( children[i] );
				}
			} else {
				if ( next.getName().endsWith( ".descriptor" ) ) {
					try {
						FileInputStream stream = new FileInputStream( next );
						OperatorDescriptor desc = DescriptorBuilder.build( stream );
						desc.setLocation( location, findSourceLocation( desc, srcLocations ) );
						set.add( desc );
						stream.close();
					} catch ( Exception ex ) {
						UILog.warn( "cannot load descriptor " + next, ex );
					}
				}
			}
			queue.removeFirst();
		}
		String key = Digester.getDigest( location.toString() );
		CacheEntry newEntry = new CacheEntry( location, key, set.toArray() );
		cache.put( key, newEntry );
		locations.put( location, key );
		updatedKeys.add( key );
	}
	
	private static final void updateFromZipfile( URI location, URI[] srcLocations ) {
		boolean useStamp = AlgorithmEditorPlugin.getDefault().getPreferenceStore().getBoolean( PreferenceDefaults.USE_MODIFICATION_STAMP );
		if ( useStamp == true ) {
			CacheEntry entry = (CacheEntry) cache.get( locations.get( location ) );
			if ( entry != null && new File( location ).lastModified() < entry.lastModified ) { // cache is newer than zipfile
				UILog.debug( "skipping examination of " + location + " because of older modification date" );
				return;
			}
		}
		HashSet set = new HashSet();
		ZipFile zipfile = null;
		try {
			zipfile = new ZipFile( new File( location ) );
		} catch ( IOException iox ) {
			UILog.error( "error accessing zipfile at " + location );
		}
		if ( zipfile != null ) {
			for( Enumeration enum = zipfile.entries(); enum.hasMoreElements(); ) {
				ZipEntry entry = (ZipEntry) enum.nextElement();
				if ( entry.getName().endsWith( ".descriptor" ) ) {
					try {
						URI uri = new URI( "jar", location.toString() + "!/" + entry.getName(), null );
						OperatorDescriptor desc = DescriptorBuilder.build( uri );
						desc.setLocation( location, findSourceLocation( desc, srcLocations ) );
						set.add( desc );
					} catch ( Exception ex ) {
						UILog.warn( "cannot load descriptor " + entry + " from " + location, ex );
					}
				}
			}
		}
		String key = Digester.getDigest( location.toString() );
		CacheEntry newEntry = new CacheEntry( location, key, set.toArray() );
		cache.put( key, newEntry );
		locations.put( location, key );
		updatedKeys.add( key );
	}
	
	// helper methods
	
	private static final URI findSourceLocation( OperatorDescriptor desc, URI[] srcLocations ) {
		for ( int i = 0; i < srcLocations.length; i++ ) {
			String srcPath = StringUtils.replace( desc.getBeanClassName(), ".", "/" );
			String descPath = StringUtils.replace( desc.getBeanClassName(), ".", "/" );
			URI srcLoc = URI.create( srcLocations[i].toString() + srcPath + ".java" );
			URI descLoc = URI.create( srcLocations[i].toString() + descPath + ".descriptor" );
			if ( new File( srcLoc ).exists() && new File( descLoc ).exists() ) {
				return srcLocations[i];
			}
		}
		return null;
	}
	
	private static final String guessJavaClass( File path, File file ) {
		String name = StringUtils.replaceOnce( file.getAbsolutePath(), path.getAbsolutePath(), "" );
		name = StringUtils.replaceOnce( name, SystemUtils.FILE_SEPARATOR, "" );
		name = StringUtils.replace( name, SystemUtils.FILE_SEPARATOR, "." );
		return name.substring( 0, name.lastIndexOf( ".descriptor" ) );
	}
	
	private static final String guessJavaClass( ZipEntry entry ) {
		String name = StringUtils.replace( entry.getName(), "/", "." );
		return name.substring( 0, name.lastIndexOf( ".descriptor" ) );
	}
	
	private static class CacheEntry implements Serializable {
		
		private String key = null;
		private URI location = null;
		private long lastModified = System.currentTimeMillis();
		private HashSet set = new HashSet();
		
		private CacheEntry( URI location, String key, Object[] descriptors ) {
			this.key = key;
			this.location = location;
			for ( int i = 0; i < descriptors.length; i++ ) {
				set.add( descriptors[i] );
			}
		}
		
		private OperatorDescriptor[] toArray() {
			return (OperatorDescriptor[]) set.toArray( new OperatorDescriptor[set.size()] );
		}
		
	}
	
}
