/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:23 $
 * $Revision: 1.9 $
 * $State: Exp $
 * $RCSfile: DescriptorEditor.java,v $
 * 
 */
package org.evolvica.ui.editors.operator;

import java.net.URI;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.*;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.*;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.part.FileEditorInput;
import org.evolvica.core.GenotypeRepository;
import org.evolvica.engine.*;
import org.evolvica.engine.AttributeDescriptor;
import org.evolvica.engine.DescriptorWriter;
import org.evolvica.engine.OperatorDescriptor;
import org.evolvica.ui.PathManager;
import org.evolvica.ui.SourceManager;
import org.evolvica.ui.UILog;
import org.evolvica.util.BitVector;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.9 $ $Date: 2004/05/26 11:53:23 $
 */
public class DescriptorEditor extends EditorPart {
	
	private static final Class[] genotypes = new Class[] {
		Object.class, Double.TYPE, double[].class, Float.TYPE, float[].class,
		Integer.TYPE, int[].class, Long.TYPE, long[].class, String.class, BitVector.class
	};
	
	private static final String[] columnTitles = new String[] {
		"name", "type", "display name", "expert", "readonly", "description"
	};
	
	private static final int[] columnStyles = new int[] {
		SWT.LEFT, SWT.LEFT, SWT.LEFT, SWT.CENTER, SWT.CENTER, SWT.LEFT
	};
	
	private OperatorDescriptor descriptor = null;
	private ICompilationUnit source = null;
	private IFile descriptorFile = null;
	private IFile sourceFile = null;

	private boolean dirty = false;
	
	private Group gOperator = null;
	private Group gProperties = null;
	
	private Label lClassname = null;
	private Text tCategory = null;
	private Text tDisplayname = null;
	private Text tDescription = null;
	private Text tVendor = null;
	private Combo cDatatype = null;
	private Combo cInputsize = null;
	private Combo cOutputsize = null;
	
	private TableViewer tViewer = null;
	
	// this constructor is called in stand-alone mode when opening a single descriptor file
	// it does not initialize anything, we have to wait for the input to be set
	public DescriptorEditor() {
		// do nothing
	}
	
	// this constructor is called from OperatorEditor
	// sourceInput is never NULL
	// descriptorInput is never NULL, but the underlying file may not exist!
	public DescriptorEditor( IFileEditorInput sourceInput, IFileEditorInput descriptorInput ) {
		this.sourceFile = sourceInput.getFile();
		if ( sourceFile.exists() ) {
			this.source = JavaUI.getWorkingCopyManager().getWorkingCopy( sourceInput );
		}
		this.descriptorFile = descriptorInput.getFile();
	}
	
	/**
	 * @see org.eclipse.ui.ISaveablePart#doSave(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void doSave( IProgressMonitor monitor ) {
		// set operator properties
		descriptor.setCategory( tCategory.getText() );
		descriptor.setDatatype( GenotypeRepository.getID( genotypes[cDatatype.getSelectionIndex()] ) );
		descriptor.setDisplayName( tDisplayname.getText() );
		descriptor.setShortDescription( tDescription.getText() );
		descriptor.setVendor( tVendor.getText() );
		descriptor.setInterface( cInputsize.getSelectionIndex(), cOutputsize.getSelectionIndex() );
		// set descriptor properties
		TableItem[] items = tViewer.getTable().getItems();
		for ( int i = 0; i < items.length; i++ ) {
			PropertyTableContentItem item = (PropertyTableContentItem) items[i].getData();
			AttributeDescriptor ad = descriptor.getAttribute( item.getName() );
			if ( ad == null ) {
				ad = new AttributeDescriptor( item.getName() );
				descriptor.addAttribute( ad );
			}
			ad.setDisplayName( item.getDisplayName() );
			ad.setShortDescription( item.getDescription() );
			ad.setExpert( item.isExpert() );
		}
		try {
			DescriptorWriter.write( descriptor, PathManager.getLocation( descriptorFile ) );
			dirty = false;
			firePropertyChange( PROP_DIRTY );
			IPath parentPath = sourceFile.getFullPath().removeFirstSegments( 1 ).removeLastSegments( 1 );
			sourceFile.getProject().getFile( parentPath ).refreshLocal( IResource.DEPTH_ONE, null );
		} catch ( Exception ex ) {
			UILog.error( "Error saving descriptor for operator " + descriptor.getBeanClassName(), ex );
			MessageDialog.openError( this.getSite().getShell(), "Error", "Error saving descriptor" );
		}
	}
	
	/**
	 * @see org.eclipse.ui.ISaveablePart#doSaveAs()
	 */
	public void doSaveAs() {
		
	}

	/**
	 * @see org.eclipse.ui.IEditorPart#gotoMarker(org.eclipse.core.resources.IMarker)
	 */
	public void gotoMarker( IMarker marker ) {
		
	}

	/**
	 * @see org.eclipse.ui.IEditorPart#init(org.eclipse.ui.IEditorSite, org.eclipse.ui.IEditorInput)
	 */
	public void init( IEditorSite site, IEditorInput input ) throws PartInitException {
		setSite( site );
		setInput( input );
		try {
			initResources( input );
		} catch ( Exception ex ) {
			UILog.error( "Error opening operator descriptor for editing", ex );
			throw new PartInitException( ex.getMessage() );
		}
		setTitle( source.getElementName() );
	}
	
	protected void initResources( IEditorInput input ) throws JavaModelException {
		if ( sourceFile == null || descriptorFile == null ) { // case 1: the editor has been called in stand-alone mode
			IFile file = ((FileEditorInput) input).getFile();
			if ( file.getFileExtension().equals( "descriptor" ) ) { // case 1a: descriptor file opened
				descriptorFile = file;
				IPath sourcePath = descriptorFile.getFullPath().removeFirstSegments( 1 ).removeFileExtension().addFileExtension( "java" );
				sourceFile = descriptorFile.getProject().getFile( sourcePath );
			} else { // case 1b: source file opened
				sourceFile = file;
				IPath descPath = sourceFile.getFullPath().removeFirstSegments( 1 ).removeFileExtension().addFileExtension( "descriptor" );
				descriptorFile = sourceFile.getProject().getFile( descPath );
			}
		}
		// now source & descriptor files have been determined, we can try to load the stuff ...
		// first: check for the source file
		if ( source != null ) { // we have been called from OperatorEditor, source file is already opened 
			// do nothing
		} else {
			if ( sourceFile.exists() ) { // source file exists: create a compilation unit
				source = JavaCore.createCompilationUnitFrom( sourceFile );
			} else { // source does not exist ... we better throw an exception (maybe we handle this case later ...) 
				throw new IllegalArgumentException( "cannot open descriptor without source file" );
			}
		}
		// second: load the descriptor (or create a new one if it does not exist)
		URI classLocation = PathManager.getOutputLocation( source.getJavaProject() );
		URI sourceLocation = PathManager.getLocation( source );
		String className = SourceManager.getType( source ).getFullyQualifiedName();
		if ( descriptorFile.exists() ) { // there's already a descriptor file: try to load it
			try {
				descriptor = DescriptorBuilder.build( PathManager.getLocation( descriptorFile ) );
				descriptor.setLocation( classLocation, sourceLocation );
				// don't know if we really need to resolve the descriptor ???
				// descriptor.resolve( PathManager.getClassPath( source.getJavaProject() ) );
			} catch ( Exception ex ) {
				descriptor = new OperatorDescriptor( className, classLocation, sourceLocation );
				UILog.error( "loading of descriptor " + descriptor.getBeanClassName() + " failed", ex );
			}
		} else {
			descriptor = new OperatorDescriptor( className, classLocation, sourceLocation );
		}
	}
	
	/**
	 * @see org.eclipse.ui.ISaveablePart#isDirty()
	 */
	public boolean isDirty() {
		return dirty;
	}
	
	public void setDirty( boolean dirty ) {
		this.dirty = true;
		firePropertyChange( IEditorPart.PROP_DIRTY );
	}
	
	/**
	 * @see org.eclipse.ui.ISaveablePart#isSaveAsAllowed()
	 */
	public boolean isSaveAsAllowed() {
		return false;
	}

	/**
	 * @see org.eclipse.ui.IWorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl( Composite parent ) {
		parent.setLayout( new GridLayout( 1, true ) );
		parent.setLayoutData( new GridData( GridData.FILL_HORIZONTAL ) );
		// create two subgroups
		gOperator = new Group( parent, SWT.NONE );
		gOperator.setText( "Operator" );
		gOperator.setLayout( new GridLayout() );
		gOperator.setLayoutData( new GridData( GridData.FILL_HORIZONTAL) );
		((GridLayout) gOperator.getLayout()).numColumns = 6;
		gProperties = new Group( parent, SWT.NONE );
		gProperties.setText( "Operator Properties" );
		gProperties.setLayout( new GridLayout() );
		gProperties.setLayoutData( new GridData( GridData.FILL_HORIZONTAL ) );
		((GridLayout) gProperties.getLayout()).numColumns = 2;
		createOperatorGroup( gOperator );
		createPropertyGroup( gProperties );
		initPartControlFromSource();
		initPartControlFromDescriptor();
	}
	
	protected void createOperatorGroup( Group parent ) {
		// row 1
		Label label = new Label( parent, SWT.NULL );
		label.setText( "Operator Class" );
		label.setLayoutData( new GridData( GridData.FILL_HORIZONTAL ) );
		lClassname = new Label( parent, SWT.BORDER | SWT.SINGLE );
		lClassname.setLayoutData( new GridData( GridData.FILL_HORIZONTAL ) );
		((GridData) lClassname.getLayoutData()).horizontalSpan = 5;
		((GridData) label.getLayoutData()).widthHint = 80;
		// row 2
		label = new Label( parent, SWT.NULL );
		label.setText( "Display Name" );
		tDisplayname = new Text( parent, SWT.BORDER | SWT.SINGLE );
		tDisplayname.setLayoutData( new GridData( GridData.FILL_HORIZONTAL ) );
		label = new Label( parent, SWT.NULL );
		label.setText( "Datatype" );
		cDatatype = new Combo( parent, SWT.BORDER | SWT.SINGLE | SWT.READ_ONLY );
		String[] typenames = new String[genotypes.length];
		for ( int i = 0; i < genotypes.length; i++ ) {
			typenames[i] = GenotypeRepository.getID( genotypes[i] ).getName();
		}
		cDatatype.setItems( typenames );
		cDatatype.select( 0 );
		cDatatype.setLayoutData( new GridData( GridData.FILL_HORIZONTAL ) );
		label = new Label( parent, SWT.NULL );
		label.setText( "Category" );
		tCategory = new Text( parent, SWT.BORDER | SWT.SINGLE );
		tCategory.setLayoutData( new GridData( GridData.FILL_HORIZONTAL ) );
		((GridData) tDisplayname.getLayoutData()).widthHint = 160;
		((GridData) cDatatype.getLayoutData()).widthHint = 160;
		((GridData) tCategory.getLayoutData()).widthHint = 160;
		// row 3
		label = new Label( parent, SWT.NULL );
		label.setText( "Input Size" );
		cInputsize = new Combo( parent, SWT.BORDER | SWT.SINGLE | SWT.READ_ONLY );
		cInputsize.setLayoutData( new GridData( GridData.FILL_HORIZONTAL ) );
		cInputsize.setItems( new String[] { "0", "1", "2", "3", "4", "5", "6", "7", "8" } );
		label = new Label( parent, SWT.NULL );
		label.setText( "Output Size" );
		cOutputsize = new Combo( parent, SWT.BORDER | SWT.SINGLE | SWT.READ_ONLY );
		cOutputsize.setLayoutData( new GridData( GridData.FILL_HORIZONTAL ) );
		cOutputsize.setItems( new String[] { "0", "1", "2", "3", "4", "5", "6", "7", "8" } );
		label = new Label( parent, SWT.NULL );
		label.setText( "Vendor" );
		tVendor = new Text( parent, SWT.BORDER | SWT.SINGLE );
		tVendor.setLayoutData( new GridData( GridData.FILL_HORIZONTAL ) );
		// row 4
		label = new Label( parent, SWT.NULL );
		label.setText( "Short Description" );
		tDescription = new Text( parent, SWT.BORDER | SWT.SINGLE );
		tDescription.setLayoutData( new GridData( GridData.FILL_HORIZONTAL ) );
		((GridData) tDescription.getLayoutData()).horizontalSpan = 5;
	}
	
	protected void createPropertyGroup( Group parent ) {
		tViewer = new TableViewer( parent, SWT.SINGLE | SWT.FULL_SELECTION );
		for ( int i = 0; i < columnTitles.length; i++ ) {
			TableColumn tableColumn = new TableColumn( tViewer.getTable(), columnStyles[i] );
			tableColumn.setText( columnTitles[i] );
		}
		// table appearance
		tViewer.getTable().setHeaderVisible( true );
		tViewer.getTable().setLinesVisible( true );
		tViewer.getTable().setBackground( new Color( null, 222, 222, 222 ) );
		// table layout
		/* ... <ggrrrr>, doesn't work
		TableLayout layout = new TableLayout();
		layout.addColumnData( new ColumnWeightData( 10, true ) );
		layout.addColumnData( new ColumnWeightData( 10, true ) );
		layout.addColumnData( new ColumnWeightData( 10, true ) );
		layout.addColumnData( new ColumnWeightData( 5, true ) );
		layout.addColumnData( new ColumnWeightData( 5, true ) );
		layout.addColumnData( new ColumnWeightData( 60, true ) );
		tViewer.getTable().setLayout( layout );
		*/
		// table layout V2
		GridData gridData = new GridData( GridData.FILL_BOTH );
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		tViewer.getTable().setLayoutData( gridData );
		// pack the columns
		for ( int i = 0; i < tViewer.getTable().getColumnCount(); i++ ) {
			tViewer.getTable().getColumn( i ).pack();
		}
		// table content setup
		tViewer.setColumnProperties( columnTitles );
		tViewer.setContentProvider( new PropertyTableContentProvider() );
		tViewer.setLabelProvider( new PropertyTableLabelProvider() );
		tViewer.setCellEditors( new CellEditor[] {
			null,
			null,
			new TextCellEditor( tViewer.getTable() ),
			new CheckboxCellEditor( tViewer.getTable() ),
			null,
			new TextCellEditor( tViewer.getTable() )
		} );
		tViewer.setCellModifier( new PropertyTableCellModifier( this, tViewer ) );
	}
	
	protected void initPartControlFromSource() {
		if ( source != null ) {
			// init elements in operator group
			lClassname.setText( SourceManager.getQualifiedName( source ) );
			tCategory.setText( "misc" );
			cDatatype.select( 0 );
			tDisplayname.setText( SourceManager.getName( source ) );
			tDescription.setText( "no description available" );
			tVendor.setText( "Evolvica.org" );
			cInputsize.select( 1 );
			cOutputsize.select( 1 );
			// init table
			tViewer.setInput( new Object[] { source, descriptor } );
		}
	}
	
	protected void initPartControlFromDescriptor() {
		if ( descriptor != null ) {
			tCategory.setText( descriptor.getCategory() );
			if ( descriptor.getDatatype() != null ) {
				for ( int i = 0; i < genotypes.length; i++ ) {
					if ( descriptor.getDatatype().getReferencedClass().equals( genotypes[i] ) ) {
						cDatatype.select( i );
						break;
					}
				}
			}
			tDisplayname.setText( descriptor.getDisplayName() );
			tDescription.setText( descriptor.getShortDescription() );
			tVendor.setText( descriptor.getVendor() );
			cInputsize.select( descriptor.getInputSize() );
			cOutputsize.select( descriptor.getOutputSize() );
		}
		Control[] children = gOperator.getChildren();
		for ( int i = 0; i < children.length; i++ ) {
			if ( children[i] instanceof Text ) {
				((Text) children[i]).addModifyListener( new InputChangeListener() );
			} else if ( children[i] instanceof Combo ) {
				((Combo) children[i]).addModifyListener( new InputChangeListener() );
			}
		}
	}
	
	public void setFocus() {
		// do nothing ?
	}
	
	public void updatePartControl() {
		if ( SourceManager.isOperator( source ) ) {
			enableControls();
			updatePartControlFromSource();
		} else {
			disableControls();
		}
	}
	
	protected void updatePartControlFromSource() {
		lClassname.setText( SourceManager.getQualifiedName( source ) );
		tViewer.refresh();
		Table t = tViewer.getTable();
		for ( int i = 0; i < t.getColumnCount(); i++ ) {
			t.getColumn( i ).pack();
		}
		// try to resize the last column
		int width = t.getSize().x;
		int columnWidth = 0;
		int columnCount = t.getColumnCount();
		for ( int i = 0; i < columnCount; i++ ) {
			if ( i != columnCount - 1 ) {
				columnWidth += t.getColumn( i ).getWidth();
			}
		}
		int lastColumnWidth = width - columnWidth;
		if ( lastColumnWidth > 0 ) {
			t.getColumn( columnCount - 1 ).setWidth( lastColumnWidth );
		}
		// try to resize the table
		int tableWidth = t.getSize().x;
		int tableHeight = t.getHeaderHeight() + t.getItemHeight() * (t.getItems().length + 1);
		t.setSize( tableWidth, tableHeight );
		// in addition the surrounding group must be resized
		gProperties.setSize( gProperties.getSize().x, tableHeight + 24 );
	}
	
	protected void disableControls() {
		cDatatype.setEnabled( false );
		tCategory.setEnabled( false );
		tDescription.setEnabled( false );
		tDisplayname.setEnabled( false );
		cInputsize.setEnabled( false );
		cOutputsize.setEnabled( false );
		tViewer.getTable().setEnabled( false );
		tVendor.setEnabled( false );
	}
	
	protected void enableControls() {
		cDatatype.setEnabled( true );
		tCategory.setEnabled( true );
		tDescription.setEnabled( true );
		tDisplayname.setEnabled( true );
		cInputsize.setEnabled( true );
		cOutputsize.setEnabled( true );
		tViewer.getTable().setEnabled( true );
		tVendor.setEnabled( true );
	}
	
	private class InputChangeListener implements ModifyListener, SelectionListener {
		
		public void modifyText( ModifyEvent e ) {
			dirty = true;
			firePropertyChange( PROP_DIRTY );
		}
		
		public void widgetDefaultSelected( SelectionEvent e ) {
			// do nothing
		}

		public void widgetSelected( SelectionEvent e ) {
			dirty = true;
			firePropertyChange( PROP_DIRTY );
		}

	}
	
}
