/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:30 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: ProjectNavigator.java,v $
 *
 */
package org.evolvica.ui.views.navigator;

import java.util.HashSet;

import org.apache.commons.lang.StringUtils;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jface.action.*;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.*;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseTrackAdapter;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.*;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.part.MultiEditorInput;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.views.navigator.LocalSelectionTransfer;
import org.evolvica.engine.OperatorDescriptor;
import org.evolvica.ui.views.navigator.actions.*;

/** Documentation missing.
 *
 * @author rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:53:30 $
 */
public class ProjectNavigator extends ViewPart implements ISelectionProvider {
	
	private static final String STATE = "expand";
	
	public static boolean FORCE_REFRESH = false; // IMPROVE: workaround to enable a forced refresh
	
	private ResourceChangeHandler resourceChangeHandler = new ResourceChangeHandler( this );
	
	protected TreeViewer viewer;
	protected IMemento memento = null;
	protected MenuManager menuManager = new MenuManager();
	
	protected HashSet selectionListeners = new HashSet();
	
	public void init( IViewSite site, IMemento memento ) throws PartInitException {
		super.init( site, memento );
		this.memento = memento;
		ResourcesPlugin.getWorkspace().addResourceChangeListener( resourceChangeHandler );
	}
	
	public void dispose() {
		super.dispose();
		ResourcesPlugin.getWorkspace().removeResourceChangeListener( resourceChangeHandler );
	}

	/**
	 * @see ViewPart#createPartControl
	 */
	public void createPartControl( Composite parent ) {
		viewer = new TreeViewer( parent, SWT.H_SCROLL | SWT.V_SCROLL );
		viewer.setContentProvider( new ResourceTreeContentProvider() );
		viewer.setLabelProvider( new ResourceLabelProvider() );
		viewer.setInput( ResourcesPlugin.getWorkspace().getRoot() );
		viewer.addDoubleClickListener( new DoubleClickListener() );
		viewer.getTree().addMouseTrackListener( new TooltipTrackListener() );
		viewer.addDragSupport(
			DND.DROP_COPY,
			new Transfer[] { LocalSelectionTransfer.getInstance() },
			new DragAdapter()
		);
		getViewSite().registerContextMenu( menuManager, viewer );
		createContextMenu();
		restoreState( memento );
	}

	private void createContextMenu() {
		MenuManager manager = new MenuManager();
		manager.setRemoveAllWhenShown( true );
		manager.addMenuListener( new IMenuListener() {
			public void menuAboutToShow( IMenuManager mgr ) {
				fillContextMenu( mgr );
			}
		} );
		Menu menu = manager.createContextMenu( viewer.getControl() );
		viewer.getControl().setMenu( menu );
		getSite().registerContextMenu( manager, viewer );
	}
	
	private void fillContextMenu( IMenuManager manager ) {
		// create manager for submenu
		MenuManager newManager = new MenuManager( "New" );
		manager.add( newManager );
		fillNewContextMenu( newManager );
		manager.add( new Separator() );
		// add actions
		manager.add( new OpenAction( this ) );
		manager.add( new CloseAction( this ) );
		manager.add( new Separator() );
		manager.add( new CutAction( this ) );
		manager.add( new CopyAction( this ) );
		manager.add( new PasteAction( this ) );
		manager.add( new DeleteAction( this ) );
		manager.add( new RenameAction( this ) );
		manager.add( new Separator() );
		manager.add( new RefreshAction( this ) );
		manager.add( new ShowPropertiesAction( this ) );
		manager.add( new Separator() );
		manager.add( new GroupMarker( IWorkbenchActionConstants.MB_ADDITIONS ) );
	}
	
	private void fillNewContextMenu( IMenuManager manager ) {
		manager.add( new NewProjectAction( this ) );
		manager.add( new Separator() );
		manager.add( new NewAlgorithmAction( this ) );
		manager.add( new NewOperatorAction( this ) );
		manager.add( new Separator() );
		manager.add( new NewAction( this ) );
	}
	
	public IResourceItem getSelectedItem() {
		ISelection selection = viewer.getSelection();
		if ( selection instanceof IStructuredSelection ) {
			IResourceItem item = (IResourceItem) ((IStructuredSelection) selection).getFirstElement();
			return item;
		}
		return null;
	}
	
	public void restoreState( IMemento memento ) {
		if ( memento != null ) {
			IMemento[] children = memento.getChildren( STATE );
			for ( int i = 0; i < children.length; i++ ) {
				IResourceItem item = findItemByKey( children[i].getID() );
				if ( item != null ) {
					viewer.setExpandedState( item, true );
				}
			}
		}
	}
	
	public void saveState( IMemento memento ) {
		Object[] expanded = viewer.getExpandedElements();
		for ( int i = 0; i < expanded.length; i++ ) {
			memento.putMemento( memento.createChild( STATE, ((IResourceItem) expanded[i]).getKey() ) );
		}
	}
	
	private IResourceItem findItemByKey( String key ) {
		String[] tokens = StringUtils.split( key, "/" );
		TreeItem[] items = viewer.getTree().getItems();
		TreeItem item = null;
		for ( int i = 0; i < tokens.length; i++ ) {
			for ( int j = 0; j < items.length; j++ ) {
				String label = ResourceLabelProvider.instance.getText( items[j].getData() );
				if ( tokens[i].equals( label ) ) {
					item = items[j];
					break; // child found -> leave for loop
				}
			}
			if ( item != null ) {
				if ( i == tokens.length - 1 ) {
					return (IResourceItem) item.getData(); // final item found
				} else {
					items = item.getItems(); // -> examine new children in next loop
				}
			} else { // no matching child found -> stop
				return null;
			}
		}
		return null;
	}
	
	public void refresh() {
		FORCE_REFRESH = true;
		viewer.refresh();
		FORCE_REFRESH = false;
	}
	
	public void refresh( IResourceItem item ) {
		viewer.refresh( item );
	}
	
	public void setFocus() {
		
	}
	
	public void addSelectionChangedListener( ISelectionChangedListener listener ) {
		selectionListeners.add( listener );
	}

	public ISelection getSelection() {
		IResourceItem selectedItem = getSelectedItem();
		if ( selectedItem != null ) {
			if ( selectedItem instanceof ProjectFolder ) {
				return new StructuredSelection( selectedItem.getItem() ); 
			} else if ( selectedItem instanceof AlgorithmItem ) {
				return new StructuredSelection( selectedItem.getItem() );
			} else if ( selectedItem instanceof OperatorItem ) {
				return new StructuredSelection( selectedItem );
			}
		}
		return new StructuredSelection();
	}

	public void removeSelectionChangedListener( ISelectionChangedListener listener ) {
		selectionListeners.remove( listener );
	}

	public void setSelection( ISelection selection ) {
		System.out.println( "setSelection() called in ProjectNavigator" );
	}
	
	
	private class DoubleClickListener implements IDoubleClickListener {
		
		public void doubleClick( DoubleClickEvent event ) {
			ISelection selection = event.getSelection();
			if ( selection instanceof IStructuredSelection ) {
				Object element = ((IStructuredSelection) selection).getFirstElement();
				if ( viewer.isExpandable( element ) ) {
					if ( viewer.getExpandedState( element ) == true ) {
						viewer.collapseToLevel( element, 1 );
					} else {
						viewer.expandToLevel( element, 1 );
					}
				} else {
					try {
						IWorkbenchPage page = ProjectNavigator.this.getSite().getPage();
						if ( element instanceof AlgorithmItem ) {
							AlgorithmItem ai = (AlgorithmItem) element;
							page.openEditor( (IFile) ai.getItem(), "org.evolvica.ui.AlgorithmEditor", true );
						} else if ( element instanceof OperatorItem ) {
							OperatorItem operatorItem = (OperatorItem) element;
							if ( operatorItem.isLocal() ) {
								page.openEditor( createMultiEditorInput( operatorItem ), "org.evolvica.ui.editors.operator.OperatorEditor", true );
							}
						}
					} catch ( PartInitException pix ) {
						if ( pix.getMessage() != null ) {
							MessageDialog.openError( ProjectNavigator.this.getSite().getShell(), "Error", pix.getMessage() );
						} else {
							MessageDialog.openError( ProjectNavigator.this.getSite().getShell(), "Error", "Error opening editor" );
						}
					}
				}
			}
		}
		
		private MultiEditorInput createMultiEditorInput( OperatorItem item ) throws PartInitException {
			OperatorDescriptor descriptor = (OperatorDescriptor) item.getItem();
			String[] editorIDs = new String[] { "source editor", "descriptor editor" };
			IEditorInput[] innerEditors = new IEditorInput[2];
			IFile srcFile = null;
			IFile descriptorFile = null;
			try {
				IPackageFragmentRoot[] fragmentRoots = item.getJavaProject().getAllPackageFragmentRoots();
				for ( int i = 0; i < fragmentRoots.length; i++ ) {
					if ( fragmentRoots[i].getKind() == IPackageFragmentRoot.K_SOURCE ) {
						IPath path = fragmentRoots[i].getPath();
						path = path.addTrailingSeparator();
						path = path.append( StringUtils.replace( descriptor.getBeanClassName(), ".", "/" ) );
						path = path.removeFirstSegments( 1 );
						IPath srcPath = path.addFileExtension( "java" );
						IPath descriptorPath = path.addFileExtension( "descriptor" );
						srcFile = item.getProject().getFile( srcPath );
						descriptorFile = item.getProject().getFile( descriptorPath );
						if ( srcFile.exists() && descriptorFile.exists() ) {
							break;
						} else {
							srcFile = null;
							descriptorFile = null;
						}
					}
				}
			} catch ( JavaModelException jmx ) {
				throw new PartInitException( "unable to find Java source or descriptor of operator" );
			}
			if ( srcFile == null || descriptorFile == null ) {
				throw new PartInitException( "unable to find Java source or descriptor of operator" );
			}
			innerEditors[0] = new FileEditorInput( srcFile );
			innerEditors[1] = new FileEditorInput( descriptorFile );
			return new MultiEditorInput( editorIDs, innerEditors );
		}
		
	}
	
	private class DragAdapter extends DragSourceAdapter {
		
		public void dragFinished(DragSourceEvent event) {
			// no clean up: do nothing
		}

		public void dragSetData( DragSourceEvent event ) {
			event.data = "DUMMY"; // event.data may not be null, so set it to a dummy
			LocalSelectionTransfer.getInstance().setSelection( ProjectNavigator.this.viewer.getSelection() );
		}

		public void dragStart( DragSourceEvent event ) {
			if ( ProjectNavigator.this.getSelectedItem() instanceof OperatorItem ) {
				event.doit = true; // set the doit flag as a notification that the drag is permitted
			} else {
				event.doit = false;
			}
		}

	}
	
	private class TooltipTrackListener extends MouseTrackAdapter {
		
		public void mouseHover( MouseEvent e ) {
			String tooltipText = null;
			TreeItem treeItem = ProjectNavigator.this.viewer.getTree().getItem( new Point( e.x, e.y ) );
			if ( treeItem != null && treeItem.getData() instanceof OperatorItem ) {
				IResourceItem resourceItem = (IResourceItem) treeItem.getData();
				tooltipText = ((OperatorDescriptor) resourceItem.getItem()).getShortDescription();
			}
			ProjectNavigator.this.viewer.getTree().setToolTipText( tooltipText );
		}

	}

}
