/*******************************************************************************
 * Copyright (c) 2000, 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.evolvica.ui.wizards;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.*;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.internal.ui.util.ExceptionHandler;
import org.eclipse.jdt.internal.ui.wizards.ClassPathDetector;
import org.eclipse.jdt.ui.PreferenceConstants;
import org.eclipse.jdt.ui.wizards.JavaCapabilityConfigurationPage;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;

/**
 * As addition to the JavaCapabilityConfigurationPage, the wizard does an
 * early project creation (so that linked folders can be defined) and, if an
 * existing external location was specified, offers to do a classpath detection
 */
public class NewAlgorithmProjectCreationWizardPage extends JavaCapabilityConfigurationPage {
	
	private static String[] requiredLibs = new String[] {
		"EVOLVICA_CORE_LIB",
		"COMMONS_CLI_LIB",
		"COMMONS_COLLECTIONS_LIB",
		"COMMONS_BEANUTILS_LIB",
		"COMMONS_LANG_LIB",
		"COMMONS_LOGGING_LIB",
		"GNU_TROVE_LIB",
		"JSX_LIB",
		"LOG4J_LIB"
	};
	
	private WizardNewProjectCreationPage fMainPage;

	private IPath fCurrProjectLocation;
	protected IProject fCurrProject;
	
	protected boolean fCanRemoveContent;

	/**
	 * Constructor for NewProjectCreationWizardPage.
	 */
	public NewAlgorithmProjectCreationWizardPage(WizardNewProjectCreationPage mainPage) {
		super();
		fMainPage= mainPage;
		fCurrProjectLocation= null;
		fCurrProject= null;
		fCanRemoveContent= false;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#setVisible(boolean)
	 */
	public void setVisible(boolean visible) {
		if (visible) {
			changeToNewProject();
		} else {
			removeProject();
		}
		super.setVisible(visible);
	}
	
	private void changeToNewProject() {
		IProject newProjectHandle= fMainPage.getProjectHandle();
		IPath newProjectLocation= fMainPage.getLocationPath();
		
		if (fMainPage.useDefaults()) {
			fCanRemoveContent= !newProjectLocation.append(fMainPage.getProjectName()).toFile().exists();
		} else {
			fCanRemoveContent= !newProjectLocation.toFile().exists();
		}
				
		final boolean initialize= !(newProjectHandle.equals(fCurrProject) && newProjectLocation.equals(fCurrProjectLocation));
		
		IRunnableWithProgress op= new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
				try {
					updateProject(initialize, monitor);
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				} 
			}
		};
	
		try {
			getContainer().run(false, true, op);
		} catch (InvocationTargetException e) {
			String title= "Create New Project";
			String message= "Enter the name and the location of the project";
			ExceptionHandler.handle(e, getShell(), title, message);
		} catch  (InterruptedException e) {
			// cancel pressed
		}
	}
	
	protected void updateProject(boolean initialize, IProgressMonitor monitor) throws CoreException, InterruptedException {
		fCurrProject= fMainPage.getProjectHandle();
		fCurrProjectLocation= fMainPage.getLocationPath();
		boolean noProgressMonitor= !initialize && fCanRemoveContent;
		
		if (monitor == null || noProgressMonitor ) {
			monitor= new NullProgressMonitor();
		}
		try {
			monitor.beginTask( "NewProjectCreationWizardPage.EarlyCreationOperation.desc", 2);
			
			createProject(fCurrProject, fCurrProjectLocation, new SubProgressMonitor(monitor, 1));
			if (initialize) {
				IClasspathEntry[] entries = null;
				IPath outputLocation = null;
				if (fCurrProjectLocation.toFile().exists() && !Platform.getLocation().equals(fCurrProjectLocation)) {
					// detect classpath
					if (!fCurrProject.getFile(".classpath").exists()) { //$NON-NLS-1$
						// if .classpath exists no need to look for files
						ClassPathDetector detector= new ClassPathDetector(fCurrProject);
						entries= detector.getClasspath();
						outputLocation= detector.getOutputLocation();
					}
				}
				IClasspathEntry sourceEntry = JavaCore.newSourceEntry( fCurrProject.getFullPath().append( "src" ) );
				IClasspathEntry[] reqEntries = new IClasspathEntry[requiredLibs.length];
				for ( int i = 0; i < requiredLibs.length; i++ ) {
					IPath libpath = JavaCore.getClasspathVariable( requiredLibs[i] );
					IPath srcpath = new Path( libpath.removeFileExtension().toString() + "src.jar" );
					if ( ! srcpath.toFile().exists() ) {
						srcpath = new Path( libpath.removeFileExtension().toString() + "src.zip" );
					}
					reqEntries[i] = JavaCore.newLibraryEntry( libpath, srcpath, null, true );
				}
				IClasspathEntry[] jreEntries= PreferenceConstants.getDefaultJRELibrary();
				if ( entries == null ) { // add JRE + Evolvica core + dependent libraries
					entries = new IClasspathEntry[jreEntries.length + reqEntries.length + 1];
					for ( int i = 0; i < jreEntries.length; i++ ) {
						entries[i] = jreEntries[i];
					}
					for ( int i = 0; i < reqEntries.length; i++ ) {
						entries[i+jreEntries.length] = reqEntries[i];
					}
					entries[jreEntries.length+reqEntries.length] = sourceEntry;
				}
				if ( outputLocation == null ) {
					outputLocation = fCurrProject.getFullPath().append( "classes" );
				}
				init(JavaCore.create(fCurrProject), outputLocation, entries, false);
			}
			monitor.worked(1);
		} finally {
			monitor.done();
		}
	}
	
	/**
	 * Called from the wizard on finish.
	 */
	public void performFinish(IProgressMonitor monitor) throws CoreException, InterruptedException {
		try {
			monitor.beginTask( "NewProjectCreationWizardPage.createproject.desc", 3);
			if (fCurrProject == null) {
				updateProject(true, new SubProgressMonitor(monitor, 1));
			}
			configureJavaProject(new SubProgressMonitor(monitor, 2));
		} finally {
			monitor.done();
			fCurrProject= null;
		}
	}

	private void removeProject() {
		if (fCurrProject == null || !fCurrProject.exists()) {
			return;
		}
		
		IRunnableWithProgress op= new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
				boolean noProgressMonitor= Platform.getLocation().equals(fCurrProjectLocation);
				if (monitor == null || noProgressMonitor) {
					monitor= new NullProgressMonitor();
				}
				monitor.beginTask( "NewProjectCreationWizardPage.removeproject.desc", 3);

				try {
					fCurrProject.delete(fCanRemoveContent, false, monitor);
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				} finally {
					monitor.done();
					fCurrProject= null;
					fCanRemoveContent= false;
				}
			}
		};
	
		try {
			getContainer().run(false, true, op);
		} catch (InvocationTargetException e) {
			String title= "NewProjectCreationWizardPage.op_error.title";
			String message= "NewProjectCreationWizardPage.op_error_remove.message";
			ExceptionHandler.handle(e, getShell(), title, message);		
		} catch  (InterruptedException e) {
			// cancel pressed
		}
	}		
			
	/**
	 * Called from the wizard on cancel.
	 */
	public void performCancel() {
		removeProject();
	}

}
