/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:35 $
 * $Revision: 1.4 $
 * $State: Exp $
 * $RCSfile: BitVector.java,v $
 * 
 */
package org.evolvica.util;


/** Replacement for Sun's unflexible BitSet.
 * 
 * @since 0.1
 * @author Andreas Rummler
 * @version $Revision: 1.4 $ $Date: 2004/05/26 11:58:35 $
 */
public class BitVector implements Cloneable {

	/** Integer array for masking single bits with one.
	* This array has the size of 32 and contains the masks for bits 0 to 31.
	* Example: MASK_ONE[1] = 00000000000000000000000000000010 (LSB last).
	*/
	private static final int[] MASK_ONE = new int[] {
		1, 2, 4, 8, 16, 32, 64, 128,
		256, 512, 1024, 2048, 4096, 8192, 16384, 32768,
		65536, 131072, 262144, 524288, 1048576, 2097152, 4194304, 8388608,
		16777216, 33554432, 67108864, 134217728, 268435456, 536870912, 1073741824, -2147483648
	};
	
	/** Integer array for masking single bits with zero.
	* This array has the size of 32 and contains the masks for bits 0 to 31.
	* Example: MASK_ZERO[1] = 11111111111111111111111111111101 (LSB last).
	*/
	private static final int[] MASK_ZERO = new int[] {
		-2, -3, -5, -9, -17, -33, -65, -129,
		-257, -513, -1025, -2049, -4097, -8193, -16385, -32769,
		-65537, -131073, -262145, -524289, -1048577, -2097153, -4194305, -8388609,
		-16777217, -33554433, -67108865, -134217729, -268435457, -536870913, -1073741825, 2147483647
 };
	
	private int length;

	private int[] vector;
	
	private int nullMask; // mask for "nulling" the uppermost unused bits
	
	/**
	 * Constructor.
	 * Constructs a bitvector of length 1.
	 */
	public BitVector() {
		this( 1 );
	}
	
	/**
	 * Constructor.
	 * Constructs a bitvector of the given length.
	 * 
	 * @param length length of the bit vector, must be greater than 0
	 */
	public BitVector( int length ) {
		if ( length < 1 ) {
			throw new IllegalArgumentException( "bit vector length must be greater than 0" );
		}
		if ( length % 32 == 0 ) {
			vector = new int[length/32];
		} else {
			vector = new int[length/32+1];
		}
		this.length = length;
		calculateNullMask();
	}
	
	/**
	 * Copy Constructor.
	 * 
	 * @param bv bit vector to be copied
	 */
	public BitVector( BitVector bv ) {
		this.length = bv.length;
		this.nullMask = bv.nullMask;
		this.vector = new int[bv.vector.length];
		System.arraycopy( bv.vector, 0, this.vector, 0, bv.vector.length );
	}
	
	/**
	 * Constructs a bit vector from a given string pattern.
	 * MSB comes first.
	 * 
	 * @param pattern string pattern
	 */
	public BitVector( String pattern ) {
		this( pattern.length() );
		for ( int i = 0, j = pattern.length() - 1; i < pattern.length(); i++, j-- ) {
			if ( pattern.charAt( i ) == '1' ) {
				set( j );
			}
		}
	}
	
	/**
	 * Performs an AND conjunction.
	 * 
	 * @param bv second bit vector
	 * @return result of the AND conjunction
	 */
	public BitVector and( BitVector bv ) {
		BitVector rval = new BitVector();
		rval.length = this.length;
		rval.vector = new int[vector.length];
		for ( int i = 0; i < vector.length; i++ ) {
			rval.vector[i] = vector[i] & bv.vector[i];
		}
		return rval;
	}
	
	/**
	 * Clears the bit at the given index.
	 * 
	 * @param index index of the bit
	 */
	public void clear( int index ) {
		int block = index/32;
		vector[block] &= MASK_ZERO[index%32];
	}
	
	/**
	 * @see java.lang.Object#clone()
	 */
	public Object clone() {
		return new BitVector( this );
	}
	
	/**
	 * Returns the bit at the given position.
	 * 
	 * @param index index of the bit
	 * @return value of the bit at the given position
	 */
	public boolean get( int index ) {
		if ( (vector[index/32] & MASK_ONE[index%32]) == 0  ) {
			return false;
		} else {
			return true;
		}
	}
	
	/**
	 * Returns the value of the bit at the given position as an integer.
	 * 
	 * @param index index of the bit
	 * @return value of the bit as an integer
	 */
	public int getAsInteger( int index ) {
		if ( (vector[index/32] & MASK_ONE[index%32]) == 0  ) {
			return 0;
		} else {
			return 1;
		}
	}
	
	/**
	 * Inverts the whole vector.
	 */
	public void invert() {
		for ( int i = 0; i < vector.length; i++ ) {
			vector[i] = ~ vector[i];
		}
	}
	
	/**
	 * Returns the length of the bit vector.
	 * 
	 * @return length of the bitstring
	 */
	public int length() {
		return length;
	}
	
	/**
	 * Performs an OR function.
	 * 
	 * @param bv second bit vector
	 * @return result of the OR function
	 */
	public BitVector or( BitVector bv ) {
		BitVector rval = new BitVector();
		rval.length = this.length;
		rval.vector = new int[vector.length];
		for ( int i = 0; i < vector.length; i++ ) {
			rval.vector[i] = vector[i] | bv.vector[i];
		}
		return rval;
	}
	
	/**
	 * Randomizes the bitstring.
	 */
	public void randomize() {
		for ( int i = 0; i < vector.length; i++ ) {
			vector[i] = RandomUtils.randomInt( Integer.MAX_VALUE );
		}
		vector[vector.length - 1] &= nullMask; // force unused bits to 0
	}
	
	/**
	 * Sets the bit at the given index to true.
	 * 
	 * @param index index of the bit
	 */
	public void set( int index ) {
		int block = index/32;
		vector[block] |= MASK_ONE[index%32];
	}
	
	/**
	 * Sets the bit at the given index to the given value.
	 * 
	 * @param index index of the bit
	 * @param value value to set
	 */
	public void set( int index, boolean value ) {
		if ( value ) {
			set( index );
		} else {
			clear( index );
		}
	}
	
	/**
	 * Performs a shift left.
	 * 
	 */
	public void shiftLeft() {
		shiftLeft( 1 );
	}
	
	/**
	 * Performs a shift left by the specified amount.
	 * 
	 * @param amount amount to shift
	 */
	public void shiftLeft( int amount ) {
		while ( amount != 0 ) {
			if ( amount < 32 ) {
				for ( int i = vector.length - 1; i > 0; i-- ) {
					vector[i] <<= amount; // shift upper word upwards
					int tmp = vector[i-1] >>> (32 - amount); // shift lower word downwards
					vector[i] |= tmp;
				}
				vector[0] <<= amount; // shift the lowest word upwards
				amount = 0; // set amount to 0 to leave the while-loop
			} else {
				for ( int i = vector.length - 1; i > 0; i-- ) {
					vector[i] = vector[i-1];
				}
				vector[0] = 0;
				amount -= 32;
			}
		}
		vector[vector.length - 1] &= nullMask; // force unused bits to 0
	}
	
	/**
	 * Performs a shift right.
	 * 
	 */
	public void shiftRight() {
		shiftRight( 1 );
	}
	
	/**
	 * Performs a shift right by the specified amount.
	 * 
	 * @param amount amount to shift
	 */
	public void shiftRight( int amount ) {
		vector[vector.length - 1] &= nullMask; // force unused bits to 0
		while ( amount != 0 ) {
			if ( amount < 32 ) {
				for ( int i = 0; i < vector.length - 1; i++ ) {
					vector[i] >>>= amount; // shift lower word downwards
					int tmp = vector[i+1] << (32 - amount); // shift upper word upwards
					vector[i] |= tmp;
				}
				vector[vector.length-1] >>>= amount; // shift the most upper word downwards
				amount = 0; // set amount to 0 to leave the while-loop
			} else {
				for ( int i = 0; i < vector.length-1; i++ ) {
					vector[i] = vector[i+1];
				}
				vector[vector.length-1] = 0;
				amount -= 32;
			}
		}
	}
	
	/**
	 * Returns the bit vector as blocks of 8 bits.
	 * The MSB comes first in each block, the most significant blocks comes also first.
	 * 
	 * @return string representation
	 */
	public String toBlockString() {
		return toBlockString( 8 );
	}
	
	/**
	 * Returns the bit vector as blocks of the given block length.
	 * The MSB comes first in each block, the most significant blocks comes also first.
	 * 
	 * @param blockLength block length
	 * @return string representation
	 */
	public String toBlockString( int blockLength ) {
		StringBuffer sb = new StringBuffer();
		for ( int i = 0; i < length; i++ ) {
			sb.insert( 0, getAsInteger( i ) );
			if ( i % blockLength == blockLength - 1 && i != length - 1 ) {
				sb.insert( 0, " " );
			}
		}
		if ( length % blockLength != 0 ) {
			for ( int i = length; i < (length / blockLength + 1) * blockLength; i++ ) {
				sb.insert( 0, "0" );
			}
		}
		return sb.toString();
	}
	
	/**
	 * Toggles the bit at the given index.
	 * 
	 * @param index position of the bit in bit vector
	 */
	public void toggle( int index ) {
		if ( get( index ) ) {
			clear( index );
		} else {
			set( index );
		}
	}
	
	/**
	 * Returns a string representation of the bit vector.
	 * This method simply prints all bits from length to 0 sequentially.
	 * MSB comes first.
	 * 
	 * @return string representation
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		for ( int i = length - 1; i >= 0; i-- ) {
			sb.append( getAsInteger( i ) );
		}
		return sb.toString();
	}
	
	/**
	 * Performs a XOR function.
	 * 
	 * @param bv second bit vector
	 * @return result of the XOR function
	 */
	public BitVector xor( BitVector bv ) {
		BitVector rval = new BitVector();
		rval.length = this.length;
		rval.vector = new int[vector.length];
		for ( int i = 0; i < vector.length; i++ ) {
			rval.vector[i] = vector[i] ^ bv.vector[i];
		}
		return rval;
	}
	
	// helper methods
	
	private void calculateNullMask() {
		int unused = 32 - length % 32;
		int mask = -1;
		for ( int i = unused; i < 32; i++ ) {
			mask &= MASK_ZERO[i]; 
		}
		nullMask = mask;
	}
	
}
