/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:35 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: ClassUtils.java,v $
 *
 */
package org.evolvica.util;

import java.lang.reflect.Constructor;
import java.lang.reflect.Modifier;


/**
 * Utility methods for class handling.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:58:35 $
 *
 * @since 0.4.0
 */
public class ClassUtils {
	
	private ClassUtils() {
		// do not instantiate
	}
	
	/**
	 * Returns the caller of the the methods which calls this method.
	 * This is a convinience method for ClassUtil.callingClass( 2 );
	 *
	 * @return caller of the method which calls this method
	 */
	public static final Class callingClass() {
		return callingClass( 3 );
	}

	/**
	 * Returns the class with the given offset from the call stack.
	 * An offset of 0 returns ClassUtils, 1 returns the caller of this method
	 * itself, 2 the caller of the calling method (and so on).
	 *
	 * @param offset offset of the class in the call stack
	 * @return class with the given offset from the call stack or null
	 * if the class cannot be found
	 */
	public static final Class callingClass( int offset ) {
		StackTraceElement caller = new Throwable().getStackTrace()[offset];
		try {
			return Class.forName( caller.getClassName().toString() );
		}
		catch ( ClassNotFoundException cnfx ) {
			return null;
		}
	}
	
	/**
	 * Returns the short classname of a class (without parent packages).
	 * 
	 * @param clazz class
	 * @return short classname
	 */
	public static final String getShortClassname( Class clazz ) {
		if ( clazz != null ) {
			String fullname = clazz.getName();
			return fullname.substring( fullname.lastIndexOf( '.' ) + 1 );
		} else {
			return "null";
		}
	}
	
	/**
	 * Checks if this class can be instantiated automatically.
	 * 
	 * @param clazz class to be checked
	 * @return true if an instance can be created, false instead
	 */
	public static final boolean isInstantiable( Class clazz ) {
		try {
			Constructor constructor = clazz.getConstructor( new Class[0] );
			int mod = constructor.getModifiers();
			if ( Modifier.isPublic( mod ) && ! Modifier.isAbstract( mod ) && ! Modifier.isInterface( mod ) ) {
				return true;
			}
		} catch ( Exception ex ) {
			// be quiet
		}
		return false;
	}
	
}
