/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:36 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: Convert.java,v $
 * 
 */
package org.evolvica.util;

import java.io.File;
import java.lang.reflect.Array;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.net.URL;
import java.net.URLClassLoader;

import org.apache.commons.beanutils.ConversionException;
import org.apache.commons.beanutils.ConvertUtils;
import org.apache.commons.beanutils.Converter;
import org.apache.commons.beanutils.converters.*;

/** Helper for converting objects.
 *
 * @author Andreas Rummler
 * @since 0.5.0
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:58:36 $
 */
public class Convert {
	
	static {
		ConvertUtils.register( new BigDecimalConverter(), BigDecimal.class );
		ConvertUtils.register( new BigIntegerConverter(), BigInteger.class );
		ConvertUtils.register( new BitVectorConverter(), BitVector.class );
		ConvertUtils.register( new BooleanConverter(), Boolean.TYPE );
		ConvertUtils.register( new BooleanConverter(), Boolean.class );
		ConvertUtils.register( new BooleanArrayConverter(), boolean[].class );
		ConvertUtils.register( new ByteConverter(), Byte.TYPE );
		ConvertUtils.register( new ByteConverter(), Byte.class );
		ConvertUtils.register( new ByteArrayConverter(), byte[].class );
		ConvertUtils.register( new CharacterConverter(), Character.TYPE );
		ConvertUtils.register( new CharacterConverter(), Character.class );
		ConvertUtils.register( new CharacterArrayConverter(), char[].class );
		ConvertUtils.register( new ClassConverter(), Class.class );
		ConvertUtils.register( new DirectoryConverter(), Directory.class );
		ConvertUtils.register( new DoubleConverter(), Double.TYPE );
		ConvertUtils.register( new DoubleConverter(), Double.class );
		ConvertUtils.register( new DoubleArrayConverter(), double[].class );
		ConvertUtils.register( new FileConverter(), File.class );
		ConvertUtils.register( new FloatConverter(), Float.TYPE );
		ConvertUtils.register( new FloatConverter(), Float.class );
		ConvertUtils.register( new FloatArrayConverter(), float[].class );
		ConvertUtils.register( new IntegerConverter(), Integer.TYPE );
		ConvertUtils.register( new IntegerConverter(), Integer.class );
		ConvertUtils.register( new IntegerArrayConverter(), int[].class );
		ConvertUtils.register( new LongConverter(), Long.TYPE );
		ConvertUtils.register( new LongConverter(), Long.class );
		ConvertUtils.register( new LongArrayConverter(), long[].class );
		ConvertUtils.register( new ShortConverter(), Short.TYPE );
		ConvertUtils.register( new ShortConverter(), Short.class );
		ConvertUtils.register( new ShortArrayConverter(), short[].class );
		ConvertUtils.register( new StringConverter(), String.class );
		ConvertUtils.register( new StringArrayConverter(), String[].class );
	}
	
	/**
	 * Separator used for array conversion.
	 */
	public static String separator = ",";
	
	/**
	 * Converts an object to a string.
	 * 
	 * @param o object to be converted
	 * @return string representation
	 */
	public static final String toString( Object o ) {
		if ( o != null && o.getClass().isArray() ) {
			int length = Array.getLength( o );
			StringBuffer sb = new StringBuffer();
			sb.append( '{' );
			if ( length > 0 ) {
				sb.append( ConvertUtils.convert( Array.get( o, 0 ) ) );
				for ( int i = 1; i  < length; i++ ) {
					sb.append( separator );
					sb.append( toString( Array.get( o, i ) ) );
				}
			}
			sb.append( '}' );
			return sb.toString();
		} else {
			return ConvertUtils.convert( o );
		}
	}
	
	/**
	 * Converts the object to a "pretty" string representation.
	 * 
	 * @param o object to convert
	 * @return string representation
	 */
	public static final String toDecoratedString( Object o ) {
		if ( o == null ) {
			return null;
		}
		if ( o.getClass().isArray() ) {
			int length = Array.getLength( o );
			StringBuffer sb = new StringBuffer();
			sb.append( '{' );
			if ( length > 0 ) {
				sb.append( toDecoratedString( Array.get( o, 0 ) ) );
				for ( int i = 1; i  < length; i++ ) {
					sb.append( separator );
					sb.append( toDecoratedString( Array.get( o, i ) ) );
				}
			}
			sb.append( '}' );
			return sb.toString();
		} else {
			Converter converter = ConvertUtils.lookup( o.getClass() );
			if ( converter != null ) {
				return ConvertUtils.convert( o );
			} else {
				return o.getClass().getName();
			}
		}
	}
	
	/**
	 * Converts a string to an object of the specified target class.
	 * 
	 * @param s string to be converted
	 * @param target target class
	 * @return converted string
	 */
	public static final Object toObject( String s, Class target ) {
		return ConvertUtils.convert( s, target );
	}
	
	/**
	 * Returns a string representation of the class of the given object.
	 * 
	 * @param o object of which the class should be converted to a string
	 * @return string representation
	 */
	public static final String toClassname( Object o ) {
		if ( o == null ) {
			return "null";
		} else {
			return toClassname( o.getClass() );
		}
	}
	
	/**
	 * Converts the given class into a string representation.
	 * 
	 * @param c class to be converted
	 * @return string representation
	 */
	public static final String toClassname( Class c ) {
		StringBuffer sb = new StringBuffer();
		if ( c == null ) {
			sb.append( "null" );
		} else if ( c.isArray() ) {
			sb.insert( 0, toClassname( c.getComponentType() ) );
			sb.append( "[]" );
		} else {
			sb.insert( 0, c.getName() );
		}
		return sb.toString();
	}
	
	/**
	 * Converts the given string into a class object.
	 * 
	 * @param s string to be converted
	 * @return class object
	 */
	public static final Class toClass( String s ) {
		return toClass( s, null );
	}
	
	/**
	 * Converts a given string into a class by using the given classpath.
	 * 
	 * @param s string
	 * @param classpath classpath
	 * @return loaded class
	 */
	public static final Class toClass( String s, URL[] classpath ) {
		try {
			if ( s.equals( "boolean" ) ) {
				return Boolean.TYPE;
			} else if ( s.equals( "byte" ) ) {
				return Byte.TYPE;
			} else if ( s.equals( "char" ) ) {
				return Character.TYPE;
			} else if ( s.equals( "double" ) ) {
				return Double.TYPE;
			} else if ( s.equals( "float" ) ) {
				return Float.TYPE;
			} else if ( s.equals( "int" ) ) {
				return Integer.TYPE;
			} else if ( s.equals( "long" ) ) {
				return Long.TYPE;
			} else if ( s.equals( "short" ) ) {
				return Short.TYPE;
			} else if ( s.equals( "void" ) ) {
				return Void.TYPE;
			} else if ( s.endsWith( "[]" ) ) {
				String type = s;
				StringBuffer sb = new StringBuffer();
				while( type.endsWith( "[]" ) ) {
					type = type.substring( 0, type.length() - 2 );
					sb.append( "[" );
				}
				if ( type.equals( "byte" ) ) {
					sb.append( "B" );
				} else if ( type.equals( "char" ) ) {
					sb.append( "C" );
				} else if ( type.equals( "double" ) ) {
					sb.append( "D" );
				} else if ( type.equals( "float" ) ) {
					sb.append( "F" );
				} else if ( type.equals( "int" ) ) {
					sb.append( "I" );
				} else if ( type.equals( "long" ) ) {
					sb.append( "J" );
				} else if ( type.equals( "short" ) ) {
					sb.append( "S" );
				} else if ( type.equals( "boolean" ) ) {
					sb.append( "Z" );
				} else if ( type.equals( "void" ) ) {
					sb.append( "V" );
				} else {
					sb.append( "L" + type + ";" );
				}
				return Class.forName( sb.toString() );
			} else {
				if ( classpath == null ) {
					return Class.forName( s );
				} else {
					return URLClassLoader.newInstance( classpath, ClassLoader.getSystemClassLoader() ).loadClass( s );
				}
			}
		} catch ( ClassNotFoundException cnfx ) {
			throw new ConversionException( "unable to convert string to class", cnfx );
		}
	}
	
	/**
	 * Checks if a converter for the given class exists.
	 * 
	 * @param clazz class
	 * @return true if a converter exists, false instead
	 */
	public static final boolean isConvertibleFromString( Class clazz ) {
		Converter converter = ConvertUtils.lookup( clazz );
		if ( converter != null ) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Checks if the given class is a boolean class or type
	 * 
	 * @param clazz class
	 * @return true if the given class is of type Boolean or boolean, false instead
	 */
	public static final boolean isConvertibleToBoolean( Class clazz ) {
		if ( clazz.equals( Boolean.class ) || clazz.equals( Boolean.TYPE ) ) {
			return true;
		} else {
			return false;
		}
	}
	
}
