/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:37 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: MetadataNode.java,v $
 * 
 */
package org.evolvica.util;

import gnu.trove.THashMap;
import gnu.trove.THashSet;

import java.util.Iterator;

import org.apache.commons.lang.StringUtils;

/** A metadata node.
 *
 * @author Andreas Rummler
 * @since 0.6.0
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:58:37 $
 */
public final class MetadataNode {
	
	/**
	 * The name of this node.
	 */ 
	String name = null;
	
	/**
	 * The children of this node.
	 */ 
	THashSet children = new THashSet(); // contains other nodes
	
	/**
	 * The attributes of this node.
	 */ 
	THashMap attributes = new THashMap(); // contains a string -> string (key -> value) mapping
	
	/**
	 * Constructor.
	 * 
	 * @param name name of this node
	 */
	public MetadataNode( String name ) {
		this.name = name;
	}
	
	/**
	 * Returns the name of this node.
	 * 
	 * @return name of this node
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * Returns a child node with the given name.
	 * 
	 * @param name name of the child node
	 * @return child node with the given name or null if there is no such child
	 */
	public MetadataNode getNode( String name ) {
		for ( Iterator it = children.iterator(); it.hasNext(); ) {
			MetadataNode node = (MetadataNode) it.next();
			if ( node.getName().equals( name ) ) {
				return node;
			}
		}
		return null;
	}
	
	/**
	 * Returns the metadata object with the given path.
	 * 
	 * @param path path to the metadata object
	 * @return metadata object with the given path or null if no such object exists
	 */
	public Metadata getValue( String path ) {
		String[] pathArray = StringUtils.split( path, Metadata.SEPARATOR );
		MetadataNode parent = this;
		for ( int i = 0; i < pathArray.length - 1; i++ ) {
			MetadataNode child = parent.getNode( pathArray[i] );
			if ( child == null ) {
				return null;
			} else {
				parent = child;
			}
		}
		if ( parent.attributes.containsKey( pathArray[pathArray.length - 1] ) ) {
			return new Metadata( path, (String) parent.attributes.get( pathArray[pathArray.length - 1] ) );
		} else {
			return null;
		}
	}
	
	/**
	 * Sets a metadata object.
	 * All subnodes are created if they not exist.
	 * 
	 * @param value metadata to be set/added
	 */
	public void setValue( Metadata value ) {
		String[] path = StringUtils.split( value.getPath(), Metadata.SEPARATOR );
		if ( path.length > 1 ) {
			MetadataNode parent = this;
			for ( int i = 0; i < path.length - 1; i++ ) {
				MetadataNode child = parent.getNode( path[i] );
				if ( child == null ) {
					child = new MetadataNode( path[i] );
					parent.children.add( child );
				}
				parent = child;
			}
			parent.attributes.put( path[path.length - 1], value.getValue() );
		}
	}
	
	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return name;
	}
	
}
