/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:38 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: MetadataStore.java,v $
 * 
 */
package org.evolvica.util;

import gnu.trove.THashMap;
import gnu.trove.THashSet;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;

/** A store for metadata.
 *
 * @author Andreas Rummler
 * @since 0.6.0
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:58:38 $
 */
public final class MetadataStore {
	
	private static final String ROOT = "ROOT";
	
	private THashMap store = new THashMap(); // contains a mapping from objects to their metadata nodes
	
	private THashSet listeners = new THashSet();
	
	/**
	 * Adds a listener that want to be notified if data in this store changes.
	 * 
	 * @param listener listener to be added
	 */
	public void addChangeListener( PropertyChangeListener listener ) {
		if ( listener != null ) {
			listeners.add( listener );
		}
	}
	
	/**
	 * Removes a change listener.
	 * 
	 * @param listener change listener to be removed
	 */
	public void removeChangeListener( PropertyChangeListener listener ) {
		if ( listener != null ) {
			listeners.remove( listener );
		}
	}
	
	/**
	 * Returns a metadata object for the given source object and the given path.
	 * 
	 * @param source source object
	 * @param path metadata path
	 * @return metadata object or null if no such object exists
	 */
	public Metadata getValue( Object source, String path ) {
		if ( store.containsKey( source ) ) {
			return ((MetadataNode) store.get( source )).getValue( path );
		}
		return null;
	}
	
	/**
	 * Sets a metadata object.
	 * 
	 * @param source source object
	 * @param value metadata object to be set/added
	 */
	public void setValue( Object source, Metadata value ) {
		Metadata oldValue = getValue( source, value.getPath() );
		MetadataNode root = null;
		if ( ! store.containsKey( source ) ) {
			root = new MetadataNode( ROOT );
			store.put( source, root );
		} else {
			root = (MetadataNode) store.get( source );
		}
		root.setValue( value );
		// notify listeners
		PropertyChangeEvent event = new PropertyChangeEvent( source, Metadata.METADATA_CHANGE, oldValue, value );
		for ( Iterator it = listeners.iterator(); it.hasNext(); ) {
			((PropertyChangeListener) it.next()).propertyChange( event );
		}
	}
	
	/**
	 * Restores the metadata tree from the given content provider.
	 * 
	 * @param provider metadata content provider
	 */
	public void restore( IMetadataProvider provider ) {
		Object[] sources = provider.getSources();
		Metadata[] values = provider.getValues();
		for ( int i = 0; i < sources.length || i < values.length; i++ ) {
			if ( sources[i] != null && values[i] != null ) {
				setValue( sources[i], values[i] );
			}
		}
	}
	
	/**
	 * Serializes the metadata tree by sending the tree content to the given serializer.
	 * 
	 * @param serializer metadata serializer
	 */
	public void serialize( IMetadataSerializer serializer ) {
		for ( Iterator it = this.store.entrySet().iterator(); it.hasNext(); ) {
			// get root node
			Map.Entry entry = (Map.Entry) it.next();
			Object source = entry.getKey();
			MetadataNode root = (MetadataNode) entry.getValue();
			// setup queues
			LinkedList nodeQueue = new LinkedList();
			LinkedList pathQueue = new LinkedList();
			for ( Iterator it2 = root.children.iterator(); it2.hasNext(); ) {
				MetadataNode child = (MetadataNode) it2.next();
				nodeQueue.add( child );
				pathQueue.add( child.name );
			}
			while ( ! nodeQueue.isEmpty() ) {
				MetadataNode node = (MetadataNode) nodeQueue.getFirst();
				String path = (String) pathQueue.getFirst();
				for ( Iterator it2 = node.attributes.entrySet().iterator(); it2.hasNext(); ) {
					Map.Entry attribute = (Map.Entry) it2.next();
					String name = (String) attribute.getKey();
					String value = (String) attribute.getValue();
					serializer.serialize( source, new Metadata( path + Metadata.SEPARATOR + name, value ) );
				}
				for ( Iterator it2 = node.children.iterator(); it2.hasNext(); ) {
					MetadataNode child = (MetadataNode) it2.next();
					nodeQueue.add( child );
					pathQueue.add( path + Metadata.SEPARATOR + child.name );
				}
				nodeQueue.removeFirst();
				pathQueue.removeFirst();
			}
		}
	}
	
}
