/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:38 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: RandomUtils.java,v $
 *
 */
package org.evolvica.util;

import java.lang.reflect.Array;
import java.util.*;

/** Utility methods for working with random objects.
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:58:38 $
 */
public class RandomUtils {
	
	/** Random generator of Java Core Libraries.
	* The generator produces uniformly distributed random numbers and is the base for all implemented methods.
	*/
	static Random rd = new Random();
	
	/** Date, for setting the current random seed.
	*/
	static Date currentDate;
	
	/** Update of random seed.
	* The seed is overwritten with the current system time.
	*/
	static public void updateSeed() {
		currentDate = new Date();
		rd.setSeed( currentDate.getTime() );
	}
	
	/** Setting of the random seeds.
	* The seed can be set to a specific value using this method to provide the possibility to generate uniform
	* sequences of random numbers.
	* 
	* @param l new random seed
	*/
	static public void setSeed( long l ) {
		rd.setSeed( l );
	}
	
	/** Random double value.
	* The return value lies between 0.0 and the given bound.
	* 
	* @param bound bound
	* @return random double
	*/
	static public double randomDouble( double bound ) {
		return rd.nextDouble() * bound;
	}
	
	/** Random double value.
	* The return value lies between lowerBound and upperBound. The method will return 0.0 silently if lowerBound >= upperBound.
	* 
	* @param lowerBound lower bound
	* @param upperBound upper bound
	* @return random double
	*/
	static public double randomDouble( double lowerBound, double upperBound ) {
		if ( lowerBound >= upperBound ) {
			return 0.0;
		} else {
			double diff = upperBound - lowerBound;
			return randomDouble( diff ) + lowerBound;
		}
	}
	
	/** Random float value.
	* The return value lies between 0.0 and the given bound.
	* 
	* @param bound bound
	* @return random float
	*/
	static public float randomFloat( float bound ) {
		return rd.nextFloat() * bound;
	}
	
	/** Random float value.
	* The return value lies between lowerBound and upperBound. The method will return 0.0 silently if lowerBound >= upperBound.
	* 
	* @param lowerBound lower bound
	* @param upperBound upper bound
	* @return random float
	*/
	static public float randomFloat( float lowerBound, float upperBound ) {
		if ( lowerBound >= upperBound ) {
			return (float) 0.0;
		} else {
			float diff = upperBound - lowerBound;
			return randomFloat( diff ) + lowerBound;
		}
	}
	
	/** Random integer value.
	* The return value lies between 0 (inclusive) and the given bound (exclusive). This is also valid for negative bounds.
	* 
	* @param bound bound
	* @return random integer
	*/
	static public int randomInt( int bound ) {
		if ( bound >= 0 ) {
			return rd.nextInt( bound );
		} else {
			return -rd.nextInt( -bound );
		}
	}
	
	/** Random integer value.
	* The return value lies between lowerBound (inclusive) and upperBound (exclusive).
	* The method will return 0 silently if lowerBound >= upperBound.
	* 
	* @param lowerBound lower bound
	* @param upperBound upper bound
	* @return random integer
	*/
	static public int randomInt( int lowerBound, int upperBound ) {
		if ( lowerBound >= upperBound ) {
			return 0;
		} else {
			int diff = upperBound - lowerBound;
			return randomInt( diff ) + lowerBound;
		}
	}
	
	/** Random long value.
	* The return value lies between 0 and the given bound.
	* 
	* @param bound bound
	* @return random long
	*/
	static public long randomLong( long bound ) {
		long l, val;
		l = rd.nextLong();
		l = l << 1;
		l = l >> 1;
		return Math.abs( l % bound );
	}
	
	/** Random long value.
	* The return value lies between lowerBound and upperBound. The method will return 0 silently if lowerBound >= upperBound.
	* 
	* @param lowerBound lower bound
	* @param upperBound upper bound
	* @return random long
	*/
	static public long randomLong( long lowerBound, long upperBound ) {
		if ( lowerBound >= upperBound ) {
			return 0;
		} else {
			long diff = upperBound - lowerBound;
			return randomLong( diff ) + lowerBound;
		}
	}
	
	// methods for generating random strings
	
	/** Produces a random string with a random length.
	* The can have a maximal length of 127 characters.
	* 
	* 
	* @return random string
	* @see #randomString( byte )
	*/
	static public String randomString() {
		return randomString( (byte) rd.nextInt( 128 ) );
	}
	
	// Anmerkung : die Methode umfat im Moment den ASCII Bereich 32..126
	//             spter kommt aber nochmal ein druckbarer Bereich 161..255
	// -> den knnte man spter nochmal implementieren
	/** Produces a random string.
	* The string can have a maximal length of 127 characters. In case that the given parameter is negative
	* a random string of length 1 is generated. The produced characters have ASCII values between #32 and
	* #126. The range from #161 to #255 (which is also printable) is not supported (yet).
	* 
	* @param b length of random string
	* @return random string
	*/
	static public String randomString( byte b ) {
		StringBuffer sb = new StringBuffer();
		char randChar;
		if ( b < 1 ) {
			b = 1;
		}
		for ( byte i = 0; i < b; i++ ) {
			randChar = (char) (rd.nextInt( 95 ) + 32);
			sb.append( randChar );
		}
		return sb.toString();
	}
	
	/** Produces a random string.
	* The is implemented for convenience reasons.
	* 
	* @param i length of string
	* @return random string
	* @see #randomString( byte )
	*/
	static public String randomString( int i ) {
		return randomString( (byte) i );
	}
	
	// methods for generating random words
	
	/** Produces a random "word".
	* Same functionality like randomString(), but produces a string containing only lower case characters.
	* 
	* @return random string
	*/
	static public String randomWord() {
		return randomWord( (byte) rd.nextInt( 128 ) );
	}
	
	/** Produces a random "word".
	* Same functionality like randomString( byte ), but produces a string containing only lower case characters.
	* 
	* @param b length of string
	* @return random string
	*/
	static public String randomWord( byte b ) {
		StringBuffer sb = new StringBuffer();
		int randInt;
		if ( b < 1 ) {
			b = 1;
		}
		for ( byte i = 0; i < b; i++ ) {
			randInt = rd.nextInt( 58 ) + 65;
			while ( randInt > 90 && randInt < 97  ) {
				randInt = rd.nextInt( 58 ) + 65;
			}
			sb.append( (char) randInt );
		}
		return sb.toString();
	}
	
	/** Produces a random "word".
	* Same functionality like randomString( int ), but produces a string containing only lower case characters.
	* 
	* @param i length of string
	* @return random string
	*/
	static public String randomWord( int i ) {
		return randomWord( (byte) i );
	}
	
	/** Choice of n random numbers from N possible.
	* The original code was written by Laurentiu Cristofor (laur72_98@yahoo.com). This method selects numbers
	* between 1 (inclusive) and samplePool (inclusive).
	* 
	* @exception IllegalArgumentException thrown, if N < 0 or N < n
	* @param sampleNumber number of selections
	* @param samplePool upper bound
	* @return array with selected numbers
	*/
	static public int[] randomIntegerSample( int sampleNumber, int samplePool ) throws IllegalArgumentException {
		return randomIntegerSample( sampleNumber, samplePool, false );
	}
	
	/** Choice of n random numbers from N possible.
	* The original code was written by Laurentiu Cristofor (laur72_98@yahoo.com). This method selects numbers
	* depending on zeroInc. In case of zeroInc = true, the range of values is from 0 (inclusive)  to samplePool - 1
	* (inclusive), else from 1 (inclusive) to samplePool (inclusive).
	* 
	* @exception IllegalArgumentException thrown, if N < 0, or N < n
	* @param sampleNumber number of selections
	* @param samplePool upper bound
	* @param zeroInc selection of range of values
	* @return array with selected numbers
	*/
	static public int[] randomIntegerSample( int sampleNumber, int samplePool, boolean zeroInc ) throws IllegalArgumentException {
		int n = sampleNumber;
		int N = samplePool;
		// setup check
		if ( N < 1 || n < 1 || N < n ) {
			throw new IllegalArgumentException( "You must provide n and N s.t. 0 < n <= N" );
		}
		int[] sample = new int[n];
		int i = 0;
		// S1. [Initialize]
		int t = 0; // how many we have seen
		int m = 0; // how many we have selected
		while ( true ) {
			// S2. [Generate U]
			double U = rd.nextDouble();
			// S3. [Test]
			if ( ( N - t ) * U >= n - m ) {
				// S5. [Skip]
				t = t + 1;
			} else {
				// S4. [Select]
				if ( zeroInc == true ) {
					sample[i++] = t; // select from 0 to N-1
				} else {
					sample[i++] = t + 1; // select from 1 to N
				}
				m = m + 1;
				t = t + 1;

				if ( m < n ) {
					; // do nothing
				} else {// sample complete !
					return sample;
				}
			}
		}
	}
	
	/** Swaps two random elements of a list.
	* 
	* @param list list with n>1 elements
	* @deprecated replaced by {@link #scramble( List ) scramble}
	*/
	static public void scrambleList( List list ) {
		try {
			int[] pos = randomIntegerSample( 2, list.size(), true );
			Object tmp = list.get( pos[0] );
			list.set( pos[0], list.get( pos[1] ) );
			list.set( pos[1], tmp );
		} catch ( IllegalArgumentException e ) {
			// do nothing ...
		}
	}
	
	/** Swap two random elements of a list n times.
	* 
	* @param list list with n > 1 elements
	* @param sCount number of swaps
	* @deprecated replaced by {@link #scramble( List, int ) scramble}
	*/
	static public void scrambleList( List list, int sCount ) {
		for ( int i = 0; i < sCount; i++ ) {
			scrambleList( list );
		}
	}
	
	/** Swaps two random elements of an object array.
	* 
	* @param array array with n>1 elements
	* @deprecated replaced by {@link #scramble( Object ) scramble}
	*/
	public static void scrambleArray( Object[] array ) {
		try {
			int[] pos = randomIntegerSample( 2, array.length, true );
			Object tmp = array[pos[0]];
			array[pos[0]] = array[pos[1]];
			array[pos[1]] = tmp;
		} catch ( IllegalArgumentException e ) {
			// do nothing ...
		}
	}
	
	/** Swap two random elements of an object array n times.
	* 
	* @param array array with n > 1 elements
	* @param sCount number of swaps
	* @deprecated replaced by {@link #scramble( Object, int ) scramble}
	*/
	static public void scrambleArray( Object[] array, int sCount ) {
		for ( int i = 0; i < sCount; i++ ) {
			scrambleArray( array );
		}
	}
	
	/** Generates a random sequence of integers from 0 to the given integer (exclusive).
	* Each number occurs only one time.
	* 
	* @param size length of the integer sequence
	* @return sequence of integers
	*/
	static public int[] randomIntSequence( int size ) {
		Vector v = new Vector( size );
		int index;
		int[] sequence = new int[size];
		for ( int i = 0; i < size; i++ ) {
			v.add( new Integer( i ) );
		}
		for ( int i = size, j = 0; i > 0; i--, j++ ) {
			index = rd.nextInt( i );
			sequence[j] = ((Integer) v.get( index )).intValue();
			v.removeElementAt( index );
		}
		return sequence;
	}
	
	/** Scrambles a list.
	* 
	* @param list list with n > 1 elements
	*/
	static public void scramble( List list ) {
		scramble( list, list.size() );
	}
	
	/** Swaps two random elements of a list n times.
	* 
	* @param list list with n > 1 elements
	* @param sCount number of swaps
	*/
	static public void scramble( List list, int sCount ) {
		for ( int i = 0; i < sCount; i++ ) {
			swap( list );
		}
	}
	
	/** Scrambles an array.
	* 
	* @param array array with n > 1 elements
	*/
	static public void scramble( Object array ) {
		if ( array.getClass().isArray() ) {
			scramble( array, Array.getLength( array ) );
		}
	}
	
	/** Swaps two random elements of an array n times.
	* 
	* @param array array with n > 1 elements
	* @param sCount number of swaps
	*/
	static public void scramble( Object array, int sCount ) {
		for ( int i = 0; i < sCount; i++ ) {
			swap( array );
		}
	}
	
	/** Swaps two random elements of a list.
	* 
	* @param list list with n > 1 elements
	*/
	static public void swap( List list ) {
		try {
			int[] pos = randomIntegerSample( 2, list.size(), true );
			Object tmp = list.get( pos[0] );
			list.set( pos[0], list.get( pos[1] ) );
			list.set( pos[1], tmp );
		} catch ( IllegalArgumentException e ) {
			// do nothing ...
		}
	}
	
	/** Swaps two random elements of an array.
	* 
	* @param array array with n > 1 objects
	*/
	static public void swap( Object array ) {
		if ( array.getClass().isArray() ) {
			int[] pos = randomIntegerSample( 2, Array.getLength( array ), true );
			Object tmp = Array.get( array, pos[0] );
			Array.set( array, pos[0], Array.get( array, pos[1] ) );
			Array.set( array, pos[1], tmp );
		}
	}
	
}
