/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:38 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: SerialUtils.java,v $
 * 
 */
package org.evolvica.util;

import gnu.trove.THashSet;

import java.beans.PropertyDescriptor;
import java.lang.reflect.Array;
import java.util.Iterator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang.StringUtils;
import org.w3c.dom.*;

/** Utility methods for serialization.
 *
 * @author Andreas Rummler
 * @since 0.5.1
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:58:38 $
 */
public class SerialUtils {
	
	private static final String TAG = "object";
	
	private static DocumentBuilder builder = null;
	
	private static Document defaultDocument = null;
	
	private static ClassLoader classloader = null;
	
	static {
		try {
			defaultDocument = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
		} catch ( ParserConfigurationException e ) {
			// be quiet
		}
	}
	
	/**
	 * Serializes a JavaBean in an XML element.
	 * 
	 * @param object bean to be serialized
	 * @return XML element containing the bean information
	 * @throws SerializationException thrown in case the serialization fails
	 */
	public static Element serialize( Object object ) throws SerializationException {
		Element root = defaultDocument.createElement( "object" );
		serialize( root, object );
		return root;
	}
	
	/**
	 * Serializes a JavaBean in an XML element.
	 * 
	 * @param parent XML element to be used as the parent/root element
	 * @param object bean to be serialized
	 * @throws SerializationException thrown in case the serialization fails
	 */
	public static void serialize( Element parent, Object object ) throws SerializationException {
		try {
			if ( StringUtils.isEmpty( parent.getAttribute( "class" ) ) ) {
				parent.setAttribute( "class", Convert.toClassname( object ) );
			}
			serializeClass( parent, object );
		} catch ( Exception ex ) {
			throw new SerializationException( ex.getMessage(), ex );
		}
	}
	
	private static void serializeArray( Element parent, Object array ) throws Exception {
		int length = Array.getLength( array );
		for ( int i = 0; i < length; i++ ) {
			Object value = Array.get( array, i );
			if ( value != null ) {
				Class type = value.getClass();
				Element element = parent.getOwnerDocument().createElement( TAG );
				element.setAttribute( "index", String.valueOf( i ) );
				if ( type.isArray() ) {
					//element.setAttribute( "convertible", "array" );
					element.setAttribute( "value", Convert.toClassname( value ) );
					element.setAttribute( "length", String.valueOf( Array.getLength( value ) ) );
					parent.appendChild( element );
					serializeArray( element, value );
				} else if ( Convert.isConvertibleFromString( type ) ) {
					element.setAttribute( "value", Convert.toString( value ) );
					//element.setAttribute( "convertible", "string" );
					parent.appendChild( element );
				} else if ( ClassUtils.isInstantiable( type ) ) {
					//element.setAttribute( "convertible", "class" );
					element.setAttribute( "value", Convert.toClassname( value ) );
					parent.appendChild( element );
					serializeClass( element, value );
				}
			}
		}
	}
	
	private static void serializeClass( Element parent, Object object ) throws Exception {
		PropertyDescriptor[] pd = PropertyUtils.getPropertyDescriptors( object );
		for ( int i = 0; i < pd.length; i++ ) {
			if ( pd[i].getReadMethod() != null && pd[i].getWriteMethod() != null ) { // ignore read-/write-only properties
				Object value = PropertyUtils.getProperty( object, pd[i].getName() );
				if ( value != null ) { // ignore null values
					Class type = value.getClass();
					Element element = parent.getOwnerDocument().createElement( TAG );
					element.setAttribute( "name", pd[i].getName() );
					if ( type.isArray() ) {
						//element.setAttribute( "convertible", "array" );
						element.setAttribute( "value", Convert.toClassname( value ) );
						element.setAttribute( "length", String.valueOf( Array.getLength( value ) ) );
						parent.appendChild( element );
						serializeArray( element, value );
					} else if ( Convert.isConvertibleFromString( type ) ) {
						element.setAttribute( "value", Convert.toString( value ) );
						//element.setAttribute( "convertible", "string" );
						parent.appendChild( element );
					} else if ( ClassUtils.isInstantiable( type ) ) {
						//element.setAttribute( "convertible", "class" );
						element.setAttribute( "value", Convert.toClassname( value ) );
						parent.appendChild( element );
						serializeClass( element, value );
					}
				}
			}
		}
	}
	
	/**
	 * Deserializes an XML element into a Java object.
	 * 
	 * @param element XML element to be deserialized
	 * @return deserialized Java object
	 * @throws SerializationException thrown in case the deserialization fails
	 */
	public static Object deserialize( Element element ) throws SerializationException {
		return deserialize( element, ClassLoader.getSystemClassLoader() );
	}
	
	/**
	 * Deserializes an XML element into a Java object.
	 * 
	 * @param element XML element to be deserialized
	 * @param classloader class loader to used instead of the system class loader
	 * @return deserialized Java object
	 * @throws SerializationException thrown in case the deserialization fails
	 */
	public static Object deserialize( Element element, ClassLoader classloader ) throws SerializationException {
		try {
			SerialUtils.classloader = classloader;
			String clazz = element.getAttribute( "class" );
			Object root = classloader.loadClass( clazz ).newInstance();
			THashSet elements = new THashSet();
			THashSet unsettable = new THashSet();
			NodeList children = element.getChildNodes();
			for ( int i = 0; i < children.getLength(); i++ ) {
				Node next = children.item( i );
				if ( next.getNodeType() == Node.ELEMENT_NODE && next.getNodeName().equals( TAG ) ) {
					unsettable.add( next );
				}
			}
			do {
				elements.addAll( unsettable );
				unsettable.clear();
				for ( Iterator it = elements.iterator(); it.hasNext(); ) {
					Element e = (Element) it.next();
					try {
						deserializeClass( e, root );
					} catch ( Exception ex ) {
						unsettable.add( e );
					}
				}
			} while ( unsettable.size() > 0 && unsettable.size() < elements.size() );
			if ( unsettable.size() > 0  ) {
				EVLog.warn( "unable to fully restore parameter(s) of object of class " + clazz );
			}
			return root;
		} catch ( Exception ex ) {
			throw new SerializationException( ex.getMessage(), ex );
		}
	}
	
	private static void deserializeClass( Element element, Object parent ) throws Exception {
		String name = element.getAttribute( "name" );
		String value = element.getAttribute( "value" );
		PropertyDescriptor descriptor = PropertyUtils.getPropertyDescriptor( parent, name );
		if ( descriptor.getPropertyType().isArray() ) {
			int length = Integer.parseInt( element.getAttribute( "length" ) );
			Object newArray = Array.newInstance( Convert.toClass( value ).getComponentType(), length );
			PropertyUtils.setProperty( parent, name, newArray );
			// analyze children
			NodeList children = element.getChildNodes();
			for ( int i = 0; i < children.getLength(); i++ ) {
				Node next = children.item( i );
				if ( next.getNodeType() == Node.ELEMENT_NODE && next.getNodeName().equals( TAG ) ) {
					deserializeArrayElement( (Element) next, newArray );
				}
			}
		} else if ( Convert.isConvertibleFromString( descriptor.getPropertyType() ) ) {
			PropertyUtils.setProperty( parent, name, Convert.toObject( value, descriptor.getPropertyType() ) );
		} else {
			PropertyUtils.setProperty( parent, name, Convert.toClass( value ).newInstance() );
			// analyze children
			NodeList children = element.getChildNodes();
			for ( int i = 0; i < children.getLength(); i++ ) {
				Node next = children.item( i );
				if ( next.getNodeType() == Node.ELEMENT_NODE && next.getNodeName().equals( TAG ) ) {
					deserializeClass( (Element) next, PropertyUtils.getProperty( parent, name ) );
				}
			}
		}
	}
	
	private static void deserializeArrayElement( Element element, Object array ) throws Exception {
		String name = element.getAttribute( "name" );
		String value = element.getAttribute( "value" );
		int index = Integer.parseInt( element.getAttribute( "index" ) );
		Class componentType = array.getClass().getComponentType();
		if ( componentType.isArray() ) {
			int length = Integer.parseInt( element.getAttribute( "length" ) );
			Object newArray = Array.newInstance( componentType.getComponentType(), length );
			Array.set( array, index, newArray );
			// analyze children
			NodeList children = element.getChildNodes();
			for ( int i = 0; i < children.getLength(); i++ ) {
				Node next = children.item( i );
				if ( next.getNodeType() == Node.ELEMENT_NODE && next.getNodeName().equals( TAG ) ) {
					deserializeArrayElement( (Element) next, newArray );
				}
			}
		} else if ( Convert.isConvertibleFromString( componentType ) ) {
			Array.set( array, index, Convert.toObject( value, componentType ) );
		} else {
			Object newObject = Convert.toClass(value).newInstance();
			Array.set( array, index, newObject );
			// analyze children
			NodeList children = element.getChildNodes();
			for ( int i = 0; i < children.getLength(); i++ ) {
				Node next = children.item( i );
				if ( next.getNodeType() == Node.ELEMENT_NODE && next.getNodeName().equals( TAG ) ) {
					deserializeClass( (Element) next, newObject );
				}
			}
		}
	}
	
}
